

/*
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2011 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */


/*******************************************************************************
* $Descr: Default Settings (overloadable from custom.h) $
*******************************************************************************/

/* This file provides default values for customizable macros. If you need to
 * override their value, define these macros in custom.h. */
/* Multiple inclusions guard: */
#ifndef DEFAULTS_H_INCLUDED
#define DEFAULTS_H_INCLUDED


/*******************************************************************************
* Language features
*******************************************************************************/

#ifndef RTRT_CPLUSPLUS_LANGUAGE
#ifdef __cplusplus
/*! C++ language */
#define RTRT_CPLUSPLUS_LANGUAGE 1
#else
#define RTRT_CPLUSPLUS_LANGUAGE 0
#endif
#endif

#ifndef RTRT_CONCAT_MACRO
/*! Concatenates two macros into one */
#define RTRT_CONCAT_MACRO(a,b) a##b
#endif

/* some compiler does not accept to use multiple time the same macro into a macro expansion
 * in that case define the following macros independently into the TDP to a##b */
#ifndef RTRT_CONCAT_MACRO2
#define RTRT_CONCAT_MACRO2(a,b) RTRT_CONCAT_MACRO(a,b)
#define RTRT_CONCAT_MACRO1(a,b) RTRT_CONCAT_MACRO(a,b)
#endif

#ifndef ATL_STR
/*! make it as string */
#define ATL_STR(s) #s
#endif

#ifndef RTRT_GLOBALVAR
/*! Prefix to global variables */
#define RTRT_GLOBALVAR
#endif

#ifndef RTRT_GLOBALFCT
/*! Prefix to global functions */
#define RTRT_GLOBALFCT
#endif

#ifndef ATL_CALLCONV
/*! calling convention Prefix to global functions */
#define ATL_CALLCONV
#endif /* ATL_CALLCONV */

/*! allows to set the section for instrumentation added variables (GNU format as default) */
/*! can be set file by file with -D ATL_SECTION=ATL_STR\(sectionName\) on compilation cmd line */
#if defined( ATL_SECTION ) && !defined(ATL_SET_SECTION)
#define ATL_SET_SECTION __attribute__ ((section ( ATL_SECTION )))
#else
#define ATL_SET_SECTION
#endif

#ifndef RTRT_STATICVAR
/*! Prefix to static variables */
#define RTRT_STATICVAR static
#endif

#ifndef RTRT_STATICFCT
/*! Prefix to static functions */
#define RTRT_STATICFCT static
#endif

#ifndef RTRT_KR
/* Set to 1 if you need K&R-compliant code */
#define RTRT_KR 0
#endif

#ifndef RTRT_RAM_SET_RANDOMLY
/*! set to 1 means that global variables are not set to 0 at start */
#define RTRT_RAM_SET_RANDOMLY 0
#endif

#ifndef RTRT_VARIABLE_INIT_SUPPORTED
/*! Global initialization of static variables is allowed and implemented by
 *  the compiler. If not, you must ensure that memory is zeroed before
 *  execution (or, with no observation tool, you may restrict this condition
 *  to the variable `_atl_initialized'). */
#define RTRT_VARIABLE_INIT_SUPPORTED 1
#endif

#ifndef RTRT_VARIABLE_INITIALIZED
/*! if Global initialization of static variables is NOT allowed,
 *  this magic number will be used to check if observation vars have been set. */
#define RTRT_VARIABLE_INITIALIZED 0x5A
#endif

#ifndef RTRT_IS_NOT_INITIALIZED
/*! if Global initialization of static variables is NOT allowed, and all memory
 *  cannot be set to zeroed, set this macro to v!=RTRT_VARIABLE_INITIALIZED in the TDP. */
#if RTRT_RAM_SET_RANDOMLY
#define RTRT_IS_NOT_INITIALIZED(v) v!=RTRT_VARIABLE_INITIALIZED
#else /* RTRT_RAM_SET_RANDOMLY */
#define RTRT_IS_NOT_INITIALIZED(v) v==0
#endif /* RTRT_RAM_SET_RANDOMLY */
#endif /* RTRT_IS_NOT_INITIALIZED */

#if RTRT_CPLUSPLUS_LANGUAGE

#ifndef RTRT_STATIC_IN_INLINE_ALLOWED
/*! Use of static variables in inline routines is allowed */
#define RTRT_STATIC_IN_INLINE_ALLOWED 1
#endif

#ifndef RTRT_STATIC_IN_STATIC_INLINE_ALLOWED
/*! Use of static variables in static inline methods is allowed */
#define RTRT_STATIC_IN_STATIC_INLINE_ALLOWED 1
#endif

#ifndef RTRT_DEFAULT_DTOR_VIRTUAL_KEYWORD
/*! Qualifier for explicited default destructors */
#define RTRT_DEFAULT_DTOR_VIRTUAL_KEYWORD virtual
#endif

#ifndef RTRT_BOOL_DEFINED
/*! 'bool' type is pre-defined by the compiler */
#define RTRT_BOOL_DEFINED 1
#endif

#ifndef RTRT_EXCEPTIONS
/*! Exceptions support */
#define RTRT_EXCEPTIONS 1
#endif

#ifndef RTRT_GENERATE_EXCEPTIONS
/*! Exceptions-based instrumentation support */
#define RTRT_GENERATE_EXCEPTIONS RTRT_EXCEPTIONS
#endif

#ifndef RTRT_RTTI_SUPPORTED
/*! RTTI (run-time type information) support */
#define RTRT_RTTI_SUPPORTED 0
#endif

#endif /* RTRT_CPLUSPLUS_LANGUAGE */

#ifndef RTRT_EXTERN
/*! Linkage directive for a function with a C compiler */
#define RTRT_EXTERN extern
#endif

#ifndef RTRT_EXTERN_C
/*! Linkage directive for a C-function with a C++ compiler */
#define RTRT_EXTERN_C extern "C"
#endif

#ifndef _ATL_EXTERN_LIBC
#if RTRT_CPLUSPLUS_LANGUAGE
/*! Linkage directive for a C-function */
#define _ATL_EXTERN_LIBC RTRT_EXTERN_C
#else
#define _ATL_EXTERN_LIBC RTRT_EXTERN
#endif
#endif

#ifndef _ATL_EXTERN_C
#if RTRT_CPLUSPLUS_LANGUAGE
/*! Linkage directive for a C-function */
#define _ATL_EXTERN_C RTRT_EXTERN_C
#else
#define _ATL_EXTERN_C RTRT_EXTERN
#endif
#endif

#ifndef RTRT_PREPRO_ERROR_SUPPORTED
/*! C/C++ preprocessor supports #error and #warning directives */
#define RTRT_PREPRO_ERROR_SUPPORTED 1
#endif

#ifndef RTRT_MAIN_HEADER
/*! Main function header */
#define RTRT_MAIN_HEADER int main (int argc, char *argv[]) {
#endif

#ifndef RTRT_MAIN_RETURN
/*! Main function return instruction */
#define RTRT_MAIN_RETURN return 0;
#endif

#ifndef RTRT_STATIC_AND_EXTERN_C_FORBIDDEN
/*! Set to 1 if declaring a static function within an extern "C" block is
 *  not allowed by the compiler. */
#define RTRT_STATIC_AND_EXTERN_C_FORBIDDEN 1
#endif

/*******************************************************************************
* Controlled-size types
*******************************************************************************/

#ifndef RTRT_INT8
/*! 8 bits integer */
#define RTRT_INT8 char
#endif

#ifndef RTRT_U_INT8
/*! 8 bits unsigned integer */
#define RTRT_U_INT8 unsigned char
#endif

#ifndef RTRT_INT16
/*! 16 bits integer */
#define RTRT_INT16 short
#endif

#ifndef RTRT_U_INT16
/*! 16 bits unsigned integer */
#define RTRT_U_INT16 unsigned short
#endif

#ifndef RTRT_INT32
/*! 32 bits integer */
#define RTRT_INT32 long
#endif

#ifndef RTRT_U_INT32
/*! 32 bits unsigned integer */
#define RTRT_U_INT32 unsigned long
#endif

#ifndef RTRT_INT64
/*! 64 bits signed integer */
/* default: undefined */
#endif

#ifndef RTRT_U_INT64
/*! 64 bits signed integer */
/* default: undefined */
#endif

#ifndef RTRT_SIZE_T
/*! Parameter type for malloc routine. Must be an integer base type, compatible
 *  in size and sign with size_t. */
#define RTRT_SIZE_T unsigned int
#endif

#ifndef RTRT_PTR_TO_INT
/*! How to cast a pointer to a integral type of same size */
#define RTRT_PTR_TO_INT(p) ((RTRT_U_INT32)p)
#endif

#ifndef RTRT_64_BITS_PTR
#define RTRT_64_BITS_PTR 0
#endif


#ifndef RTRT_INT_SIZE_PTR
/*! type to move a pointer to a integral type of same size */
#if RTRT_64_BITS_PTR
#define RTRT_INT_SIZE_PTR RTRT_U_INT64
#else
#define RTRT_INT_SIZE_PTR RTRT_U_INT32
#endif
#endif

#ifndef RTRT_LARGEST_STRING
#define RTRT_LARGEST_STRING 16384
#endif

#ifndef RTRT_THIDX_T
#define RTRT_THIDX_T int
#endif /* RTRT_THIDX_T */
/*******************************************************************************
* Computed settings
*******************************************************************************/
/* Macros in this section are for internal use only: do NOT define them in
 * custom.h. */

#if USE_ATK || USE_ATC || USE_ATT || USE_ATP || USE_ATQ
/*! At least one observation is used */
#define _ATL_OBSERVATION_TOOL 1
#else
#define _ATL_OBSERVATION_TOOL 0
#endif

#if USE_ATU || USE_ATV || USE_ATS || USE_ATO
/*! A testing tool is used. */
#define _ATL_TESTING_TOOL 1
#else
#define _ATL_TESTING_TOOL 0
#endif

#if RTRT_CPLUSPLUS_LANGUAGE
/*! Linkage directive for an exported C-function definition */
#define _ATL_C_GLOBALFCT RTRT_EXTERN_C
#else
#define _ATL_C_GLOBALFCT RTRT_GLOBALFCT
#endif

#if defined(_WIN32) && (defined(_MSC_VER) || defined(__BORLANDC__) || defined (__CYGWIN__) || defined(__MSVCRT__) )
/* Are the Win32 functions available? POSIX ones will be used otherwise. */
#define _ATL_WIN32_API 1
#else
#define _ATL_WIN32_API 0
#endif

/*******************************************************************************
* initexit default settings
*******************************************************************************/

#ifndef RTRT_EXIT
/*! This defines which kind of exit routine is to be used. Avalaible values are:
 *  RTRT_STD (standard exit function)
 *  RTRT_USR (user-defined exit function)
 *  RTRT_NONE (no exit function available) */
#define RTRT_EXIT RTRT_NONE
#endif

#ifndef RTRT_ATEXIT
/*! This defines which kind of atexit routine is to be used. Available values
 *  are:
 *  RTRT_STD
 *  RTRT_USR
 *  RTRT_NONE */
#define RTRT_ATEXIT RTRT_NONE
#endif

#ifndef RTRT_UNCONTROLED_EXIT
#if _ATL_TESTING_TOOL
/*! 1 if an exit may occur outside instrumented or testing sources */
#define RTRT_UNCONTROLED_EXIT 0
#else
#define RTRT_UNCONTROLED_EXIT 1
#endif
#endif

/*******************************************************************************
* priv default settings
*******************************************************************************/

#ifndef RTRT_IO
/*! This defines which kind of I/O routines are to be used. Available values
 *  are:
 *  RTRT_STD
 *  RTRT_USR
 *  RTRT_NONE */
#define RTRT_IO RTRT_NONE
#endif

#if RTRT_IO == RTRT_USR
#ifndef RTRT_FILE_TYPE
#define RTRT_FILE_TYPE int
#endif
#elif  RTRT_IO == RTRT_SOCKET
#ifndef RTRT_FILE_TYPE
#define RTRT_FILE_TYPE int
#endif
#ifndef RTRT_LINE_SPLIT_MARKER_BEGIN
#define RTRT_LINE_SPLIT_MARKER_BEGIN "^"
#define RTRT_LINE_SPLIT_MARKER_END   "^"
#endif
#define RTRT_OPEN_LATE
#define RTRT_FILENAME_NOT_USED
#elif  RTRT_IO == RTRT_NONE
#define RTRT_NEED_CLOSE_CALL 1
#endif /* RTRT_IO == RTRT_NONE */

/*! Character of trace markers. This character is generated before and after
 *  each buffer dump, so that user-output can be separated from RTRT output,
 *  when these outputs are done on the same communication channel.
 *  Do not define if user outputs cannot be mixed with RTRT outputs
 *  (e.g. a real file system is used, or RTRT outputs are retrieved
 *  through a debugger). */
/* #define RTRT_LINE_SPLIT_MARKER '^' */

#ifndef RTRT_GETENV
/*! This defines which kind of getenv routine is to be used. Available values
 *  are:
 *  RTRT_STD
 *  RTRT_USR
 *  RTRT_NONE */
#define RTRT_GETENV RTRT_NONE
#endif

#ifndef RTRT_SIGNAL
/*! This defines which kind of signal registering routine is to be used.
 *  Avalaible values are:
 *  RTRT_STD
 *  RTRT_USR
 *  RTRT_NONE */
#define RTRT_SIGNAL RTRT_NONE
#endif

#ifndef RTRT_ERROR
/*! This defines which kind of error function is to be used. Available values
 *  are:
 *  RTRT_STD
 *  RTRT_USR
 *  RTRT_NONE */
#define RTRT_ERROR RTRT_NONE
#endif

#ifndef RTRT_MEMCPY
/*! This defines which kind of memcpy routine is to be used. Avalaible values
 *  are:
 * RTRT_STD
 * RTRT_USR
 * RTRT_NONE */
#define RTRT_MEMCPY RTRT_STD
#endif

#ifndef RTRT_GETPID
/*! This defines which kind of getpid routine is to be used. Avalaible values
 *  are:
 * RTRT_STD
 * RTRT_USR
 * RTRT_NONE */
#define RTRT_GETPID RTRT_NONE
#endif

#ifndef RTRT_GETHOSTNAME
/*! This defines which kind of get_host_name routine is to be used. Available
 *  values are:
 *  RTRT_STD
 *  RTRT_USR
 *  RTRT_NONE */
#define RTRT_GETHOSTNAME RTRT_NONE
#endif

#ifndef RTRT_RDV
/*! This defines which kind of rendez-vous implementation is to be used.
 *  Available values are:
 *  RTRT_STD
 *  RTRT_USR
 *  RTRT_NONE */
#define RTRT_RDV RTRT_STD
#endif

#ifndef RTRT_DYNAMIC_TRACKING
/*! Specifies whether dynamic tracking is supported by the virtual testers. */
#define RTRT_DYNAMIC_TRACKING 1
#endif

#if !RTRT_DYNAMIC_TRACKING
#ifndef ATL_TSTAGT_COMMTYPE
/*! Communication type for tagtlk routines. */
#define ATL_TSTAGT_COMMTYPE int
#endif
#endif

/* if the TDP define RTRT_BUFFER_SIZE it take priority on GUI definition */
#ifdef RTRT_BUFFER_SIZE
#ifdef ATL_BUFFER_SIZE
#if RTRT_BUFFER_SIZE < ATL_BUFFER_SIZE
#undef ATL_BUFFER_SIZE
#define ATL_BUFFER_SIZE RTRT_BUFFER_SIZE
#endif
#endif
#endif

#ifndef RTRT_FILENAME_UNUSED
#define RTRT_FILENAME_UNUSED 0
#endif

#ifndef RTRT_WATCH_DOG
#define RTRT_WATCH_DOG
#endif

#ifndef RTRT_SOCK_IPV6
#define RTRT_SOCK_IPV6 0
#endif
#if RTRT_SOCK_IPV6
#define RTRT_INET_FAMILY    AF_INET6
#define RTRT_SOCKADDR_IN	sockaddr_in6
#define RTRT_SIN_ADDR		sin6_addr
#define RTRT_SIN_PORT		sin6_port
#define RTRT_SIN_FAMILY		sin6_family
#define RTRT_ADDR_LOOPBACK	in6addr_loopback
#define RTRT_ADDR_ANY		in6addr_any
#else
#define RTRT_INET_FAMILY    AF_INET
#define RTRT_SOCKADDR_IN	sockaddr_in
#define RTRT_SIN_ADDR		sin_addr.s_addr
#define RTRT_SIN_PORT		sin_port
#define RTRT_SIN_FAMILY		sin_family
#define RTRT_ADDR_LOOPBACK	htonl (INADDR_LOOPBACK)
#define RTRT_ADDR_ANY		INADDR_ANY
#endif

#ifndef RTRT_SIN_ADDR_CONN_INIT
#if _ATL_WIN32_API || defined(__CYGWIN__)
#define RTRT_SIN_ADDR_CONN_INIT RTRT_ADDR_LOOPBACK
#else
#define RTRT_SIN_ADDR_CONN_INIT RTRT_ADDR_ANY
#endif
#endif

/*******************************************************************************
* Dll default settings
*******************************************************************************/
#ifndef ATL_USE_VOLATIL_DLL
#ifdef RTRT_USE_VOLATIL_DLL /* defined on compiler command for all application sources */
#define ATL_USE_VOLATIL_DLL 1
#else
#define ATL_USE_VOLATIL_DLL 0
#endif
#endif

#if ATL_USE_VOLATIL_DLL
#define ATL_T_FAMILY RTRT_FAMILY_TYPE
#ifdef RTRT_UNLOADABLE_DLL /* defined on compiler command for all sources involved into the unloadable DLL */
#define ATL_FAMILY_INIT_VALUE RTRT_FAMILY_INIT_VALUE
#endif /* RTRT_UNLOADABLE_DLL */
#endif /* RTRT_USE_VOLATIL_DLL */

#ifdef RTRT_TDP_COMPILATION
#if ATL_RUNTIME_FORMAT == ATL_DLL
#define ATL_TP_PROTO_PREFIX RTRT_DLL_EXPORT
#endif /* ATL_RUNTIME_FORMAT == ATL_DLL */
#endif /* RTRT_TDP_COMPILATION */

#ifdef RTRT_USE_TPDLL
#define ATL_TP_PROTO_PREFIX RTRT_DLL_IMPORT
#endif

#ifndef ATL_TP_PROTO_PREFIX
#define ATL_TP_PROTO_PREFIX
#endif /* ATL_TP_PROTO_PREFIX */

/*******************************************************************************
* threads default settings
*******************************************************************************/

#ifndef RTRT_THREADS
/* This defines which kind of thread is to be used. Avalaible values are:
 * RTRT_STD (POSIX on UNIX, win32 on Windows, OSF/1 on DEC systems)
 * RTRT_SOLARIS (Solaris threads on Solaris systems)
 * RTRT_USR (user-defined threads)
 * RTRT_NONE (no threads available) */
#define RTRT_THREADS RTRT_NONE
#endif

#if ATL_MULTI_THREADS && (RTRT_THREADS == RTRT_USR)

#ifndef RTRT_MT_THREAD_SELF_T
/*! Type of a thread identifier */
#define RTRT_MT_THREAD_SELF_T thread_t
#endif

#ifndef RTRT_MT_THREAD_SELF
/*! Expression returning the identifier of the calling thread */
#define RTRT_MT_THREAD_SELF thread_self ()
#endif

#ifndef RTRT_MT_THREAD_EQUAL
/*! Returns a boolean expression, true if the two thread identifiers refer to
 *  the same thread. */
#define RTRT_MT_THREAD_EQUAL(t1,t2) thread_equals(t1,t2)
#endif

#ifndef RTRT_MT_THREAD_ISALIVE
/*! says if the thread id is alive or not (=0). */
#define RTRT_MT_THREAD_ISALIVE(id) (id)
#endif

#ifndef RTRT_MT_THREAD_EXIT
/*! Code exiting the calling thread with v as return status value. */
#define RTRT_MT_THREAD_EXIT(v) thread_exit((int(v));
#endif

#ifndef RTRT_MT_CRITICAL_VAR_DECL
/*! Declaration a critical-section variable v. */
#define RTRT_MT_CRITICAL_VAR_DECL(v) thread_mutex_t v;
#endif

#ifndef RTRT_MT_CRITICAL_VAR_INIT
/*! Code initializing critical-section variable v. */
#define RTRT_MT_CRITICAL_VAR_INIT(v) thread_mutex_init(&v);
#endif

#ifndef RTRT_MT_CRITICAL_VAR_LOCK
/*! Code locking critical-section variable v. */
#define RTRT_MT_CRITICAL_VAR_LOCK(v) thread_mutex_lock(&v);
#endif

#ifndef RTRT_MT_CRITICAL_VAR_UNLOCK
/*! Code releasing critical-section variable v. */
#define RTRT_MT_CRITICAL_VAR_UNLOCK(v) thread_mutex_unlock(&v);
#endif

#ifndef RTRT_MT_CRITICAL_VAR_RECURSIVE
/*! Boolean value set to 1 if the critical-section is recursive. */
#define RTRT_MT_CRITICAL_VAR_RECURSIVE 0
#endif
#endif /* ATL_MULTI_THREADS && (RTRT_THREADS == RTRT_USR) */


#ifdef RTRT_TDP_COMPILATION
#if ATL_MULTI_THREADS
#if RTRT_THREADIDX == RTRT_USR

#ifndef RTRT_MT_THREAD_SPECIFIC_DATA_DECLARE
/*! Type of a thread-specific data key. */
#define RTRT_MT_THREAD_SPECIFIC_DATA_DECLARE int _atl_thread_data;
#endif

#ifndef RTRT_MT_THREAD_SPECIFIC_DATA_CREATE
/*! Code creating a new thread-specific data key. */
#define RTRT_MT_THREAD_SPECIFIC_DATA_CREATE
#endif

#ifndef RTRT_MT_THREAD_SPECIFIC_DATA_DELETE
/*! Code deleting a thread-specific data key, and the associated data. */
#define RTRT_MT_THREAD_SPECIFIC_DATA_DELETE
#endif

#ifndef RTRT_MT_THREAD_SPECIFIC_DATA_GET
/*! Code retrieving in v the thread-specific data associated to key k. */
#define RTRT_MT_THREAD_SPECIFIC_DATA_GET(v) v=_atl_thread_data;
#endif

#ifndef RTRT_MT_THREAD_SPECIFIC_DATA_SET
/*! Code assigning to v the thread-specific data associated to key t. */
#define RTRT_MT_THREAD_SPECIFIC_DATA_SET(v) _atl_thread_data=v;
#endif

#endif /* ATL_MULTI_THREADS && (RTRT_THREADIDX == RTRT_USR) */
#endif /* ATL_MULTI_THREADS && (RTRT_THREADIDX == RTRT_USR) */
#endif /* RTRT_TDP_COMPILATION */

/*******************************************************************************
* clock default settings
*******************************************************************************/

#ifndef RTRT_CLOCK
/*! This defines which kind of clock implementation is to be used. Available
 *  values are:
 *  RTRT_STD
 *  RTRT_USR
 *  RTRT_NONE */
#define RTRT_CLOCK RTRT_NONE
#endif

#if RTRT_CLOCK == RTRT_USR

#ifndef RTRT_CLOCK_TYPE
/*! Type of clock data */
#define RTRT_CLOCK_TYPE RTRT_U_INT32
#endif

#ifndef RTRT_CLOCK_RESOLUTION
/*! Resolution of clock (number of ticks per second). Set to 0 if resolution
    needs to be computed at run-time. */
#define RTRT_CLOCK_RESOLUTION 1
#endif

#ifndef RTRT_UNIT_TYPE
/*! Type of unit data */
#define RTRT_UNIT_TYPE RTRT_U_INT32
#endif

#ifndef RTRT_UNIT_TYPE_SIZE
/*! Size of unit type: sizeof (RTRT_UNIT_TYPE) */
#define RTRT_UNIT_TYPE_SIZE 4
#endif

#ifndef RTRT_UNIT_TYPE_NAME
/*! String describing the clock unit */
#define RTRT_UNIT_TYPE_NAME "Dummy Unit"
#endif

#ifndef RTRT_UNIT_TYPE_FORMAT
/*! Number describing the clock unit (0=milliseconds, 1=microseconds,
 *  2=cycles, 3=system clock) */
#define RTRT_UNIT_TYPE_FORMAT "3"
#endif

#ifndef RTRT_CLOCK_TO_UNITS
/*! Clock converted to units */
#define RTRT_CLOCK_TO_UNITS(c) (c)
#endif

#ifndef RTRT_CLOCK_DIFF
/*! Difference between two clocks */
#define RTRT_CLOCK_DIFF(r,c1,c2) ((r)=(a)-(b))
#endif

#ifndef RTRT_TIME_TYPE
/*! Type of time unit */
#define RTRT_TIME_TYPE RTRT_U_INT32
#endif

#ifndef RTRT_CLOCK_TO_TIME
/*! Clock converted to time */
#define RTRT_CLOCK_TIME(c) (c)
#endif

#ifndef RTRT_TIME_TYPE_STEP
/*! Step of time unit (in microseconds) */
#define RTRT_TIME_TYPE_STEP 1000L
#endif

#ifndef RTRT_TIME_TYPE_FORMAT
/*! Number describing the time unit (0=milliseconds, 1=microseconds,
 *  2=cycles, 3=system clock) */
#define RTRT_TIME_TYPE_FORMAT "3"
#endif

#ifndef RTRT_CLOCK_INIT_NEEDED
/*! Boolean value, set to 1 if usr_clock_init() function is provided and
 *  needs to be called at initialization. */
#define RTRT_CLOCK_INIT_NEEDED 0
#endif

#ifndef RTRT_SLEEP
/*! This defines whether a sleep function is available. Available values are:
 *  RTRT_USR
 *  RTRT_NONE */
#define RTRT_SLEEP RTRT_USR
#endif

#endif /* RTRT_CLOCK == RTRT_USR */

#ifndef RTRT_THREAD_CLOCK
#define RTRT_THREAD_CLOCK RTRT_NONE
#endif

#ifndef RTRT_GET_DATE
/*! This defines whether a date function is available. Available values are:
 *  RTRT_USR
 *  RTRT_NONE */
#define RTRT_GET_DATE RTRT_NONE
#endif
/*******************************************************************************
* Image default settings
*******************************************************************************/

#ifndef RTRT_STRING
/*! Strings image support */
#define RTRT_STRING 1
#endif

#ifndef RTRT_FLOAT
/*! Floating-point numbers image support */
#define RTRT_FLOAT 1
#endif

/*! Set this option to 1 into the TDP when ALL the RAM is set to 0
 *  before the code run. */
#ifndef RTRT_RAM_CLEAN
#define RTRT_RAM_CLEAN 0
#endif

/*! some function may be reduced for tiny target */
#ifndef RTRT_TINY_TARGET
#define RTRT_TINY_TARGET 0
#endif
/* keep compatibility with old version */
#ifdef ATL_TINY_TARGET
#undef RTRT_TINY_TARGET
#define RTRT_TINY_TARGET 1
#endif

#ifndef RTRT_SPRINTF
/*! Use printf family functions when available if set to 1. Otherwise use
 *  internal routines. native sprintf routines may be more efficient (and
 *  more accurate with floating numbers) than generic internal routines. */
#define RTRT_SPRINTF 1
#endif

#if RTRT_SPRINTF

#ifndef RTRT_SPRINTF_INTEGER_FORMAT
/*! String format for the largest signed integer supported (for sprintf) */
#ifdef RTRT_INT64
#ifdef _MSC_VER
#define RTRT_SPRINTF_INTEGER_FORMAT "%I64d"
#else
#define RTRT_SPRINTF_INTEGER_FORMAT "%lld"
#endif /* _MSC_VER */
#else /* RTRT_INT64 */
#define RTRT_SPRINTF_INTEGER_FORMAT "%ld"
#endif /* RTRT_INT64 */
#endif /* RTRT_SPRINTF_INTEGER_FORMAT */

#ifndef RTRT_SPRINTF_UNSIGNED_FORMAT
/*! String format for the largest signed integer supported (for sprintf) */
#ifdef RTRT_U_INT64
#ifdef _MSC_VER
#define RTRT_SPRINTF_UNSIGNED_FORMAT "%I64u"
#else
#define RTRT_SPRINTF_UNSIGNED_FORMAT "%llu"
#endif /* _MSC_VER */
#else /* RTRT_U_INT64 */
#define RTRT_SPRINTF_UNSIGNED_FORMAT "%lu"
#endif /* RTRT_U_INT64 */
#endif /* RTRT_SPRINTF_INTEGER_FORMAT */

#endif /* RTRT_SPRINTF */

#ifndef RTRT_LONG_DOUBLE
#if RTRT_CPLUSPLUS_LANGUAGE
/*! Long double numbers image support */
#define RTRT_LONG_DOUBLE 1
#else
#define RTRT_LONG_DOUBLE 0
#endif
#endif

#if ! RTRT_SPRINTF
#ifndef RTRT_FLOAT_DUMP
/*! Use internal representation for floating-point numbers ("dump" image) if
 *  if set to 1. Otherwise, use "natural" representation (less efficient, more
 *  readable). "Dump" representation may be chosen only when
 *  RTRT_SPRINTF == 0.
 *  */
#define RTRT_FLOAT_DUMP 0
#endif
#endif /* RTRT_SPRINTF */

#ifndef RTRT_NBDIGITS_FLT
/*! Number of significant digits encoded in float numbers */
#define RTRT_NBDIGITS_FLT 6
#endif

#ifndef RTRT_NBDIGITS_DBL
/*! Number of significant digits encoded in double numbers */
#define RTRT_NBDIGITS_DBL 14
#endif

#ifndef RTRT_NBDIGITS_LDBL
/*! Number of significant digits encoded in long double numbers */
#define RTRT_NBDIGITS_LDBL 14
#endif

#ifndef RTRT_FLOAT_NAN
#define RTRT_FLOAT_NAN 1
#endif

#if !RTRT_SPRINTF && RTRT_FLOAT_DUMP
#ifndef RTRT_DUMP_OBJ
/*! type used to cast floats in functions float_image, double_image and
 * long_doubel_image when  RTRT_FLOAT_DUMP==1. This type must be UNSIGNED. */
#define RTRT_DUMP_OBJ unsigned long int
#endif
#endif /* !RTRT_SPRINTF && RTRT_FLOAT_DUMP */

#ifndef RTRT_PRECISION
/*! Precision to test equality between two reals (float and double) (by default
 *  1.0e-5) */
#define RTRT_PRECISION  1.0E-5
#endif

#ifndef RTRT_QUOT
#define RTRT_QUOT '"' /* " add this to balance the uniq quot here (need for old compilers) */
#endif

#ifndef RTRT_CONST_STRING
#define RTRT_CONST_STRING(s) s
#endif

#ifndef RTRT_CONST_STRING_DECL
#define RTRT_CONST_STRING_DECL const char *
#endif /* RTRT_CONST_STRING_DECL */

/*******************************************************************************
* TestRT Visual Test default settings
*******************************************************************************/
/* ATVUSE_OPTIMIZE is defined by the TS name .h which is included by the product.h as attol_cfg.h when optimization is required */
#if defined( ATVUSE_OPTIMIZE ) || !USE_ATV
#define ATVUSE_UNUSED 0
#else /* ATVUSE_OPTIMIZE */
/* if no optimization required the complete runtime is generated */
#define ATVUSE_UNUSED 1
#endif /* ATVUSE_OPTIMIZE */

#ifndef ATVUSE_FLOAT
#define ATVUSE_FLOAT ATVUSE_UNUSED
#endif
#ifndef ATVUSE_DOUBLE
#define ATVUSE_DOUBLE ATVUSE_UNUSED
#endif
#ifndef ATVUSE_STRING
#define ATVUSE_STRING ATVUSE_UNUSED
#endif
#ifndef ATVUSE_CHAR
#define ATVUSE_CHAR ATVUSE_UNUSED
#endif
#ifndef ATVUSE_INTEGER
#define ATVUSE_INTEGER ATVUSE_UNUSED
#endif
#ifndef ATVUSE_UNSIGNED
#define ATVUSE_UNSIGNED ATVUSE_UNUSED
#endif
#ifndef ATVUSE_BOOLEAN
#define ATVUSE_BOOLEAN ATVUSE_UNUSED
#endif
#ifndef ATVUSE_POINTER
#define ATVUSE_POINTER ATVUSE_UNUSED
#endif
#ifndef ATVUSE_STUBS
#define ATVUSE_STUBS ATVUSE_UNUSED
#endif
#ifndef ATV_REPORT_OUTLAY
/*! standard mode expand arrays */
#define ATV_REPORT_OUTLAY 0
#endif

/* for futur use: selection of the memory space allocation by setting */
#ifndef RTRT_ALLOC_IN_STACK
#define RTRT_ALLOC_IN_STACK 0
#endif
/*******************************************************************************
* TestRT Unit Testing default settings
*******************************************************************************/

#ifndef RTRT_STR_CONCAT_SUPPORTED
/*! In some specific Target Deployment Ports, the CONCAT_STR_C(PP) function is
 *  defined into the customisable part. This macro is used to avoid multiple
 *  definition. By default, the CONCAT_STR is defined in non-custo part. */
#if RTRT_STRING
#ifdef RTRT_STRING_CONCAT_SUPPORTED
#define RTRT_STR_CONCAT_SUPPORTED RTRT_STRING_CONCAT_SUPPORTED
#else /* RTRT_STRING_CONCAT */
#define RTRT_STR_CONCAT_SUPPORTED 1
#endif /* RTRT_STRING_CONCAT */
#else
#define RTRT_STR_CONCAT_SUPPORTED 0
#endif
#endif

#ifndef RTUT_NOT_INIT_NIL
/*! Definition used in function init_local_vars() in order to protect the
 *  initialization of NIL. */
#define RTUT_NOT_INIT_NIL 0
#endif

#ifndef RTUT_NOT_INIT_NONIL
/*! Definition used in function init_local_vars() in order to protect the
 *  initialization of NONIL. */
#define RTUT_NOT_INIT_NONIL 0
#endif

#ifndef RTUT_NONIL_VALUE
/*! Definition used in function init_local_vars() for
 *  initialization of NONIL. */
#define RTUT_NONIL_VALUE -7
#endif

#ifndef RTRT_ATU_DELAY
/*! Incompressible time spent during the test, due to RTRT procedures.
 *  Default is 0 i.e. time spent for an empty test case */
#define RTRT_ATU_DELAY 0
#endif
#define RTUT_DELAY RTRT_ATU_DELAY

/*! Runtime optimization  */
#ifndef RTRTUSE_EDV_BOOLEAN_C
#define RTRTUSE_EDV_BOOLEAN_C 1
#endif

#ifndef RTRTUSE_ARRAY_BOOLEAN_C
#define RTRTUSE_ARRAY_BOOLEAN_C 1
#endif

#ifndef RTRTUSE_CONSTANT_BOOLEAN_C
#define RTRTUSE_CONSTANT_BOOLEAN_C 1
#endif

#ifndef RTRTUSE_EDV_CHAR_C
#define RTRTUSE_EDV_CHAR_C 1
#endif

#ifndef RTRTUSE_ARRAY_CHAR_C
#define RTRTUSE_ARRAY_CHAR_C 1
#endif

#ifndef RTRTUSE_CONSTANT_CHAR_C
#define RTRTUSE_CONSTANT_CHAR_C 1
#endif

#ifndef RTRTUSE_EDV_DOUBLE_C
#define RTRTUSE_EDV_DOUBLE_C 1
#endif

#ifndef RTRTUSE_ARRAY_DOUBLE_C
#define RTRTUSE_ARRAY_DOUBLE_C 1
#endif

#ifndef RTRTUSE_CONSTANT_DOUBLE_C
#define RTRTUSE_CONSTANT_DOUBLE_C 1
#endif

#ifndef RTRTUSE_EDV_FLOAT_C
#define RTRTUSE_EDV_FLOAT_C 1
#endif

#ifndef RTRTUSE_ARRAY_FLOAT_C
#define RTRTUSE_ARRAY_FLOAT_C 1
#endif

#ifndef RTRTUSE_CONSTANT_FLOAT_C
#define RTRTUSE_CONSTANT_FLOAT_C 1
#endif

#ifndef RTRTUSE_EDV_INTEGER_C
#define RTRTUSE_EDV_INTEGER_C 1
#endif

#ifndef RTRTUSE_ARRAY_INTEGER_C
#define RTRTUSE_ARRAY_INTEGER_C 1
#endif

#ifndef RTRTUSE_CONSTANT_INTEGER_C
#define RTRTUSE_CONSTANT_INTEGER_C 1
#endif

#ifndef RTRTUSE_EDV_POINTER_C
#define RTRTUSE_EDV_POINTER_C 1
#endif

#ifndef RTRTUSE_ARRAY_POINTER_C
#define RTRTUSE_ARRAY_POINTER_C 1
#endif

#ifndef RTRTUSE_CONSTANT_POINTER_C
#define RTRTUSE_CONSTANT_POINTER_C 1
#endif

#ifndef RTRTUSE_EDV_STR_C
#define RTRTUSE_EDV_STR_C 1
#endif

#ifndef RTRTUSE_ARRAY_STR_C
#define RTRTUSE_ARRAY_STR_C 1
#endif

#ifndef RTRTUSE_CONSTANT_STR_C
#define RTRTUSE_CONSTANT_STR_C 1
#endif

#ifndef RTRTUSE_EDV_UNSIGNED_C
#define RTRTUSE_EDV_UNSIGNED_C 1
#endif

#ifndef RTRTUSE_ARRAY_UNSIGNED_C
#define RTRTUSE_ARRAY_UNSIGNED_C 1
#endif

#ifndef RTRTUSE_CONSTANT_UNSIGNED_C
#define RTRTUSE_CONSTANT_UNSIGNED_C 1
#endif

#ifndef RTRTUSE_PTU_BOOLEAN_C
#define RTRTUSE_PTU_BOOLEAN_C 1
#endif

#ifndef RTRTUSE_PTU_CHAR_C
#define RTRTUSE_PTU_CHAR_C 1
#endif

#ifndef RTRTUSE_PTU_DOUBLE_C
#define RTRTUSE_PTU_DOUBLE_C 1
#endif

#ifndef RTRTUSE_PTU_FLOAT_C
#define RTRTUSE_PTU_FLOAT_C 1
#endif

#ifndef RTRTUSE_PTU_INTEGER_C
#define RTRTUSE_PTU_INTEGER_C 1
#endif

#ifndef RTRTUSE_PTU_UNSIGNED_C
#define RTRTUSE_PTU_UNSIGNED_C 1
#endif

#ifndef RTRTUSE_INI_STR_C
#define RTRTUSE_INI_STR_C 1
#endif

#ifndef RTRTUSE_CONCAT_STR_C
#define RTRTUSE_CONCAT_STR_C RTRTUSE_INI_STR_C+RTRTUSE_EDV_STR_C
#endif

#ifndef RTRTUSE_CHRONO_MUET_C
#define RTRTUSE_CHRONO_MUET_C 1
#endif

#ifndef RTRTUSE_COMP_STR_C
#define RTRTUSE_COMP_STR_C 1
#endif

#ifndef RTRTUSE_DCL_FAC_REP_C
#define RTRTUSE_DCL_FAC_REP_C 1
#endif

#ifndef RTRTUSE_MUET_CHECK
#define RTRTUSE_MUET_CHECK 1
#endif

#ifndef RTRTUSE_OPEN_BRA_C
#define RTRTUSE_OPEN_BRA_C 1
#endif

#ifndef RTRTUSE_RUPT
#define RTRTUSE_RUPT 1
#endif

#if RTRTUSE_ARRAY_BOOLEAN_C+RTRTUSE_ARRAY_CHAR_C+RTRTUSE_ARRAY_DOUBLE_C+RTRTUSE_ARRAY_FLOAT_C+RTRTUSE_ARRAY_INTEGER_C+RTRTUSE_ARRAY_POINTER_C+RTRTUSE_ARRAY_STR_C+RTRTUSE_ARRAY_UNSIGNED_C
#define RTRTUSE_ARRAY 1
#else
#define RTRTUSE_ARRAY 0
#endif
#if RTRTUSE_CONSTANT_BOOLEAN_C+RTRTUSE_CONSTANT_CHAR_C+RTRTUSE_CONSTANT_DOUBLE_C+RTRTUSE_CONSTANT_FLOAT_C+RTRTUSE_CONSTANT_INTEGER_C+RTRTUSE_CONSTANT_POINTER_C+RTRTUSE_CONSTANT_STR_C+RTRTUSE_CONSTANT_UNSIGNED_C
#define RTRTUSE_MODE_CONSTANT 1
#else
#define RTRTUSE_MODE_CONSTANT 0
#endif
#if RTRTUSE_EDV_DOUBLE_C+RTRTUSE_PTU_DOUBLE_C+ATVUSE_DOUBLE
#define RTRTUSE_DOUBLE 1
#else
#define RTRTUSE_DOUBLE 0
#endif
#if RTRTUSE_EDV_FLOAT_C+RTRTUSE_PTU_FLOAT_C+ATVUSE_FLOAT
#define RTRTUSE_FLOAT 1
#else
#define RTRTUSE_FLOAT 0
#endif
#if RTRTUSE_INI_STR_C+RTRTUSE_EDV_STR_C+ATVUSE_STRING
#define RTRTUSE_STRING 1
#else
#define RTRTUSE_STRING 0
#endif

/*******************************************************************************
* TestRT System Testing default settings
*******************************************************************************/

#ifndef RTST_ID_SIZE
/*! Maximum number of connection identifiers for a channel (by default 10) */
#define RTST_ID_SIZE 10
#endif

#ifndef RTST_HEX_LG
/*! Length used for the hexadecimal dump (by default eight bytes) */
#define RTST_HEX_LG 8
#endif

#ifndef RTST_THREAD_RETURN
/*! Return type of start routine in thread mode */
#define RTST_THREAD_RETURN void
#endif

#ifndef RTST_WTIME_SECONDS
/*! Time granularity */
#define RTST_WTIME_SECONDS 0L
#endif

#ifndef RTST_WTIME_MICRO_SECONDS
/*! Time granularity */
#define RTST_WTIME_MICRO_SECONDS 10000L
#endif

#ifndef RTST_DURATION_SIZE
#define RTST_DURATION_SIZE 20
#endif

#ifndef RTST_IMAGE_SIZE
/*! Maximum size in bytes of number image */
#define RTST_IMAGE_SIZE 65
#endif

#ifndef RTST_BLOCK_STACK_SIZE
/*! Maximum size of block level */
#define RTST_BLOCK_STACK_SIZE 20
#endif

#ifndef RTST_BLOCK_NAME_SIZE
/*! Maximum size in char of block name */
#define RTST_BLOCK_NAME_SIZE 100
#endif

#ifndef RTST_CTX_STACK_SIZE
/*! Maximum size of block ctx (circular buffer) */
#define RTST_CTX_STACK_SIZE 100
#endif

/*******************************************************************************
* TestRT Object Testing default settings
*******************************************************************************/
/* possibility to change the entry point for a test node */
#ifdef RTRT_HEADER_MAIN
#undef RTRT_MAIN_HEADER
#define RTRT_MAIN_HEADER RTRT_HEADER_MAIN
#endif /* RTRT_HEADER_MAIN */
/*******************************************************************************
* TestRT Contract Checks default settings
*******************************************************************************/

/*******************************************************************************
* TestRT Coverage default settings
*******************************************************************************/

#ifndef ATC_DUMP_DRIVER
#define ATC_DUMP_DRIVER 0
#endif

/* coverage on the fly is available only in PASS mode */
#ifdef ATC_INFORMATION
#if ATC_INFORMATION
#undef ATC_ON_THE_FLY
#endif
#endif
#ifndef ATC_ON_THE_FLY
#define ATC_ON_THE_FLY 0
#endif

#ifndef ATC_ON_INTERRUPT
#define ATC_ON_INTERRUPT 0
#endif

/*******************************************************************************
* TestRT Trace default settings
*******************************************************************************/
#ifndef ATT_TIMESTAMP
#define ATT_TIMESTAMP 0
#endif

#if RTRT_HEAP == RTRT_NONE
#undef ATT_HEAPSIZESTAMP
#endif
#ifndef ATT_HEAPSIZESTAMP
#define ATT_HEAPSIZESTAMP 0
#endif

/* allows to display the max call stack level found during exec */
#ifndef ATT_COMPUTE_MAX_CALLING_LEVEL
#define ATT_COMPUTE_MAX_CALLING_LEVEL 0
#endif

/* allows to get the call stack of the deepest stack size */
#ifndef ATP_STACK_SIZE_LEVEL
#define ATP_STACK_SIZE_LEVEL 0
#endif

#ifndef ATL_STACK_CALLING_LEVEL
#if (USE_ATT && ATT_COMPUTE_MAX_CALLING_LEVEL) || (ATP_STACK_SIZE_LEVEL !=0)
#define ATL_STACK_CALLING_LEVEL 1
#else  /* ATT_COMPUTE_MAX_CALLING_LEVEL */
#define ATL_STACK_CALLING_LEVEL 0
#endif /* ATT_COMPUTE_MAX_CALLING_LEVEL */
#endif /* ATL_STACK_CALLING_LEVEL */

#if RTRT_IO == RTRT_NONE
#ifdef ATT_ON_THE_FLY
#if ATT_ON_THE_FLY
#undef ATT_ON_THE_FLY
#define ATT_ON_THE_FLY 0
#endif
#endif
#endif

/*******************************************************************************
* TestRT PurifyLT default settings
*******************************************************************************/

/* Allocation Functions are:
 *   malloc/call/realloc/free -> RTRT_STD
 *   user defined -> RTRT_USR
 *   do not exist -> RTRT_NONE */
#ifndef RTRT_HEAP
#define RTRT_HEAP RTRT_STD
#endif /* RTRT_HEAP */

/* Overriding method of malloc/call/realloc/free:
 *   link time -> RTRT_USR
 *   preprocessing time -> RTRT_NONE */
#ifndef RTRT_LOWLEVEL_MEMLIB
#define RTRT_LOWLEVEL_MEMLIB RTRT_NONE
#endif /* RTRT_LOWLEVEL_MEMLIB */

/* Extension of malloc/calloc/realloc/free family:
 *   Enabled -> RTRT_USR
 *   Not used -> RTRT_NONE */
#ifndef RTRT_MEMLIB_EXTENSION
#define RTRT_MEMLIB_EXTENSION RTRT_NONE
#endif /* RTRT_MEMLIB_EXTENSION */

#ifndef ATP_REPORTS_FIU
#define ATP_REPORTS_FIU 0
#endif

#ifndef ATP_REPORTS_FUM
#define ATP_REPORTS_FUM 1
#endif

#ifndef ATP_REPORTS_MPKC
#define ATP_REPORTS_MPKC 1
#endif

#ifndef ATP_REPORTS_MPK
#define ATP_REPORTS_MPK 1
#endif

#ifndef ATP_REPORTS_MLK
#define ATP_REPORTS_MLK 1
#endif

#ifndef ATP_REPORTS_PMLK
#define ATP_REPORTS_PMLK 0
#endif

#ifndef ATP_CLEAN
#define ATP_CLEAN 0
#endif

#ifndef RTRT_NEW_INLINE
#define RTRT_NEW_INLINE 0
#endif
#if RTRT_NEW_INLINE
#define RTRT_NEW_DELETE_INLINE
#endif


/* Allows to add an exception specification to fileno() and malloc() family */
#ifndef RTRT_THROWING_EXCEPTION
#define RTRT_THROWING_EXCEPTION
#endif /* RTRT_THROWING_EXCEPTION */

/* Allows to add an exception specification to new() delete() family */
#ifndef RTRT_NEW_THROW
#define RTRT_NEW_THROW RTRT_THROWING_EXCEPTION
#endif /* RTRT_NEW_THROW */

#ifndef RTRT_DELETE_THROW
#define RTRT_DELETE_THROW RTRT_NEW_THROW
#endif /* RTRT_DELETE_THROW */

/* Allows to track strdup (1) or not (0) */
#if ( RTRT_HEAP == RTRT_STD ) && ( RTRT_LOWLEVEL_MEMLIB == RTRT_NONE ) && !defined(RTRT_ATP_STRDUP)
#define RTRT_ATP_STRDUP 1
#endif

/* since v7.0.0.1 the global new and delete can be defined as inline methode by ticking RTRT_NEW_INLINE in the C++ Specific section of the TDP */
#if ( RTRT_HEAP != RTRT_NONE ) && ( RTRT_LOWLEVEL_MEMLIB == RTRT_NONE ) && defined(RTRT_NEW_DELETE_INLINE)
/* or by the Implementation of the C++ global ::new and ::delete functions as follow in the User-defined extra code */
#define RTRT_INLINE_NEW_DELETE              \
                                            \
inline void * ATL_CALLCONV operator new(RTRT_SIZE_T size) RTRT_NEW_THROW \
{                                           \
  return _atp_alloc ( size ,1);             \
}                                           \
                                            \
inline void * ATL_CALLCONV operator new[] (RTRT_SIZE_T size) RTRT_NEW_THROW  \
{                                           \
  return _atp_alloc ( size ,2);             \
}                                           \
                                            \
inline void  ATL_CALLCONV operator delete ( void *ptr ) RTRT_DELETE_THROW  \
{                                           \
  _atp_freeT ( ptr,1);                      \
}                                           \
                                            \
inline void  ATL_CALLCONV operator delete[] ( void *ptr ) RTRT_DELETE_THROW \
{                                           \
  _atp_freeT ( ptr,2);                      \
}                                           \

#endif

#if ( RTRT_HEAP == RTRT_STD ) && ( RTRT_LOWLEVEL_MEMLIB == RTRT_NONE )  && !defined(RTRT_INLINE_NEW_DELETE) && !defined(RTRT_DEFINE_NEW_DELETE)
/* the Implementation of the C++ global ::new and ::delete functions in the runtime can be done as follow in the User-defined extra code */
#define RTRT_DEFINE_NEW_DELETE              \
                                            \
void * ATL_CALLCONV operator new ( RTRT_SIZE_T size ) RTRT_NEW_THROW    \
{                                           \
  return _atp_alloc ( size ,1);             \
}                                           \
                                            \
void * ATL_CALLCONV operator new[] ( RTRT_SIZE_T size ) RTRT_NEW_THROW  \
{                                           \
  return _atp_alloc ( size ,2);             \
}                                           \
                                            \
void  ATL_CALLCONV operator delete ( void *ptr ) RTRT_DELETE_THROW         \
{                                           \
  _atp_freeT ( ptr,1);                      \
}                                           \
                                            \
void  ATL_CALLCONV operator delete[] ( void *ptr ) RTRT_DELETE_THROW       \
{                                           \
  _atp_freeT ( ptr,2);                      \
}                                           \

#endif

#ifndef RTRT_CARE_ABOUT_PTR_ALIGNMENT
/*! 1 if must take care when dereferencing an unaligned data */
#if RTRT_IO == RTRT_STD
#define RTRT_CARE_ABOUT_PTR_ALIGNMENT 0
#else /* RTRT_IO == RTRT_STD */
#define RTRT_CARE_ABOUT_PTR_ALIGNMENT 1
#endif /* RTRT_IO == RTRT_STD */
#endif /* RTRT_CARE_ABOUT_PTR_ALIGNMENT */

/* Care about stack link datas (1) or not (0), no check is faster */
#ifndef RTRT_CHECK_STACK
#define RTRT_CHECK_STACK 0
#endif /* RTRT_CHECK_STACK */

/* use different size for head and foot red zone if needed */
#ifndef ATP_RED_ZONE_HEAD_SIZE
#define ATP_RED_ZONE_HEAD_SIZE ATP_RED_ZONE_SIZE
#endif
#ifndef ATP_RED_ZONE_TAIL_SIZE
#define ATP_RED_ZONE_TAIL_SIZE ATP_RED_ZONE_SIZE
#endif

/* Maximum Number of Bytes for 1 block Kept Unfreed in the Free Queue */
#ifndef ATP_MAX_FREEQ_BLOCK_SIZE
#define ATP_MAX_FREEQ_BLOCK_SIZE ATP_MAX_FREEQ_SIZE
#else
#if ATP_MAX_FREEQ_BLOCK_SIZE > ATP_MAX_FREEQ_SIZE
#undef ATP_MAX_FREEQ_BLOCK_SIZE
#define ATP_MAX_FREEQ_BLOCK_SIZE ATP_MAX_FREEQ_SIZE
#endif
#if ATP_MAX_FREEQ_BLOCK_SIZE == 0
#undef ATP_MAX_FREEQ_BLOCK_SIZE
#define ATP_MAX_FREEQ_BLOCK_SIZE ATP_MAX_FREEQ_SIZE
#endif
#endif

/* does not allow ABWL and FMWL detection on selective list of memory blocks */
#ifndef ATP_CHECK_SELECTED_BLOCKS
#define ATP_CHECK_SELECTED_BLOCKS 0
#endif

/* allows breaking the code under debug when ATP errors are detected */
#ifndef BREAK_ON_ATP_ERROR
#define BREAK_ON_ATP_ERROR ATL_NO
#endif

#ifndef RTRT_FREE_PARAM
#define RTRT_FREE_PARAM 0
#endif

#ifndef ATP_KEEP_CONTENT
#define ATP_KEEP_CONTENT 0
#endif /* ATP_KEEP_CONTENT */

#ifndef ATP_NO_TRACK_ALLOCS_BEFORE_INIT
#define ATP_NO_TRACK_ALLOCS_BEFORE_INIT 0
#endif /* ATP_NO_TRACK_ALLOCS_BEFORE_INIT */

#ifndef RTRT_ATP_API_REALLOC
#define RTRT_ATP_API_REALLOC 1
#endif
#ifndef RTRT_ATP_API_CHECK
#define RTRT_ATP_API_CHECK 0
#endif
#ifndef RTRT_ATP_API_RESIZE
#define RTRT_ATP_API_RESIZE 0
#endif

#ifndef RTRT_ATP_API_TRACK
#define RTRT_ATP_API_TRACK 0
#endif /* RTRT_ATP_API_TRACK */

#ifndef RTRT_ISOPENMET
#if RTRT_IO == RTRT_STD
#define RTRT_ISOPENMET RTRT_STD
#else
#define RTRT_ISOPENMET RTRT_NONE
#endif /* RTRT_IO == RTRT_STD */
#endif /* RTRT_ISOPENMET */

#ifndef ATP_DUMP_UNFREED_ONLY_WITH_STACK
#define ATP_DUMP_UNFREED_ONLY_WITH_STACK 0
#endif

#ifndef ATL_NAMESPACE_BEGIN
#define ATL_NAMESPACE_BEGIN
#endif

#ifndef ATL_NAMESPACE_END
#define ATL_NAMESPACE_END
#endif

#if RTRT_HEAP == RTRT_NONE
#undef ATP_REPORTS_ABWL
#undef ATP_REPORTS_FFM_FMWL
#undef ATP_REPORTS_FUM
#undef ATP_REPORTS_FIM
#undef ATP_REPORTS_MLK
#undef ATP_REPORTS_MPK
#undef ATP_REPORTS_MIU
#define ATP_REPORTS_ABWL 0
#define ATP_REPORTS_FFM_FMWL 0
#define ATP_REPORTS_FUM 0
#define ATP_REPORTS_FIM 0
#define ATP_REPORTS_MLK 0
#define ATP_REPORTS_MPK 0
#define ATP_REPORTS_MIU 0
#endif /*RTRT_HEAP == RTRT_NONE*/

#if ATP_REPORTS_ABWL || ATP_REPORTS_FFM_FMWL || ATP_REPORTS_FUM || ATP_REPORTS_FIM || ATP_REPORTS_MLK || ATP_REPORTS_MPK || ATP_REPORTS_MIU
#define ATP_HEAP_TRACKING 1
#else
#define ATP_HEAP_TRACKING 0
#endif
/*******************************************************************************
* TestRT QuantifyLT default settings
*******************************************************************************/
#ifndef ATQ_COMPUTE_MINMAX
#define ATQ_COMPUTE_MINMAX 0
#endif

#ifndef ATQ_DUMP_DRIVER
#define ATQ_DUMP_DRIVER 0
#endif

#ifndef ATQ_THREAD_FILTERING
#define ATQ_THREAD_FILTERING
#endif
/*******************************************************************************
* TestRT General default settings
*******************************************************************************/

#ifndef ATL_CHECK_STACK
#define ATL_CHECK_STACK RTRT_CHECK_STACK
#else
#if RTRT_CHECK_STACK
#undef  ATL_CHECK_STACK
#define ATL_CHECK_STACK 1
#endif
#endif

#ifndef ATL_DYN_TABLE
/* define if instrumentation table for ATC, ATQ ATP must be allocated in the Heap */
#define ATL_DYN_TABLE 0
#endif

/* DEBUG the runtime
#define  TDPDEBUG(txt) printf("#" txt "\n");
#define  TDPDEBUG(txt) atl_buffer_add("WD " txt "\n");atl_buffer_dump(); // to print debug info into spt file
 */
#ifndef TDPDEBUG
#define TDPDEBUG(txt)
#endif

#endif

