

/*
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2011 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */


/*******************************************************************************
* $Descr: QuantifyLT Library Source (included by TP.c[pp]) $
*******************************************************************************/


/*******************************************************************************
* Global variables
*******************************************************************************/

/* Header of linked-list */
RTRT_GLOBALVAR _atq_TP _atq_first;
RTRT_GLOBALVAR RTRT_U_INT8 _atq_header;

#if ATL_MULTI_THREADS

RTRT_MT_CRITICAL_VAR_DECL(atq_critical)

#endif /* ATL_MULTI_THREADS */

#if ATQ_COMPUTE_OVERHEAD
/* Self-computed overhead results */
RTRT_GLOBALVAR _atl_T_time _atq_overhead_in;
RTRT_GLOBALVAR _atl_T_time _atq_overhead_out;
RTRT_GLOBALVAR _atl_T_time _atq_overhead_call;
#endif /* ATQ_COMPUTE_OVERHEAD */

/* What is the message corresponding to an index */
#define _ATQ_MESSAGE_FROM_INDEX(i) \
  ((((i)+1)&0x0000001f)|((((i)+1)&0xffffffe0)<<1))

/* What is the index corresponding to a message */
#define _ATQ_INDEX_FROM_MESSAGE(m) \
  ((((m)&0x0000001f)|(((m)&0xffffffc0)>>1))-1)

#if ATL_MULTI_THREADS==1  && RTRT_THREAD_CLOCK == RTRT_USR
#define ATL_CPU_TIME(id) _atl_cpu_time(id)
/*******************************************************************************
*  Service: _atl_cpu_time
*******************************************************************************/
RTRT_STATICFCT _atl_T_time _atl_cpu_time
#if !RTRT_KR
( RTRT_MT_THREAD_SELF_T  threadId)
#else
( threadId )
RTRT_MT_THREAD_SELF_T  threadId;
#endif
{ /* as default use the clock */
  RTRT_CLOCK_TYPE cl;

  usr_thread_clock ( &cl , threadId);
  return (_atl_T_time) RTRT_CLOCK_TO_UNITS ( cl );
}
#else /* ATL_MULTI_THREADS==1  && RTRT_THREAD_CLOCK == RTRT_USR */
#ifndef ATL_CPU_TIME
#define ATL_CPU_TIME(x) _atl_cpu_time()
#define ATQ_CPU_CLOCK /* we use the same machine clock for all threads */
/*******************************************************************************
*  Service: _atl_cpu_time
*******************************************************************************/
RTRT_STATICFCT _atl_T_time _atl_cpu_time
#if !RTRT_KR
( void )
#else
( )
#endif
{ /* as default use the clock */
  RTRT_CLOCK_TYPE cl;

  priv_clock ( &cl );
  return (_atl_T_time) RTRT_CLOCK_TO_UNITS ( cl );
}
#endif /* ATL_CPU_TIME */

#endif /* ATL_MULTI_THREADS==1  && RTRT_THREAD_CLOCK == RTRT_USR */


#if ATL_MULTI_THREADS==1
#if RTRT_THREAD_CLOCK == RTRT_HOOK || RTRT_THREAD_CLOCK == RTRT_AUTO
/*******************************************************************************
*  Service: _atq_schedule_hook
*  when RTRT_THREAD_CLOCK == RTRT_HOOK this methode must be called by a callback
*  when the RTOS reschedule to change the active task.
*  The parameters are the internal thead index  for the current (stopped task)
*  and the next task as they are memorized by the specific data of the task.
*******************************************************************************/
_ATL_C_GLOBALFCT void _atq_schedule_hook
#if !RTRT_KR
(RTRT_THIDX_T current, RTRT_THIDX_T next )
#else
(current, next  )
RTRT_THIDX_T current;
RTRT_THIDX_T next;
#endif
{
  _atl_T_time now,elapse;
  _AtlStackLinkP stack_link;

  now = ATL_CPU_TIME ( 0 );

  stack_link=_atl_stack_table_current[current];
  if(stack_link && current <= atl_threads_index_max)
  {
	  /* stop count for the current thread */
	  elapse = now - stack_link->PeriodStart;
	  stack_link->LocalAmount += elapse;
	  stack_link->TotalAmount += elapse;
	  stack_link->PeriodStart = now;
  }

  /* restart count for the next thread */
  stack_link=_atl_stack_table_current[next];
  if(stack_link && next <= atl_threads_index_max)
  {
	  stack_link->PeriodStart = now;
  }
}

#if RTRT_THREAD_CLOCK == RTRT_HOOK
/*******************************************************************************
*  Service: _atl_schedule_hook
*  when RTRT_THREAD_CLOCK == RTRT_HOOK this methode must be called by a callback
*  when the RTOS reschedule to change the active task.
*  The parameters are the internal thead index  for the current (stopped task)
*  and the next task as they are memorized by the specific data of the task.
*******************************************************************************/
_ATL_C_GLOBALFCT void atl_schedule_hook
#if !RTRT_KR
(RTRT_MT_THREAD_SELF_T current, RTRT_MT_THREAD_SELF_T next )
#else
(current, next  )
RTRT_MT_THREAD_SELF_T current;
RTRT_MT_THREAD_SELF_T next;
#endif
{
	RTRT_THIDX_T currentIdx;
	RTRT_THIDX_T thedata;
    RTRT_MT_THREAD_SELF_T self=current;

 	RTRT_MT_THREAD_SPECIFIC_DATA_GET ( thedata );

 	if ( _atl_threads_table_used[thedata] != self )
 	{
 		ATL_ERROR('q',"atl_schedule_hook current Unknow thread");
 		thedata=0;
 	}
    currentIdx=thedata;

    self=next;
 	RTRT_MT_THREAD_SPECIFIC_DATA_GET ( thedata );

 	if ( _atl_threads_table_used[thedata] != self )
 	{
 		ATL_ERROR('q',"atl_schedule_hook next Unknow thread");
 		thedata=0;
 	}

 	_atq_schedule_hook(currentIdx,thedata);
}
#endif /* RTRT_THREAD_CLOCK == RTRT_HOOK  */

#endif /* RTRT_THREAD_CLOCK == RTRT_HOOK|RTRT_AUTO  */
#endif /* ATL_MULTI_THREADS==1 */

/*******************************************************************************
*  Service: _atq_end_now
*******************************************************************************/
RTRT_STATICFCT int _atq_end_now
#if !RTRT_KR
( _AtlStackLinkP stack_link,
  _atl_T_time now )
#else
( stack_link, now )
  _AtlStackLinkP stack_link;
  _atl_T_time now;
#endif
{
  _atq_TP thedata;
  _AtlStackLinkP up;
  RTRT_U_INT32 i;

#if ATL_CHECK_STACK
  if ( _ATL_STACK_CHECK_MAGIC(stack_link) )
  {
    ATL_ERROR ( 'P',"_atq_end_now(): runtime analysis stack data is corrupted" );
    return 0;
  }
#endif /* ATL_CHECK_STACK */
  i=_ATQ_INDEX_FROM_MESSAGE(stack_link->Info.MessageTsf);
  RTRT_MT_CRITICAL_VAR_LOCK(atq_critical)
  stack_link->LocalAmount += now - stack_link->PeriodStart;
  stack_link->TotalAmount += now - stack_link->PeriodStart;
  thedata = (_atq_TP)(stack_link->AtqData);
  thedata->Local[i] += stack_link->LocalAmount;
  /* Is it a recursive call ? */
  for ( up = stack_link->Up;
        up &&
#if ATL_CHECK_STACK
        ! _ATL_STACK_CHECK_MAGIC(up) &&
#endif
        ( up->Info.Source != stack_link->Info.Source ||
          up->Info.MessageTsf != stack_link->Info.MessageTsf );
        up = up->Up );
  if ( ! up )
  {
    thedata->Total[i] += stack_link->TotalAmount;
#if ATQ_COMPUTE_MINMAX
    if (thedata->Tmax[i] < stack_link->TotalAmount)
        thedata->Tmax[i] = stack_link->TotalAmount;
    if((thedata->Tmin[i] > stack_link->TotalAmount) || (thedata->Tmin[i]==0))
        thedata->Tmin[i] = stack_link->TotalAmount;
#endif
  }
  if ( stack_link->Up
#if ATL_CHECK_STACK
       && ! _ATL_STACK_CHECK_MAGIC(stack_link->Up)
#endif
                      )
  {
    stack_link->Up->TotalAmount += stack_link->TotalAmount;
    stack_link->Up->PeriodStart = now;
  }
  RTRT_MT_CRITICAL_VAR_UNLOCK(atq_critical)
  /* Must return 0 */
  return 0;
}

/*******************************************************************************
*  Service: _atq_stop
*******************************************************************************/
RTRT_STATICFCT void _atq_stop
#if !RTRT_KR
( void )
#else
( )
#endif
{
  _atl_T_time now;
  _AtlStackLinkP current;
#if ATL_MULTI_THREADS
  /* RTRT_U_INT32 i; */
  RTRT_THIDX_T thread_index = _atl_threads_get_index ();
#endif /* ATL_MULTI_THREADS */

  now = ATL_CPU_TIME ( 0 );

#if ATQ_COMPUTE_OVERHEAD
  now -= _atq_overhead_out;
#endif /* ATQ_COMPUTE_OVERHEAD */

#if 0 /* ATL_MULTI_THREADS */
  RTRT_MT_CRITICAL_VAR_LOCK(atl_stack_critical)
  for ( i = 0;
        i <= ATL_THREADS_MAX;
        i++ )
  {
    if ( RTRT_MT_THREAD_ISALIVE(_atl_threads_table_used[i]) )
    {
      for ( current = _atl_stack_table_current[i];
            current
#if ATL_CHECK_STACK
            && ! _ATL_STACK_CHECK_MAGIC(current)
#endif
            ;
            current = current->Up )
      {
	    if(current->AtqData)
	    {
#ifndef ATQ_CPU_CLOCK
			now = ATL_CPU_TIME ( _atl_threads_table_used[i]);
#endif
		if(now)
	       _atq_end_now ( current, now );
	    }
      }
    }
  }
  RTRT_MT_CRITICAL_VAR_UNLOCK(atl_stack_critical)
#else /* ATL_MULTI_THREADS */
  for ( current = _atl_stack_current_link;
        current
#if ATL_CHECK_STACK
            && ! _ATL_STACK_CHECK_MAGIC(current)
#endif
            ;
        current = current->Up )
  {
	if(current->AtqData)
	   _atq_end_now ( current, now );
  }
#endif /* ATL_MULTI_THREADS */
}

/*******************************************************************************
*  Service: _atq_dump
*******************************************************************************/
_ATL_C_GLOBALFCT void _atq_dump
#if !RTRT_KR
( int reset )
#else
( reset )
  int reset;
#endif
{
  _atq_TP thedata;
  RTRT_U_INT32 i;
#if RTRT_GET_DATE != RTRT_NONE
  char *date;
#endif /* RTRT_GET_DATE */

  _atq_stop ( );
  if ( reset & _ATCPQ_QTF)
  {

	  RTRT_MT_CRITICAL_VAR_LOCK(atl_line_mutex)

	  if ( ! _atq_header )
	  {
		_atq_header = 1;
		/* Writes heading codes */
		ATL_LINE_ADD_OPCODE_SPACE ( ATL_TQF_QU );
		atl_line_add_char ( RTRT_QUOT );
		atl_buffer_add (RTRT_UNIT_TYPE_NAME);
		atl_line_add_char ( RTRT_QUOT );
		atl_line_add_char ( ' ' );
		atl_line_add_char ( RTRT_QUOT );
		atl_buffer_add (ATL_NODE_NAME);
		atl_line_add_char ( RTRT_QUOT );
		atl_line_dump ( );
	  }

	  /* New Test */
	  ATL_LINE_ADD_OPCODE ( ATL_TQF_QT );
      _atl_obstools_test_name ( );
	  atl_line_dump ( );

	  ATL_LINE_ADD_OPCODE_SPACE ( ATL_TQF_QD );
#if RTRT_GET_DATE != RTRT_NONE
	  date = priv_date();
	  if ( date != (char *)0 )
	  {
		atl_line_add_char ( RTRT_QUOT );
		atl_line_add ( date );
		atl_line_add_char ( RTRT_QUOT );
	  }
	  else
#endif /* RTRT_GET_DATE */
	  {
		atl_line_add_char ( '0' );
	  }
	  atl_line_dump ( );

	  for ( thedata = _atq_first;
			thedata;
			thedata = thedata->Next )
	  {
		ATL_LINE_ADD_OPCODE_SPACE ( ATL_TQF_QS );

#if ATL_DYN_TABLE
		atl_line_inc ( unsigned_reverse_image_16 ( thedata->Key,
												   atl_line_cur ) );
		atl_line_add_char ( ' ' );
		atl_line_inc ( unsigned_reverse_image_16 ( thedata->CheckSum,
												   atl_line_cur ) );
#else /* ATL_DYN_TABLE */
		atl_line_inc ( unsigned_reverse_image_16 ( thedata->Source->Key,
												   atl_line_cur ) );
		atl_line_add_char ( ' ' );
		atl_line_inc ( unsigned_reverse_image_16 ( thedata->Source->CheckSum,
												   atl_line_cur ) );
#endif /* ATL_DYN_TABLE */

		atl_line_dump ( );
		for ( i = 0; i < thedata->Routines; i++ )
		{
		  if ( thedata->Calls[i] )
		  {
			ATL_LINE_ADD_OPCODE_SPACE ( ATL_TQF_QF );
			atl_line_inc ( unsigned_reverse_image_16 (
							 _ATQ_MESSAGE_FROM_INDEX(i),
							 atl_line_cur ) );
			atl_line_add_char ( ' ' );
			atl_line_inc ( unsigned_reverse_image_16 (
							 thedata->Calls[i],
							 atl_line_cur ) );
			atl_line_add_char ( ' ' );
#if RTRT_UNIT_TYPE_SIZE <= 4
			atl_line_inc ( unsigned_reverse_image_16 (
							 thedata->Local[i],
							 atl_line_cur ) );
#elif RTRT_UNIT_TYPE_SIZE <= 8
			if ( thedata->Local[i]>>32 )
			{
			  atl_line_inc ( unsigned_reverse_image_16 (
							   (RTRT_U_INT32)(thedata->Local[i]>>32),
							   atl_line_cur ) );
			  atl_line_add_char ( ',' );
			}
			atl_line_inc ( unsigned_reverse_image_16 (
							 (RTRT_U_INT32)(thedata->Local[i]&0xFFFFFFFFUL),
							 atl_line_cur ) );
#endif
			atl_line_add_char ( ' ' );
#if RTRT_UNIT_TYPE_SIZE <= 4
			atl_line_inc ( unsigned_reverse_image_16 (
							 thedata->Total[i],
							 atl_line_cur ) );
#elif RTRT_UNIT_TYPE_SIZE <= 8
			if ( thedata->Total[i]>>32 )
			{
			  atl_line_inc ( unsigned_reverse_image_16 (
							   (RTRT_U_INT32)(thedata->Total[i]>>32),
							   atl_line_cur ) );
			  atl_line_add_char ( ',' );
			}
			atl_line_inc ( unsigned_reverse_image_16 (
							 (RTRT_U_INT32)(thedata->Total[i]&0xFFFFFFFFUL),
							 atl_line_cur ) );
#endif
#if ATQ_COMPUTE_MINMAX
			atl_line_add_char ( ' ' );
#if RTRT_UNIT_TYPE_SIZE <= 4
			atl_line_inc ( unsigned_reverse_image_16 (
							 thedata->Tmin[i],
							 atl_line_cur ) );
#elif RTRT_UNIT_TYPE_SIZE <= 8
			if ( thedata->Tmin[i]>>32 )
			{
			  atl_line_inc ( unsigned_reverse_image_16 (
							   (RTRT_U_INT32)(thedata->Tmin[i]>>32),
							   atl_line_cur ) );
			  atl_line_add_char ( ',' );
			}
			atl_line_inc ( unsigned_reverse_image_16 (
							 (RTRT_U_INT32)(thedata->Tmin[i]&0xFFFFFFFFUL),
							 atl_line_cur ) );
#endif
			atl_line_add_char ( ' ' );
#if RTRT_UNIT_TYPE_SIZE <= 4
			atl_line_inc ( unsigned_reverse_image_16 (
							 thedata->Tmax[i],
							 atl_line_cur ) );
#elif RTRT_UNIT_TYPE_SIZE <= 8
			if ( thedata->Tmax[i]>>32 )
			{
			  atl_line_inc ( unsigned_reverse_image_16 (
							   (RTRT_U_INT32)(thedata->Tmax[i]>>32),
							   atl_line_cur ) );
			  atl_line_add_char ( ',' );
			}
			atl_line_inc ( unsigned_reverse_image_16 (
							 (RTRT_U_INT32)(thedata->Tmax[i]&0xFFFFFFFFUL),
							 atl_line_cur ) );
#endif
#endif /* ATQ_COMPUTE_MINMAX */
			atl_line_dump ( );
			if ( reset & _ATCPQ_RESET_QTF)
			{
			  thedata->Calls[i] = (RTRT_U_INT32)0;
			  thedata->Local[i] = (_atl_T_time)0;
			  thedata->Total[i] = (_atl_T_time)0;
			}
		  }
		}
	  }
	  RTRT_MT_CRITICAL_VAR_UNLOCK(atl_line_mutex)
  }
  else if( reset & _ATCPQ_RESET_QTF)
  {
	  for ( thedata = _atq_first;
			thedata;
			thedata = thedata->Next )
	  {
		for ( i = 0; i < thedata->Routines; i++ )
		{
			{
			  thedata->Calls[i] = (RTRT_U_INT32)0;
			  thedata->Local[i] = (_atl_T_time)0;
			  thedata->Total[i] = (_atl_T_time)0;
			}
	    }
	  }

  }
}
/*******************************************************************************
*  remove data for a source Family
*******************************************************************************/
#ifdef ATL_T_FAMILY
void _atq_remove_family
#if !RTRT_KR
(ATL_T_FAMILY family)
#else
(family)
ATL_T_FAMILY family;
#endif
{
#if !ATL_DYN_TABLE

  _atq_TP thedata,dataPrev;
 	RTRT_MT_CRITICAL_VAR_LOCK(atq_critical)
   thedata = _atq_first;
    while((thedata) && (thedata->Source->Family == family))
    {
		thedata = thedata->Next;
	}
	_atq_first = thedata;
    dataPrev = thedata;
	while(thedata)
	{
       if(thedata->Source->Family == family)
       {
	 	 dataPrev->Next = thedata->Next;
	   }
	   else
	   {
		   dataPrev = thedata;
	   }
	   thedata = thedata->Next;
	}
	RTRT_MT_CRITICAL_VAR_UNLOCK(atq_critical)
#endif /* ATL_DYN_TABLE */
}
#endif

/*******************************************************************************
*  Dump Driver
*******************************************************************************/
#if ATQ_DUMP_DRIVER

enum {
        _ATQ_DD_HEADER_UNIT,
        _ATQ_DD_HEADER_TEST,
        _ATQ_DD_FILE_INFO,
        _ATQ_DD_LOOP_ROUTINE
};

RTRT_GLOBALVAR int _atq_dd_step, _atq_dd_reset;
RTRT_GLOBALVAR RTRT_U_INT32 _atq_dd_index;
RTRT_GLOBALVAR _atq_TP _atq_dd_current;

/*******************************************************************************
*  Service: _atq_dd_open
*******************************************************************************/
_ATL_C_GLOBALFCT void _atq_dd_open
#if !RTRT_KR
( int dump_header,
  int reset )
#else
( dump_header, reset )
  int dump_header;
  int reset;
#endif
{
  _atq_stop ( );
  if ( dump_header )
  { /* Dump of QU (Unit) and QT (Test) lines */
    _atq_dd_step = _ATQ_DD_HEADER_UNIT;
  }
  else
  {
    _atq_dd_step = _ATQ_DD_FILE_INFO;
  }
  /* reset value of 0 is deprecated: 1 is assumed */
  reset = 1;
  _atq_dd_reset = reset;
  _atq_dd_index = 0;
  _atq_dd_current = _atq_first;
}

/*******************************************************************************
*  Service: _atq_dd_routine
*******************************************************************************/
RTRT_STATICFCT void _atq_dd_routine
#if !RTRT_KR
( RTRT_U_INT32 i )
#else
( n )
  RTRT_U_INT32 i;
#endif
{
  if ( _atq_dd_current->Calls[i] )
  {
    _atl_obstools_str_add ( ATL_TQF_QF );
    _atl_obstools_str_add_char ( ' ' );
    _atl_obstools_str_inc ( unsigned_reverse_image_16 (
                             _ATQ_MESSAGE_FROM_INDEX(i),
                             _atl_obstools_str_cur ) );
    _atl_obstools_str_add_char ( ' ' );
    _atl_obstools_str_inc ( unsigned_reverse_image_16 (
                             _atq_dd_current->Calls[i],
                             _atl_obstools_str_cur ) );
    _atl_obstools_str_add_char ( ' ' );
#if RTRT_UNIT_TYPE_SIZE <= 4
    _atl_obstools_str_inc ( unsigned_reverse_image_16 (
                             _atq_dd_current->Local[i],
                             _atl_obstools_str_cur ) );
#elif RTRT_UNIT_TYPE_SIZE <= 8
    if ( _atq_dd_current->Local[i]>>32 )
    {
      _atl_obstools_str_inc ( unsigned_reverse_image_16 (
                               (RTRT_U_INT32)(_atq_dd_current->Local[i]>>32),
                               _atl_obstools_str_cur ) );
      _atl_obstools_str_add_char ( ',' );
    }
    _atl_obstools_str_inc ( unsigned_reverse_image_16 (
                             (RTRT_U_INT32)(_atq_dd_current->Local[i]&0xFFFFFFFFUL),
                             _atl_obstools_str_cur ) );
#endif
    _atl_obstools_str_add_char ( ' ' );
#if RTRT_UNIT_TYPE_SIZE <= 4
    _atl_obstools_str_inc ( unsigned_reverse_image_16 (
                     _atq_dd_current->Total[i],
                     _atl_obstools_str_cur ) );
#elif RTRT_UNIT_TYPE_SIZE <= 8
    if ( _atq_dd_current->Total[i]>>32 )
    {
      _atl_obstools_str_inc ( unsigned_reverse_image_16 (
                               (RTRT_U_INT32)(_atq_dd_current->Total[i]>>32),
                               _atl_obstools_str_cur ) );
      _atl_obstools_str_add_char ( ',' );
    }
    _atl_obstools_str_inc ( unsigned_reverse_image_16 (
                           (RTRT_U_INT32)(_atq_dd_current->Total[i]&0xFFFFFFFFUL),
                             _atl_obstools_str_cur ) );
#endif
#if ATQ_COMPUTE_MINMAX
			_atl_obstools_str_add_char ( ' ' );
#if RTRT_UNIT_TYPE_SIZE <= 4
			_atl_obstools_str_inc ( unsigned_reverse_image_16 (
							 _atq_dd_current->Tmin[i],
							 _atl_obstools_str_cur ) );
#elif RTRT_UNIT_TYPE_SIZE <= 8
			if ( _atq_dd_current->Tmin[i]>>32 )
			{
			  _atl_obstools_str_inc ( unsigned_reverse_image_16 (
							   (RTRT_U_INT32)(_atq_dd_current->Tmin[i]>>32),
							   _atl_obstools_str_cur ) );
			  _atl_obstools_str_add_char ( ',' );
			}
			_atl_obstools_str_inc ( unsigned_reverse_image_16 (
							 (RTRT_U_INT32)(_atq_dd_current->Tmin[i]&0xFFFFFFFFUL),
							 _atl_obstools_str_cur ) );
#endif
			_atl_obstools_str_add_char ( ' ' );
#if RTRT_UNIT_TYPE_SIZE <= 4
			_atl_obstools_str_inc ( unsigned_reverse_image_16 (
							 _atq_dd_current->Tmax[i],
							 _atl_obstools_str_cur ) );
#elif RTRT_UNIT_TYPE_SIZE <= 8
			if ( _atq_dd_current->Tmax[i]>>32 )
			{
			  _atl_obstools_str_inc ( unsigned_reverse_image_16 (
							   (RTRT_U_INT32)(_atq_dd_current->Tmax[i]>>32),
							   _atl_obstools_str_cur ) );
			  _atl_obstools_str_add_char ( ',' );
			}
			_atl_obstools_str_inc ( unsigned_reverse_image_16 (
							 (RTRT_U_INT32)(_atq_dd_current->Tmax[i]&0xFFFFFFFFUL),
							 _atl_obstools_str_cur ) );
#endif
#endif /* ATQ_COMPUTE_MINMAX */
    if ( _atq_dd_reset )
    {
      _atq_dd_current->Calls[i] = (RTRT_U_INT32)0;
      _atq_dd_current->Local[i] = (_atl_T_time)0;
      _atq_dd_current->Total[i] = (_atl_T_time)0;
    }
  }
}

/*******************************************************************************
*  Service: _atq_dd_read_internal
* return -1 when dump finished, or length of the bufer to dump (can be 0)
*******************************************************************************/
RTRT_STATICFCT int _atq_dd_read_internal
#if !RTRT_KR
( char *dump_str )
#else
( dump_str )
  char *dump_str;
#endif
{
  /* Let's go for one trace line */
  _atl_obstools_str_init ( dump_str )

  switch ( _atq_dd_step )
  {
    case _ATQ_DD_HEADER_UNIT :
      _atq_dd_step++;
      _atl_obstools_str_add ( ATL_TQF_QU );
      _atl_obstools_str_add_char ( ' ' );
      _atl_obstools_str_add_char ( RTRT_QUOT );
      _atl_obstools_str_inc ( string_image ( RTRT_UNIT_TYPE_NAME,
                                            _atl_obstools_str_cur,
                                            48 ) );
      _atl_obstools_str_add_char ( RTRT_QUOT );
      _atl_obstools_str_add_char ( ' ' );
      _atl_obstools_str_add_char ( RTRT_QUOT );
      _atl_obstools_str_inc ( string_image ( ATL_NODE_NAME,
                                            _atl_obstools_str_cur,
                                            48 ) );
      _atl_obstools_str_add_char ( RTRT_QUOT );
      break;

    case _ATQ_DD_HEADER_TEST :
      _atq_dd_step++;
      _atl_obstools_str_add ( ATL_TQF_QT );
      _atl_obstools_test_name ( );
      break;

    case _ATQ_DD_FILE_INFO :
      if ( _atq_dd_current == (_atq_TP)0 )
      {
        return -1;
      }
      _atq_dd_step++;
      _atl_obstools_str_add ( ATL_TQF_QS );
      _atl_obstools_str_add_char ( ' ' );
      _atl_obstools_str_inc ( unsigned_reverse_image_16 (
                               _atq_dd_current->Source->Key,
                               _atl_obstools_str_cur ) );
      _atl_obstools_str_add_char ( ' ' );
      _atl_obstools_str_inc ( unsigned_reverse_image_16 (
                               _atq_dd_current->Source->CheckSum,
                               _atl_obstools_str_cur ) );
      break;

    case _ATQ_DD_LOOP_ROUTINE :
      if ( _atq_dd_index < _atq_dd_current->Routines )
      {
        _atq_dd_routine ( _atq_dd_index );
        _atq_dd_index++;
        break;
      }
      else
      {
        _atq_dd_current = _atq_dd_current->Next;
        if ( _atq_dd_current == (_atq_TP)0 )
        {
          return -1;
        }
        _atq_dd_step = _ATQ_DD_FILE_INFO;
        _atq_dd_index = 0;
      }
  }

  return (int)(_atl_obstools_str_cur - dump_str);
}

/*******************************************************************************
*  Service: _atq_dd_read
*******************************************************************************/
_ATL_C_GLOBALFCT int _atq_dd_read
#if !RTRT_KR
( char *dump_str )
#else
( dump_str )
  char *dump_str;
#endif
{
  int ret; /* length of the string */

  while ( ret = _atq_dd_read_internal ( dump_str ) == 0 );
  if(ret < 0) ret=0;
  /* ret==0: finished,
   * ret!=0: needs more calls */
  return ret;
}

#endif /* ATQ_DUMP_DRIVER */

/*******************************************************************************
*  Service: _atq_start
*******************************************************************************/
_ATL_C_GLOBALFCT void _atq_start
#if !RTRT_KR
( _AtlStackLinkP stack_link )
#else
( stack_link )
  _AtlStackLinkP stack_link;
#endif
{
  _atl_T_time now;
  _atq_TP thedata;

  ATQ_THREAD_FILTERING
#if ATL_CHECK_STACK
  if ( _ATL_STACK_CHECK_MAGIC(stack_link) )
  {
    ATL_ERROR ( 'Q',"_atq_start(): runtime analysis stack data is corrupted" );
    return;
  }
#endif /* ATL_CHECK_STACK */
  now = ATL_CPU_TIME ( 0 );
  RTRT_MT_CRITICAL_VAR_LOCK(atq_critical)
#if ATQ_COMPUTE_OVERHEAD
  now -= _atq_overhead_in;
#endif /* ATQ_COMPUTE_OVERHEAD */
  thedata = (_atq_TP)(stack_link->AtqData);
#if ATL_DYN_TABLE
  if ( ! thedata->Key )
  {
    RTRT_U_INT32 i;

    thedata->Key = stack_link->Info.Source->Key;
    thedata->CheckSum = stack_link->Info.Source->CheckSum;
#else /* ATL_DYN_TABLE */
  if ( ! thedata->Source )
  {
    RTRT_U_INT32 i;

    thedata->Source = stack_link->Info.Source;
#endif /* ATL_DYN_TABLE */
    for ( i = 0; i < thedata->Routines; i++ )
    {
      thedata->Calls[i] = (RTRT_U_INT32)0;
      thedata->Local[i] = (_atl_T_time)0;
      thedata->Total[i] = (_atl_T_time)0;
#if ATQ_COMPUTE_MINMAX
      thedata->Tmax[i] = (_atl_T_time)0;
      thedata->Tmin[i] = (_atl_T_time)0;
#endif
    }
    thedata->Next = _atq_first;
    _atq_first = thedata;
  }
  thedata->Calls[_ATQ_INDEX_FROM_MESSAGE(stack_link->Info.MessageTsf)]++;
  stack_link->LocalAmount = (_atl_T_time)0;
  stack_link->TotalAmount = (_atl_T_time)0;
  if ( stack_link->Up )
  {
    stack_link->Up->LocalAmount += now - stack_link->Up->PeriodStart
#if ATQ_COMPUTE_OVERHEAD
                                       - _atq_overhead_call
#endif /* ATQ_COMPUTE_OVERHEAD */
                                                                    ;
    stack_link->Up->TotalAmount += now - stack_link->Up->PeriodStart
#if ATQ_COMPUTE_OVERHEAD
                                       - _atq_overhead_call
#endif /* ATQ_COMPUTE_OVERHEAD */
                                                                    ;
  }
  stack_link->PeriodStart = ATL_CPU_TIME ( 0 );
  RTRT_MT_CRITICAL_VAR_UNLOCK(atq_critical)
}

/*******************************************************************************
*  Service: _atq_end
*******************************************************************************/
_ATL_C_GLOBALFCT int _atq_end
#if !RTRT_KR
( _AtlStackLinkP stack_link )
#else
( stack_link )
  _AtlStackLinkP stack_link;
#endif
{
  _atl_T_time now;
  _atq_TP thedata;
  _AtlStackLinkP up;
  RTRT_U_INT32 i;

  ATQ_THREAD_FILTERING
#if ATL_CHECK_STACK
  if ( _ATL_STACK_CHECK_MAGIC(stack_link) )
  {
    ATL_ERROR ('R', "_atq_end(): runtime analysis stack data is corrupted" );
    return 0;
  }
#endif /* ATL_CHECK_STACK */
  i=_ATQ_INDEX_FROM_MESSAGE(stack_link->Info.MessageTsf);
  RTRT_MT_CRITICAL_VAR_LOCK(atq_critical)
  now = ATL_CPU_TIME ( 0 );
#if ATQ_COMPUTE_OVERHEAD
  now -= _atq_overhead_out;
#endif /* ATQ_COMPUTE_OVERHEAD */
  stack_link->LocalAmount += now - stack_link->PeriodStart;
  stack_link->TotalAmount += now - stack_link->PeriodStart;
  thedata = (_atq_TP)(stack_link->AtqData);
  thedata->Local[i] += stack_link->LocalAmount;
  /* Is it a recursive call ? */
  for ( up = stack_link->Up;
        up &&
#if ATL_CHECK_STACK
        ! _ATL_STACK_CHECK_MAGIC(up) &&
#endif
        ( up->Info.Source != stack_link->Info.Source ||
          up->Info.MessageTsf != stack_link->Info.MessageTsf );
        up = up->Up );
  if ( ! up )
  {
    thedata->Total[i] += stack_link->TotalAmount;
#if ATQ_COMPUTE_MINMAX
    if (thedata->Tmax[i] < stack_link->TotalAmount)
        thedata->Tmax[i] = stack_link->TotalAmount;
    if((thedata->Tmin[i] > stack_link->TotalAmount) || (thedata->Tmin[i]==0))
	thedata->Tmin[i] = stack_link->TotalAmount;
#endif
  }
  if ( stack_link->Up
#if ATL_CHECK_STACK
       && ! _ATL_STACK_CHECK_MAGIC(stack_link->Up)
#endif
                      )
  {
    stack_link->Up->TotalAmount += stack_link->TotalAmount;
    stack_link->Up->PeriodStart = ATL_CPU_TIME ( 0 );
  }
  RTRT_MT_CRITICAL_VAR_UNLOCK(atq_critical)
  /* Must return 0 */
  return 0;
}

#if ATQ_COMPUTE_OVERHEAD

/*#define _ATQ_OVERHEAD_DEBUG*/
#define ATQ_OVERHEAD_ITERATIONS 32

/*******************************************************************************
*  Service: _atq_compute_overhead_in_func
*  Goal: simulates function incoming
*******************************************************************************/
_ATL_C_GLOBALFCT _atl_T_time _atq_compute_overhead_in_func
#if !RTRT_KR
( _AtlStackLinkP stack_link )
#else
( stack_link )
  _AtlStackLinkP stack_link;
#endif
{
  /* Same starting code as in _atq_start() */
  _atl_T_time now;

  now = ATL_CPU_TIME (0);
  /* End of similar code */

  return now;
}

/*******************************************************************************
*  Service: _atq_compute_overhead_in
*  Goal: computes overhead due to instrumentation
*******************************************************************************/
_ATL_C_GLOBALFCT void _atq_compute_overhead_in
#if !RTRT_KR
( void )
#else
( )
#endif
{
  _atl_T_time before, after, spent = (_atl_T_time)0;
  int i;
  _AtlStackLink stack_link;

  for ( i = 0; i < ATQ_OVERHEAD_ITERATIONS; i++ )
  {
    before = ATL_CPU_TIME (0);
    after = _atq_compute_overhead_in_func ( &stack_link );
    spent += after - before;
  }
#ifdef _ATQ_OVERHEAD_DEBUG
fprintf(stderr,"spent=%llu\n",spent);
#endif
  _atq_overhead_in = spent / ATQ_OVERHEAD_ITERATIONS;
#ifdef _ATQ_OVERHEAD_DEBUG
fprintf(stderr,"overhead_in=%llu\n",_atq_overhead_in);
#endif
}

/*******************************************************************************
*  Service: _atq_compute_overhead_out_func
*  Goal: simulates function return
*******************************************************************************/
_ATL_C_GLOBALFCT _atl_T_time _atq_compute_overhead_out_func
#if !RTRT_KR
( _AtlStackLinkP stack_link )
#else
( stack_link )
  _AtlStackLinkP stack_link;
#endif
{
  /* Same starting code as in _atq_end() */
  _atl_T_time now;
  _atq_TP thedata;

  now = ATL_CPU_TIME (0);
  /* End of similar code */

  /* To avoid warning on unused local variable data */
  thedata = (_atq_TP)0;
  return now/* + (_atl_T_time)data */;
}

/*******************************************************************************
*  Service: _atq_compute_overhead_out
*  Goal: computes overhead due to instrumentation
*******************************************************************************/
_ATL_C_GLOBALFCT void _atq_compute_overhead_out
#if !RTRT_KR
( void )
#else
( )
#endif
{
  _atl_T_time before, after, spent = (_atl_T_time)0;
  int i;
  _AtlStackLink stack_link;

  for ( i = 0; i < ATQ_OVERHEAD_ITERATIONS; i++ )
  {
    before = ATL_CPU_TIME (0);
    after = _atq_compute_overhead_out_func ( &stack_link );
    spent += after - before;
  }
#ifdef _ATQ_OVERHEAD_DEBUG
fprintf(stderr,"spent=%llu\n",spent);
#endif
  _atq_overhead_out = spent / ATQ_OVERHEAD_ITERATIONS;
#ifdef _ATQ_OVERHEAD_DEBUG
fprintf(stderr,"overhead_out=%llu\n",_atq_overhead_out);
#endif
}

/*******************************************************************************
*  Service: _atq_compute_overhead_call_func
*  Goal: simulates function call
*******************************************************************************/
_ATL_C_GLOBALFCT _atl_T_time _atq_compute_overhead_call_func
#if !RTRT_KR
( void )
#else
( )
#endif
{
  /* Same starting code as instrumented */
  _AtlStackLink stack_link;
  _AtlStackSource  source_struct0 = { _ATL_STACK_SN_ZERO 0, 0 };
  _atq_T data0 = { (_AtlStackSourceP)0, (RTRT_U_INT32)0,
                   (RTRT_U_INT32 *)0, (_atl_T_time *)0, (_atl_T_time *)0,
                   (_atq_T *)0 };

  stack_link.Up = (_AtlStackLinkP)0;
  stack_link.Down = (_AtlStackLinkP)0;
  stack_link.Info.Source = &source_struct0;
  stack_link.Info.MessageTsf = 2;
  _atl_stack_link_push ( &stack_link );
  stack_link.AtqData = (void *)(&data0);
  /* Code added by another component (ATx...) is not taken into account */
  /* here is a call to _atq_start() yet computed in _atq_overhead_in */
  /* End of similar code */

  { /* Declaration is here not to disturb measure */
    _atl_T_time now;

    now = ATL_CPU_TIME (0);
    _atl_stack_link_pop ( &stack_link );
    return now;
  }
}

/*******************************************************************************
*  Service: _atq_compute_overhead_call
*  Goal: computes overhead due to instrumentation
*******************************************************************************/
_ATL_C_GLOBALFCT void _atq_compute_overhead_call
#if !RTRT_KR
( void )
#else
( )
#endif
{
  _atl_T_time before, after, spent = (_atl_T_time)0;
  int i;

  for ( i = 0; i < ATQ_OVERHEAD_ITERATIONS; i++ )
  {
    before = ATL_CPU_TIME (0);
    after = _atq_compute_overhead_call_func ( );
    spent += after - before;
  }
#ifdef _ATQ_OVERHEAD_DEBUG
fprintf(stderr,"spent=%llu\n",spent);
#endif
  _atq_overhead_call = spent / ATQ_OVERHEAD_ITERATIONS;
#ifdef _ATQ_OVERHEAD_DEBUG
fprintf(stderr,"overhead_call=%llu\n",_atq_overhead_call);
#endif
}

#endif /* ATQ_COMPUTE_OVERHEAD */

/*******************************************************************************
*  Service: atq_early_init
*  Goal: initializes global variables
*******************************************************************************/
_ATL_C_GLOBALFCT void atq_early_init
#if !RTRT_KR
( void )
#else
( )
#endif
{
  RTRT_MT_CRITICAL_VAR_INIT(atq_critical)
#if ! RTRT_RAM_CLEAN
  _atq_first = (_atq_TP)0;
  _atq_header = 0;
#endif
#if ATQ_COMPUTE_OVERHEAD
  /* One and only one thread executes this code (the initializer one) */
  _atq_compute_overhead_in ( );
  _atq_compute_overhead_out ( );
  _atq_compute_overhead_call ( );
#endif /* ATQ_COMPUTE_OVERHEAD */
}

#ifdef _ATL_EXIT_PENDING
/*******************************************************************************
*  Service: atq_exit_pending
*  Goal: called when it is about time to exit
*******************************************************************************/
_ATL_C_GLOBALFCT void atq_exit_pending
#if !RTRT_KR
( void )
#else
( )
#endif
{
#if ! USE_ATU && ! USE_ATO && ! USE_ATV
  _atq_dump ( _ATCPQ_ALL );
#endif /* USE_ATU && USE_ATO */
}
#else
/*******************************************************************************
*  Service: atq_late_exit
*  Goal: called when it is time to quit
*******************************************************************************/
_ATL_C_GLOBALFCT void atq_late_exit
#if !RTRT_KR
( void )
#else
( )
#endif
{
#if ! USE_ATU && ! USE_ATO && ! USE_ATV
  _atq_dump ( _ATCPQ_ALL );
#endif /* USE_ATU && USE_ATO */
  RTRT_MT_CRITICAL_VAR_DESTROY(atq_critical)
}
#endif

