

/*
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2011 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */


/*******************************************************************************
* $Descr: System Testing header file (included in TP.h) $
*******************************************************************************/
/* Multiple inclusions guard: */
#ifndef ATS_H_INCLUDED
#define ATS_H_INCLUDED

#ifdef RTRT_INT64
/*! Largest representable signed integer */
/* see "image.c" ==>  _ATL_LARGEST_INT */
#define _ATS_LARGEST_INT RTRT_INT64
#else
#define _ATS_LARGEST_INT RTRT_INT32
#endif

#ifdef RTRT_U_INT64
/*! Largest representable unsigned integer */
/* see "image.c" ==>  _ATL_LARGEST_U_INT */
#define _ATS_LARGEST_U_INT RTRT_U_INT64
#else
#define _ATS_LARGEST_U_INT RTRT_U_INT32
#endif


#if RTRT_CPLUSPLUS_LANGUAGE == 0

#define ATS_CTXP ATL_PT_RUNCTX ctx,
#define ATS_CTXP1 ATL_PT_RUNCTX ctx
#define CEXT extern ATL_TP_PROTO_PREFIX
#define CSTATIC RTRT_STATICFCT
#define CYTSTK(s) s

#if !RTRT_KR

/* Macros definitions for ANSI C prototyping. */
#define ATL_CONST const
#define ATL_PP(ARGS) ARGS
#define ATL_DP(ARGS)
#define ATL_BP (
#define ATL_SP ,
#define ATL_EP )
#define ATL_VOID (void)
#define ATL_NO_PARAM (void)

#else /* !RTRT_KR */

/* Macros definitions for Kernighan & Ritchie C prototyping. */
#define ATL_CONST
#define ATL_PP(ARGS) ()
#define ATL_DP(ARGS) ARGS
#define ATL_BP
#define ATL_SP ;
#define ATL_EP ;
#define ATL_VOID
#define ATL_NO_PARAM

#endif /* !RTRT_KR */

#else

#define ATS_CTXP
#define ATS_CTXP1
#define CEXT ATL_TP_PROTO_PREFIX
#define CSTATIC
#define ATL_PP(ARGS) ARGS
#define CYTSTK(s) RTRT_CONCAT_MACRO(atl_ctxCYTctx_stack_,s)[RTST_CTX_STACK_SIZE]

#define ATL_CONST const

#endif /* RTRT_CPLUSPLUS_LANGUAGE == 0 */

/* Translation macros from previous ATS `runtime' to new TDP design. */
#define ATL_K_ID_SIZE RTST_ID_SIZE
#define atl_sleep(t) priv_sleep(t)
#ifndef RTRT_OWNRDV
#define atl_rdv(s) priv_rdv(s)
#endif
#define atl_assign_union(dummy,a,b) priv_memcpy(&(a),&(b),sizeof(a))
#define atl_malloc(s) priv_malloc(s)
#define atl_free(p) priv_free(p)
#define atl_realloc(p,s) priv_realloc(p,s)
#define atl_thr_exit(i) priv_thread_exit(i)
#define atl_memcpy priv_memcpy
#define ATL_T_DATE RTRT_CLOCK_TYPE
#define ATS_IMAGE_SIZE RTST_IMAGE_SIZE
#define ATS_EXIT(c) ats_exit(c)

/*! Maximum number of filters */
#define ATS_FILTERS_MAX 26

/*! Taille de la pile des cleanups */
#define ATS_CLEANUP_STACK_SIZE 50

/* Definitions for specific constants (should they be customizable ?) */
#define ATL_NOEVT 0
#define ATL_EVTOK 1
#define ATL_EVTKO 2
#define NIL   ((void *) 0)
#define NONIL ((void *) -1)

/* Declarations for elementary types (formerly customizable) */
typedef int ATL_T_INT;
typedef int * ATL_PT_INT;
typedef unsigned int ATL_T_UINT;
typedef unsigned int * ATL_PT_UINT;
#ifndef ATL_T_LONG
typedef _ATS_LARGEST_INT ATL_T_LONG;
typedef _ATS_LARGEST_U_INT ATL_T_ULONG;
#endif
#ifndef ATL_T_ENUM
typedef int ATL_T_ENUM;
#endif
typedef char ATL_T_CAR;
typedef unsigned char ATL_T_UCAR;
typedef char * ATL_PT_CAR;
typedef float ATL_T_REAL;
#ifndef ATL_T_DOUBLE
typedef double ATL_T_DOUBLE;
#endif
typedef void ATL_T_VOID;
typedef void * ATL_PT_VOID;

/* ATL_T_UTIME is the biggest integer as default
 Its size can be re-defined into the TDP as:
 #define ATL_T_UTIME RTRT_INTxx like RTRT_UINT64 or RTRT_UINT32
 doing so the print format may be also redefined differently from
 #define ATL_TP_UTIME (char *)"%d"
*/
#ifndef ATL_T_UTIME
#define ATL_T_UTIME ATL_T_ULONG
#endif /*ATL_T_UTIME*/

#define ATL_P_BASE16 16
#define ATL_P_BASE10 10
#define ATL_P_BASE8   8
#define ATL_P_BASE2   2
/* Definition for miscellaneous types */
struct atl_s_runctx;

typedef int (*ATL_T_CLEANUP) (struct atl_s_runctx *ctx);

typedef ATL_T_INT  ATL_T_FILTERTABLE[ATS_FILTERS_MAX];

typedef RTRT_TIME_TYPE ATL_T_DURATION[RTST_DURATION_SIZE];

#define ATL_WTIME_TO_TIME (((RTST_WTIME_SECONDS*1000000L)+ \
  RTST_WTIME_MICRO_SECONDS)/RTRT_TIME_TYPE_STEP)

#ifndef ATL_PAUSE_TIME_TO_TIME
#define ATL_PAUSE_TIME_TO_TIME ATL_WTIME_TO_TIME
#endif

#define atl_get_date(d) priv_clock(d)

#ifndef ATL_T_THREAD_RETURN
#define ATL_T_THREAD_RETURN RTST_THREAD_RETURN
#endif /* ATL_T_THREAD_RETURN */

#if RTRT_CPLUSPLUS_LANGUAGE

#if RTRT_EXCEPTIONS
#define ATL_HANDLE_EXCEPTION
#define ATL_TRY   try{
#define ATL_END_TRY   }
#define ATL_CATCH }catch(...) {atl_runtime.atl_catch();}
#else
#define ATL_TRY
#define ATL_END_TRY
#define ATL_CATCH
#endif

#endif /* RTRT_CPLUSPLUS_LANGUAGE */

/*! Expected argument type in start routine of thread mode name of rio file
 * to generate list of filters set. */
#ifndef _ATSAPI_INCLUDED
typedef struct {
  char *atl_riofilename;
  char *atl_filters;
  char *atl_instance;
  int   atl_occid;
} ATL_T_ARG, *ATL_PT_ARG;
#endif /* _ATSAPI_INCLUDED */

typedef struct atl_timer {
  RTRT_CLOCK_TYPE timer;
} ATL_T_TIMER, *ATL_PT_TIMER;

/* Macros used by WAITTIL instruction. */

#define ATL_MATCHING_1(a)        (a.atl_state == ATL_EVTOK)
#define ATL_NOTMATCHING_1(a)     (a.atl_state == ATL_EVTKO)
#define ATL_MATCHED_1(a)         (a.atl_stateok)
#define ATL_NOTMATCHED_1(a)      (a.atl_stateko)

#define ATL_MATCHING_2(a,b)      ((a.atl_state == ATL_EVTOK) \
                               &&(((a.b) & ATL_EVTOK))==ATL_EVTOK)
#define ATL_NOTMATCHING_2(a,b)   ((a.atl_state == ATL_EVTKO) \
                               &&(((a.b) & ATL_EVTKO))==ATL_EVTKO)
#define ATL_MATCHED_2(a,b)       (((a.b) & ATL_EVTOK)==ATL_EVTOK)
#define ATL_NOTMATCHED_2(a,b)    (((a.b) & ATL_EVTKO)==ATL_EVTKO)

#define atl_gran ATL_WTIME_TO_TIME

/* Macros used in callback/procsend. */
#define _ATS_ERROR        1
#define ATL_NO_MESSAGE    10
#define ATL_MESSAGE_READ  11
#define ATL_MESSAGE_SENT  0 /* lower than 12 */

#define K_TRACECTX_MIN_PRIORITY	  1
#define K_TRACECTX_SCN_PRIORITY   10
#define K_TRACECTX_TIME_PRIORITY  20
#define K_TRACECTX_ERROR_PRIORITY 30
#define K_TRACECTX_CIRCULAR_PRIORITY 50
#define K_TRACECTX_MAX_PRIORITY   127

#if RTRT_CPLUSPLUS_LANGUAGE == 1
class ATL_C_RUNTIME
{
  public :
#endif /* RTRT_CPLUSPLUS_LANGUAGE == 1 */

#if RTRT_CPLUSPLUS_LANGUAGE == 0
/*! Circular buffer context. */
typedef struct atl_s_cyclictrace_ctxstack
{
#endif /* RTRT_CPLUSPLUS_LANGUAGE == 0 */

  char *CYTSTK(lastline);
  char *CYTSTK(currentline);
  char *CYTSTK(linetoerase);
  ATL_T_LONG CYTSTK(buffersizeleft);
  ATL_T_LONG CYTSTK(overpassindex);

#if RTRT_CPLUSPLUS_LANGUAGE == 0
} ATL_T_CYT_CTXSTACK, *ATL_PT_CYT_CTXSTACK;
#endif

#if RTRT_CPLUSPLUS_LANGUAGE == 0
/*! Dynamic buffer for the trace context. */
typedef struct atl_s_tracectx {
#endif

  /*! pointeur sur le buffer circulaire de trace */
  ATL_PT_CAR atl_ctxbuffertrace;
  /*! taille du buffer circulaire */
  ATL_T_INT atl_ctxbuffersize;
  /*! priorite de trace demandee error, scn, time */
  ATL_T_UCAR atl_ctxtracepriority;
  /*! pointeur de la ligne courante dans le buffer circulaire */
  ATL_PT_CAR atl_ctxcurrentline;
  /*! pointeur sur la prochaine ligne a effacer */
  ATL_PT_CAR atl_ctxlinetoerase;
  /*! pointeur sur la fin de la derniere ligne valide du buffer */
  ATL_PT_CAR atl_ctxlastline;
  /*! taille de la place utilisable dans le buffer */
  ATL_T_LONG atl_ctxbuffersizeleft;
  /*! priorite de la lettre rio en cours */
  ATL_T_UCAR atl_ctxcurrentcodepriority;
  /*! simulation d'une pile pour les blocs init */
  ATL_T_CAR  atl_ctxbinitstack[RTST_BLOCK_STACK_SIZE][RTST_BLOCK_NAME_SIZE];
  /*! simulation d'une pile pour les blocs term */
  ATL_T_CAR  atl_ctxbtermstack[RTST_BLOCK_STACK_SIZE][RTST_BLOCK_NAME_SIZE];
  /*! simulation d'une pile pour les blocs except */
  ATL_T_CAR  atl_ctxbexceptstack[RTST_BLOCK_STACK_SIZE][RTST_BLOCK_NAME_SIZE];
  /*! simulation d'une pile pour les scenarios */
  ATL_T_CAR  atl_ctxbscnstack[RTST_BLOCK_STACK_SIZE][RTST_BLOCK_NAME_SIZE];
  /*! simulation d'une pile pour les procs */
  ATL_T_CAR  atl_ctxbprocstack[RTST_BLOCK_STACK_SIZE][RTST_BLOCK_NAME_SIZE];

  /* ---- Additionnal Fields for the -TRACE=CIRCULAR option ---- */

  /*! contains the type of the current line */
  ATL_T_INT  atl_ctxCYTstatus;
#if RTRT_CPLUSPLUS_LANGUAGE == 0
  /*! stack storing contexts (last,current,...) of the circular buffer */
  ATL_T_CYT_CTXSTACK  atl_ctxCYTctx_stack[RTST_CTX_STACK_SIZE];
#endif
  /*! current index in the stack (described just herebefore) */
  ATL_T_INT  atl_ctxCYTctx_stackindex;

  /* Current fields are necessary for closing stack when a 'fb' follows
   * a 'ft' */

  /*! scenario stack associated to the 2 last 'ft' */
  ATL_T_CAR  atl_ctxCYTbscnstackclosure[2][RTST_BLOCK_STACK_SIZE][RTST_BLOCK_NAME_SIZE];
  /*! tracelevel indexes associated to the 2 last 'ft' */
  ATL_T_INT  atl_ctxCYTlasttracelevel_forclosure[2];
  /*! init blocks associated to the 2 last 'ft' */
  ATL_T_CAR  atl_ctxCYTstoredinit[2][RTST_BLOCK_NAME_SIZE];
  /*! term blocks associated to the 2 last 'ft' */
  ATL_T_CAR  atl_ctxCYTstoredterm[2][RTST_BLOCK_NAME_SIZE];
  /*! except blocks associated to the 2 last 'ft' */
  ATL_T_CAR  atl_ctxCYTstoredexcept[2][RTST_BLOCK_NAME_SIZE];

  /*! cyclic buffer has already overpassed */
  ATL_T_INT  atl_ctxCYTbufferoverpassed;
  /*! cyclic buffer current overpassing index */
  ATL_T_LONG atl_ctxCYToverpassindex;
  /*! status about encountered FLUSH_TRACE instructions */
  /*! (0: none, 1: one in current block, 2: one in older block) */
  ATL_T_INT  atl_ctxCYTft_status;
  /*! counter of flush_traces (0=0,1=1, 2=2..+oo) */
  ATL_T_INT  atl_ctxCYTftoccured;
  /*! fullbuffer has previously occured */
  ATL_T_INT  atl_ctxCYTfboccured;

  /* ---- End of Additionnal Fields for the -TRACE=CIRCULAR trace mode ---- */
  /*! pointeur sur la fonction d'ecriture dans le rio */
#if RTRT_CPLUSPLUS_LANGUAGE == 1
  void (ATL_C_RUNTIME::*atl_ctxgeneric_flush_line) ();
#elif RTRT_KR == 0
  void (*atl_ctxgeneric_flush_line) (struct atl_s_runctx *ctx);
#else
  void (*atl_ctxgeneric_flush_line) ();
#endif

#if RTRT_CPLUSPLUS_LANGUAGE == 0
} ATL_T_TRACECTX, *ATL_PT_TRACECTX;
#endif

#if RTRT_CPLUSPLUS_LANGUAGE == 0
/*! Run-time context */
typedef struct atl_s_runctx {
#endif

  /*! numero d'instance executee */
  ATL_T_INT      atl_ctxnuminstance;
  /*! index minimum d'une plage dans un var */
  ATL_T_INT      atl_ctxidxmin;

  /*! nom d'identifiant du testeur */
  ATL_T_CAR      atl_ctxtesternaming[255];
  /*! numero d'occurrence d'un testeur */
  ATL_T_INT      atl_ctxoccid;

  /*! dernier caractere dans bufout */
  ATL_T_INT      atl_ctxbufindex;
  /*! buffer de sortie vers le fichier rio */
  ATL_T_CAR      atl_ctxbufout[ATS_BUFFER_SIZE];
  /*! verrou de recursivite sur les cleanup */
  ATL_T_INT      atl_ctxlock;
  /*! liste des filtres positionnes */
  ATL_T_INT      atl_ctxfilter[ATS_FILTERS_MAX];
  /*! date de reception logique du message recu */
  ATL_T_UTIME    atl_ctxlogdate;
  /*! temps du waittil pour affichage */
  ATL_T_CAR      atl_ctxstrwaittil[20];
  /*! date before send */
  ATL_T_UTIME    atl_ctxdsend;
  /*! date before intersend */
  ATL_T_UTIME    atl_ctxdisend;
  /*! date before interrecv */
  ATL_T_UTIME    atl_ctxdirecv;
  /*! date before call function */
  ATL_T_UTIME    atl_ctxlbcf;
  /*! date before rendezvous function */
  ATL_T_UTIME    atl_ctxlbrdvf;
  /*! date debut le waittil */
  ATL_T_UTIME    atl_ctxdwait;
  /*! duree d'execution de l'instruction */
  ATL_T_UTIME    atl_ctxwtime;
  /*! timeout pass au waittil */
  ATL_T_UTIME    atl_ctxtimeout;
  /*! resultat derniere comparaison du VAR */
  ATL_T_INT      atl_ctxlastcmp;
  /*! valeur byte pour un VAR */
  ATL_T_CAR      atl_ctxvarbyte[ATS_IMAGE_SIZE];
  /*! derniere valeur recue dans un VAR */
  ATL_T_CAR      atl_ctxvarrec[ATS_BUFFER_SIZE];
  /*! derniere valeur attendue dans un VAR */
  ATL_T_CAR      atl_ctxvarexpect[ATS_BUFFER_SIZE];
  /*! derniere valeur rec float dans un VAR */
  ATL_T_REAL     atl_ctxvarfrec;
  /*! derniere valeur attendue float dans un VAR */
  ATL_T_REAL     atl_ctxvarfexpect;
  /*! derniere valeur recue double dans un VAR */
  ATL_T_DOUBLE   atl_ctxvardrec;
  /*! derniere valeur attendue double dans un VAR */
  ATL_T_DOUBLE   atl_ctxvardexpect;
  /*! derniere valeur recue pointeur dans un VAR */
  ATL_PT_VOID    atl_ctxprev;
#if RTRT_CPLUSPLUS_LANGUAGE == 0
  /*! pile des cleanup ENABLE */
  ATL_T_CLEANUP  atl_ctxgestcleanup[ATS_CLEANUP_STACK_SIZE];
  /*! index dernier element pile des cleanup */
  ATL_T_INT      atl_ctxindexcleanup;
#endif
  /*! niveau courant de scenario/proc/blocks */
  ATL_T_INT      atl_ctxlevel;
  /*! flag de comparaison CALL, VAR, ... */
  ATL_T_INT      atl_ctxflagrt;
  /*! tableau des index utilises dans un VAR */
#ifndef RTRT_IDXVAR_7_MAX
  ATL_T_INT      atl_ctxI[20];
#else /* RTRT_IDXVAR_7_MAX */
  ATL_T_INT      atl_ctxI[8];
#endif /* RTRT_IDXVAR_7_MAX */
  /*! tableau des index utilises dans un VAR */
  ATL_T_INT      atl_ctxJ[20];
  /*! fd du fichier rio */
  RTRT_FILE_TYPE atl_ctxfdrio;

  /*! activation ou non des remontees de suivi dynamique */
  ATL_T_INT      atl_ctxtagtcommactivated;
  /*! socket de comm. permanente testeur-agent (suivi dynamique + comm.
   *  inter-testeurs) */
  ATL_T_INT      atl_ctxtagtsocket;
  /*! timeout associe au canal de comm. testeur-agent (suivi dynamique + comm.
   *  inter-testeurs) */
  ATL_T_INT      atl_ctxtagttimeout;

  /*! tableau des durees */
  ATL_T_DURATION atl_ctxduration;
  /*! date de debut d'execution */
  RTRT_CLOCK_TYPE
                 atl_ctxstdate;
  /*! date donne par l'utilisateur */
  RTRT_CLOCK_TYPE
		 atl_ctxusrdate;
  /*! flag d'activation du mode PROBE */
  ATL_T_CAR      atl_ctxflagprobe;
 /*! sortie vers le RIO oui ou non */
  ATL_T_INT      atl_ctxrio_onoff;
  /*! TRACE ON or TRACE OFF current status */
  ATL_T_INT      atl_ctxtrace_onoff;

#if RTRT_CPLUSPLUS_LANGUAGE == 0
  /*! gestion du buffer circulaire de trace */
  ATL_T_TRACECTX atl_ctxtracectx;
#endif

  /*! niveau courant des blocs init, term, except et scn pour erreur */
  ATL_T_INT	 atl_ctxtracelevel;

#if RTRT_CPLUSPLUS_LANGUAGE == 1
  /*! nom du fichier rio */
  ATL_T_CAR      atl_riofilename[255];
  /*! liste de filtres */
  ATL_PT_CAR      atl_lstflt;
  /*! validite de l'instance par rapport a la liste du PTS */
  ATL_T_INT     atl_instancevalid;
  /*! necessite de fournir le nom de l'instance a l'execution du testeur */
  ATL_T_INT     atl_instancerequired;
  /*! nom de l'instance */
  ATL_T_CAR     atl_instancename[255];
#endif

#if ATL_BUFFER_SIZE > 1024
  ATL_PT_CAR     atl_ctxp_out_buff;
  ATL_T_CAR      atl_ctxlocal_out_buff[ATL_BUFFER_SIZE];
#endif /* ATL_BUFFER_SIZE > 1024 */

#if RTRT_CPLUSPLUS_LANGUAGE == 0
} ATL_T_RUNCTX, *ATL_PT_RUNCTX;
#endif

#if RTRT_CPLUSPLUS_LANGUAGE == 0

#define WTIME (atl_ctx->atl_ctxrun.atl_ctxwtime)
#define I0 (atl_ctx->atl_ctxrun.atl_ctxI[0])
#define I1 (atl_ctx->atl_ctxrun.atl_ctxI[1])
#define I2 (atl_ctx->atl_ctxrun.atl_ctxI[2])
#define I3 (atl_ctx->atl_ctxrun.atl_ctxI[3])
#define I4 (atl_ctx->atl_ctxrun.atl_ctxI[4])
#define I5 (atl_ctx->atl_ctxrun.atl_ctxI[5])
#define I6 (atl_ctx->atl_ctxrun.atl_ctxI[6])
#define I7 (atl_ctx->atl_ctxrun.atl_ctxI[7])
#ifndef RTRT_IDXVAR_7_MAX
#define I8 (atl_ctx->atl_ctxrun.atl_ctxI[8])
#define I9 (atl_ctx->atl_ctxrun.atl_ctxI[9])
#define I10 (atl_ctx->atl_ctxrun.atl_ctxI[10])
#define I11 (atl_ctx->atl_ctxrun.atl_ctxI[11])
#define I12 (atl_ctx->atl_ctxrun.atl_ctxI[12])
#define I13 (atl_ctx->atl_ctxrun.atl_ctxI[13])
#define I14 (atl_ctx->atl_ctxrun.atl_ctxI[14])
#define I15 (atl_ctx->atl_ctxrun.atl_ctxI[15])
#define I16 (atl_ctx->atl_ctxrun.atl_ctxI[16])
#define I17 (atl_ctx->atl_ctxrun.atl_ctxI[17])
#define I18 (atl_ctx->atl_ctxrun.atl_ctxI[18])
#define I19 (atl_ctx->atl_ctxrun.atl_ctxI[19])
#endif /* RTRT_IDXVAR_7_MAX */

#define J0 (atl_ctx->atl_ctxrun.atl_ctxJ[0])
#define J1 (atl_ctx->atl_ctxrun.atl_ctxJ[1])
#define J2 (atl_ctx->atl_ctxrun.atl_ctxJ[2])
#define J3 (atl_ctx->atl_ctxrun.atl_ctxJ[3])
#define J4 (atl_ctx->atl_ctxrun.atl_ctxJ[4])
#define J5 (atl_ctx->atl_ctxrun.atl_ctxJ[5])
#define J6 (atl_ctx->atl_ctxrun.atl_ctxJ[6])
#define J7 (atl_ctx->atl_ctxrun.atl_ctxJ[7])
#define J8 (atl_ctx->atl_ctxrun.atl_ctxJ[8])
#define J9 (atl_ctx->atl_ctxrun.atl_ctxJ[9])
#define J10 (atl_ctx->atl_ctxrun.atl_ctxJ[10])
#define J11 (atl_ctx->atl_ctxrun.atl_ctxJ[11])
#define J12 (atl_ctx->atl_ctxrun.atl_ctxJ[12])
#define J13 (atl_ctx->atl_ctxrun.atl_ctxJ[13])
#define J14 (atl_ctx->atl_ctxrun.atl_ctxJ[14])
#define J15 (atl_ctx->atl_ctxrun.atl_ctxJ[15])
#define J16 (atl_ctx->atl_ctxrun.atl_ctxJ[16])
#define J17 (atl_ctx->atl_ctxrun.atl_ctxJ[17])
#define J18 (atl_ctx->atl_ctxrun.atl_ctxJ[18])
#define J19 (atl_ctx->atl_ctxrun.atl_ctxJ[19])

#endif /* RTRT_CPLUSPLUS_LANGUAGE == 0 */

#if RTRT_CPLUSPLUS_LANGUAGE == 1
  public :

    void atl_catch ();
    void atl_var_catch (const ATL_T_INT index);
#endif

  CEXT void atl_rio_on              ATL_PP ((ATS_CTXP1));
  CEXT void atl_rio_off             ATL_PP ((ATS_CTXP1));
  CEXT void atl_trace_on            ATL_PP ((ATS_CTXP
				             const ATL_T_INT index));
  CEXT void atl_trace_off           ATL_PP ((ATS_CTXP
				             const ATL_T_INT index));
  CEXT void atl_bcf                 ATL_PP ((ATS_CTXP
					     const ATL_T_INT index ));

/*! macros used to test the coherence of the dynfollow    */
/*! selection for target deployment port & virtual tester */
#if ATS_DYNFLW
#define atl_initrio atl_initrio_with_dynamicfollowing
#else
#define atl_initrio atl_initrio_no_dynamicfollowing
#endif

#if RTRT_CPLUSPLUS_LANGUAGE == 0
  CEXT void atl_initrio             ATL_PP ((ATS_CTXP
					     const ATL_PT_CAR filename,
					     const ATL_PT_CAR filters,
					     const ATL_T_INT  instance_valid,
					     const ATL_T_INT  instance_required,
					     const ATL_PT_CAR instance_name,
					     const ATL_T_INT  occid,
					     const ATL_PT_CAR timestamp,
                                             const ATL_PT_CAR debug));
#else
  CEXT void atl_initrio             ATL_PP ((ATS_CTXP
					     const ATL_PT_CAR timestamp,
                                             const ATL_PT_CAR debug));
#endif

  CEXT void atl_send                ATL_PP ((ATS_CTXP
					     const ATL_T_INT index,
					     const ATL_PT_CAR msg_name));

  CEXT void atl_end_send            ATL_PP ((ATS_CTXP
					     const ATL_T_INT statusKO,
					     const ATL_T_INT index));

  CEXT void atl_isend               ATL_PP ((ATS_CTXP
					     const ATL_T_INT index,
					     const ATL_T_INT statusKO,
					     const ATL_PT_CAR msg,
					     const ATL_PT_CAR receip_tester));

  CEXT ATL_T_INT atl_cf             ATL_PP ((ATS_CTXP
					     const ATL_T_INT index));

  CEXT void atl_brdv                ATL_PP ((ATS_CTXP
					     const ATL_T_INT index,
					     const ATL_PT_CAR rdv_name));
  CEXT void atl_ardv                ATL_PP ((ATS_CTXP
					     const ATL_T_INT index,
					     const ATL_T_INT result,
					     const ATL_PT_CAR rdv_name));
  CEXT void atl_startchronomarkloop ATL_PP ((ATS_CTXP
					     const ATL_T_INT index,
					     const ATL_T_INT idxfin,
					     const ATL_T_INT scn_level,
					     const ATL_T_INT scn_loop,
					     const ATL_PT_CAR scn_name,
					     const ATL_T_INT tsfindex,
					     const ATL_T_INT tsffileindex));
  CEXT void atl_stopchronomarkloop  ATL_PP ((ATS_CTXP
					     ATL_T_INT state,
					     const ATL_T_INT index,
					     const ATL_T_INT scn_level,
					     const ATL_T_INT scn_loop,
					     const ATL_PT_CAR scn_name,
					     const ATL_T_INT tsfindex,
					     const ATL_T_INT tsffileindex));

  CEXT void atl_string_c            ATL_PP ((ATS_CTXP
					     const ATL_T_INT dotest,
					     const ATL_PT_CAR vinit,
					     const ATL_PT_CAR vexpect,
					     const ATL_PT_CAR vrec,
					     const ATL_T_INT index ));
  CEXT void atl_long_c              ATL_PP ((ATS_CTXP
					     const ATL_T_INT dotest,
					     const ATL_T_LONG vinit,
					     const ATL_T_LONG vexpect,
					     const ATL_T_LONG vrec,
					     const ATL_T_INT index));
  CEXT void atl_int_c               ATL_PP ((ATS_CTXP
					     const ATL_T_INT dotest,
					     const ATL_T_INT vinit,
					     const ATL_T_INT vexpect,
					     const ATL_T_INT vrec,
					     const ATL_T_INT index));
  CEXT void atl_pointer_c           ATL_PP ((ATS_CTXP
					     const ATL_T_INT dotest,
					     const ATL_PT_VOID vinit,
					     const ATL_PT_VOID vexpect,
					     const ATL_PT_VOID vrec,
					     const ATL_T_INT index));
  CEXT void atl_nonil_c             ATL_PP ((ATS_CTXP
					     const ATL_T_INT dotest,
					     const ATL_PT_VOID vinit,
					     const ATL_PT_VOID vexpect,
					     const ATL_PT_VOID vrec,
					     const ATL_T_INT index ));
  CEXT void atl_char_c              ATL_PP ((ATS_CTXP
					     const ATL_T_INT dotest,
					     const ATL_T_CAR vinit,
					     const ATL_T_CAR vexpect,
					     const ATL_T_CAR vrec,
					     const ATL_T_INT index));
  CEXT void atl_float_c             ATL_PP ((ATS_CTXP
					     const ATL_T_INT dotest,
					     const ATL_T_REAL vinit,
					     const ATL_T_REAL vexpect,
					     const ATL_T_REAL vrec,
					     const ATL_T_INT index));
  CEXT void atl_double_c            ATL_PP ((ATS_CTXP
					     const ATL_T_INT dotest,
					     const ATL_T_DOUBLE vinit,
					     const ATL_T_DOUBLE vexpect,
					     const ATL_T_DOUBLE vrec,
					     const ATL_T_INT index));
#if !RTRT_CPLUSPLUS_LANGUAGE
  CEXT void atl_enable              ATL_PP ((ATS_CTXP
					     ATL_T_INT (* proc) ( void ) ));
  CEXT void atl_disable             ATL_PP ((ATS_CTXP
					     ATL_T_INT (* proc) ( void ) ));
#endif
  CEXT void atl_endwaittil          ATL_PP ((ATS_CTXP
					     const ATL_T_INT state,
					     const ATL_T_INT index,
					     const ATL_PT_CAR msg_name));

  CEXT void atl_waittil             ATL_PP ((ATS_CTXP
					     const ATL_T_INT index));


  CEXT void atl_irecv               ATL_PP ((ATS_CTXP
				             const ATL_T_INT index));

  CEXT void atl_end_irecv           ATL_PP ((ATS_CTXP
				             const ATL_T_INT index,
				             const ATL_T_INT state_timeout,
				             const ATL_PT_CAR emit_tster,
				             const ATL_PT_CAR info_msg));

  CEXT ATL_T_INT atl_notify_tsf     ATL_PP ((ATS_CTXP
					     ATL_CONST ATL_T_INT locnumfic,
					     ATL_CONST ATL_T_INT id1,
					     ATL_CONST ATL_T_INT id2));

  CEXT ATL_T_INT atl_declare_ni_tsf ATL_PP ((ATS_CTXP
					     ATL_CONST ATL_T_INT file_index));

  CEXT ATL_T_INT atl_rdv_reached_tsf
                                    ATL_PP ((ATS_CTXP
					     ATL_CONST ATL_T_INT statusOK,
					     ATL_CONST ATL_T_INT tsfindex,
					     ATL_CONST ATL_T_INT file_index));

  CEXT ATL_T_INT atl_send_tsf       ATL_PP ((ATS_CTXP
					     ATL_CONST ATL_T_INT tsfindex,
					     ATL_CONST ATL_T_INT file_index));

  CEXT ATL_T_INT atl_waittil_tsf    ATL_PP ((ATS_CTXP
					     const ATL_PT_CAR msgtype,
					     const ATL_T_INT tsfindex,
					     const ATL_T_INT file_index));

  CEXT ATL_T_INT atl_begin_waittil_tsf
                                    ATL_PP ((ATS_CTXP
					     const ATL_T_INT tsfindex,
					     const ATL_T_INT file_index));

  CEXT ATL_T_INT atl_end_waittil_tsf
				    ATL_PP ((ATS_CTXP
					     const ATL_T_INT tsfindex,
					     const ATL_T_INT file_index,
					     const ATL_T_INT waittil_status));

  CEXT ATL_T_INT atl_intersend_tsf  ATL_PP ((ATS_CTXP
					     const ATL_T_INT tsfindex,
					     const ATL_T_INT file_index,
					     const ATL_PT_CAR fullidentifier));

  CEXT ATL_T_INT atl_interrecv_tsf  ATL_PP ((ATS_CTXP
					     const ATL_T_INT tsfindex,
					     const ATL_T_INT file_index,
					     const ATL_PT_CAR recved_msg,
					     const ATL_PT_CAR emitter));

  CEXT ATL_T_INT atl_special_block_tsf
                                    ATL_PP ((ATS_CTXP
					     const ATL_T_INT tsfindex,
					     const ATL_T_INT file_index,
					     const ATL_T_INT block_type,
					     const ATL_T_INT is_beginning));

  CEXT ATL_T_INT atl_endoftester_tagtlk
                                    ATL_PP ((ATS_CTXP1));

#if ATS_DYNFLW
  CEXT  ATL_T_INT atl_intersend     ATL_PP ((ATS_CTXP
					     ATL_CONST ATL_PT_CAR line,
					     ATL_CONST ATL_PT_CAR dest_tester));
#endif

  CEXT void atl_error               ATL_PP ((ATS_CTXP
					     const ATL_T_INT index));
  CEXT void atl_nomsg               ATL_PP ((ATS_CTXP
					     const ATL_T_INT index));
  CEXT void atl_msgdate             ATL_PP ((ATS_CTXP
					     const ATL_T_INT index));
  CEXT void atl_nodefmsg            ATL_PP ((ATS_CTXP
					     const ATL_T_INT index));
  CEXT void atl_noaddid             ATL_PP ((ATS_CTXP
					     const ATL_T_INT index));
  CEXT void atl_nomofid             ATL_PP ((ATS_CTXP
						const ATL_T_INT index));
  CEXT void atl_rioexit             ATL_PP ((ATS_CTXP
					     const ATL_T_INT index));
  CEXT void atl_setfilter           ATL_PP ((ATS_CTXP
					     const ATL_PT_CAR filter,
					     const ATL_T_INT index));
  CEXT void atl_unsetfilter         ATL_PP ((ATS_CTXP
					     const ATL_PT_CAR filter,
					     const ATL_T_INT index));
  CEXT void atl_comment             ATL_PP ((ATS_CTXP
					     const ATL_T_INT index));
  CEXT void atl_evt                 ATL_PP ((ATS_CTXP
					     const ATL_T_INT index,
					     const ATL_T_INT idx_ch,
                                             const ATL_T_INT display,
                                             const ATL_PT_CAR type));
  CEXT void atl_endevt              ATL_PP ((ATS_CTXP
					     const ATL_T_INT index));
  CEXT ATL_T_UTIME atl_getwtime     ATL_PP ((ATS_CTXP1));
  CEXT ATL_T_UTIME atl_getevtdate   ATL_PP ((ATS_CTXP1));
  CEXT void atl_state               ATL_PP ((ATS_CTXP
					     const ATL_T_INT etat,
					     const ATL_T_INT index));
  CEXT void atl_def_msg             ATL_PP ((ATS_CTXP
					     const ATL_T_INT index));



  CEXT ATL_T_INT atl_var_va         ATL_PP ((ATS_CTXP
					     const ATL_T_INT index));
  CEXT ATL_T_INT atl_end_var_va     ATL_PP ((ATS_CTXP
					     const ATL_T_INT index));
  CEXT ATL_T_INT atl_var            ATL_PP ((ATS_CTXP
					     const ATL_T_INT index));
  CEXT void atl_vartab              ATL_PP ((ATS_CTXP
					     const ATL_T_INT index));
  CEXT void atl_varstruct           ATL_PP ((ATS_CTXP
					     const ATL_T_INT index));
  CEXT void atl_varplage            ATL_PP ((ATS_CTXP
					     const ATL_T_INT index,
					     const ATL_T_INT idxmin,
					     const ATL_T_INT idxmax));
  CEXT void atl_varreset            ATL_PP ((ATS_CTXP1));
  CEXT void atl_var_long_c          ATL_PP ((ATS_CTXP
					     const ATL_T_INT index,
					     const ATL_T_LONG v_rec,
					     const ATL_T_LONG v_expect,
					     const ATL_T_INT base,
					     const ATL_T_LONG idxcur,
					     const ATL_T_INT idxmax));
  CEXT void atl_var_int_c           ATL_PP ((ATS_CTXP
					     const ATL_T_INT index,
					     const ATL_T_INT v_rec,
					     const ATL_T_INT v_expect,
					     const ATL_T_INT base,
					     const ATL_T_INT idxcur,
					     const ATL_T_INT idxmax));
  CEXT void atl_var_float_c         ATL_PP ((ATS_CTXP
					     const ATL_T_INT index,
					     const ATL_T_REAL v_rec,
					     const ATL_T_REAL v_expect,
					     const ATL_T_INT idxcur,
					     const ATL_T_INT idxmax));
  CEXT void atl_var_double_c        ATL_PP ((ATS_CTXP
					     const ATL_T_INT index,
					     const ATL_T_DOUBLE v_rec,
					     const ATL_T_DOUBLE v_expect,
					     const ATL_T_INT idxcur,
					     const ATL_T_INT idxmax));
  CEXT void atl_var_pointer_c       ATL_PP ((ATS_CTXP
					     const ATL_T_INT index,
					     const ATL_PT_VOID v_rec,
					     const ATL_PT_VOID v_expect,
					     const ATL_T_INT idxcur,
					     const ATL_T_INT idxmax));
  CEXT void atl_var_nonil_c         ATL_PP ((ATS_CTXP
					     const ATL_T_INT index,
					     ATL_PT_VOID v_rec,
					     const ATL_PT_VOID v_expect,
					     const ATL_T_INT idxcur,
					     const ATL_T_INT idxmax));
  CEXT void atl_var_byte_c          ATL_PP ((ATS_CTXP
					     const ATL_T_INT index,
					     const ATL_T_CAR v_rec,
					     const ATL_T_CAR v_expect,
					     const ATL_T_INT base,
					     const ATL_T_INT idxcur,
					     const ATL_T_INT idxmax));
  CEXT void atl_var_bytetab_c       ATL_PP ((ATS_CTXP
					     const ATL_T_INT index,
					     const ATL_T_CAR v_rec,
					     const ATL_T_CAR v_expect,
					     const ATL_T_INT idxcur,
					     const ATL_T_INT idxmax));
  CEXT void atl_var_char_c          ATL_PP ((ATS_CTXP
					     const ATL_T_INT index,
					     const ATL_T_CAR v_rec,
					     const ATL_T_CAR v_expect,
					     const ATL_T_INT idxcur,
					     const ATL_T_INT idxmax));
  CEXT void atl_var_string_c        ATL_PP ((ATS_CTXP
					     const ATL_T_INT index,
					     const ATL_PT_CAR v_rec,
					     const ATL_PT_CAR v_expect,
					     const ATL_T_INT idxcur,
					     const ATL_T_INT idxmax));
  CEXT void atl_var_range_byte_c    ATL_PP ((ATS_CTXP
					     const ATL_T_INT idx,
					     const ATL_T_CAR v_rec,
					     const ATL_T_INT v_min_opt,
					     const ATL_T_CAR v_min,
					     const ATL_T_INT v_max_opt,
					     const ATL_T_CAR v_max,
					     const ATL_T_INT base,
					     const ATL_T_INT curidx,
					     const ATL_T_INT idxmax));
  CEXT void atl_var_range_int_c     ATL_PP ((ATS_CTXP
					     const ATL_T_INT idx,
					     const ATL_T_INT v_rec,
					     const ATL_T_INT v_min_opt,
					     const ATL_T_INT v_min,
					     const ATL_T_INT v_max_opt,
					     const ATL_T_INT v_max,
					     const ATL_T_INT base,
					     const ATL_T_INT curidx,
					     const ATL_T_INT idxmax));
  CEXT void atl_var_range_long_c    ATL_PP ((ATS_CTXP
					     const ATL_T_INT idx,
					     const ATL_T_LONG v_rec,
					     const ATL_T_INT v_min_opt,
					     const ATL_T_LONG v_min,
					     const ATL_T_INT v_max_opt,
					     const ATL_T_LONG v_max,
					     const ATL_T_INT base,
					     const ATL_T_INT curidx,
					     const ATL_T_INT idxmax));
  CEXT void atl_var_range_float_c   ATL_PP ((ATS_CTXP
					     const ATL_T_INT idx,
					     const ATL_T_REAL v_rec,
					     const ATL_T_INT v_min_opt,
					     const ATL_T_REAL v_min,
					     const ATL_T_INT v_max_opt,
					     const ATL_T_REAL v_max,
					     const ATL_T_INT curidx,
					     const ATL_T_INT idxmax));
  CEXT void atl_var_range_double_c  ATL_PP ((ATS_CTXP
					     const ATL_T_INT idx,
					     const ATL_T_DOUBLE v_rec,
					     const ATL_T_INT v_min_opt,
					     const ATL_T_DOUBLE v_min,
					     const ATL_T_INT v_max_opt,
					     const ATL_T_DOUBLE v_max,
					     const ATL_T_INT curidx,
					     const ATL_T_INT idxmax));
  CEXT ATL_T_INT atl_cmp            ATL_PP ((ATS_CTXP1));
  CEXT void atl_setflagrt           ATL_PP ((ATS_CTXP1));
  CEXT void atl_cmp_long_c          ATL_PP ((ATS_CTXP
					     const ATL_T_INT index,
					     const ATL_T_LONG v_rec,
					     const ATL_T_LONG v_expect,
					     const ATL_T_INT idxcur,
					     const ATL_T_INT idxmax));
  CEXT void atl_cmp_int_c           ATL_PP ((ATS_CTXP
					     const ATL_T_INT index,
					     const ATL_T_INT v_rec,
					     const ATL_T_INT v_expect,
					     const ATL_T_INT idxcur,
					     const ATL_T_INT idxmax));
  CEXT void atl_cmp_float_c         ATL_PP ((ATS_CTXP
					     const ATL_T_INT index,
					     const ATL_T_REAL v_rec,
					     const ATL_T_REAL v_expect,
					     const ATL_T_INT idxcur,
					     const ATL_T_INT idxmax));
  CEXT void atl_cmp_double_c        ATL_PP ((ATS_CTXP
					     const ATL_T_INT index,
					     const ATL_T_DOUBLE v_rec,
					     const ATL_T_DOUBLE v_expect,
					     const ATL_T_INT idxcur,
					     const ATL_T_INT idxmax));
  CEXT void atl_cmp_pointer_c       ATL_PP ((ATS_CTXP
					     const ATL_T_INT index,
					     const ATL_PT_VOID v_rec,
					     const ATL_PT_VOID v_expect,
					     const ATL_T_INT idxcur,
					     const ATL_T_INT idxmax));
  CEXT void atl_cmp_nonil_c         ATL_PP ((ATS_CTXP
					     const ATL_T_INT index,
					     ATL_PT_VOID v_rec,
					     const ATL_PT_VOID v_expect,
					     const ATL_T_INT idxcur,
					     const ATL_T_INT idxmax));
  CEXT void atl_cmp_char_c          ATL_PP ((ATS_CTXP
					     const ATL_T_INT index,
					     const ATL_T_CAR v_rec,
					     const ATL_T_CAR v_expect,
					     const ATL_T_INT idxcur,
					     const ATL_T_INT idxmax));
  CEXT void atl_cmp_string_c        ATL_PP ((ATS_CTXP
					     const ATL_T_INT index,
					     const ATL_PT_CAR v_rec,
					     const ATL_PT_CAR v_expect,
					     const ATL_T_INT idxcur,
					     const ATL_T_INT idxmax));
  CEXT void atl_cmp_range_byte_c    ATL_PP ((ATS_CTXP
					     const ATL_T_INT idx,
					     const ATL_T_CAR v_rec,
					     const ATL_T_INT v_min_opt,
					     const ATL_T_UCAR v_min,
					     const ATL_T_INT v_max_opt,
					     const ATL_T_UCAR v_max,
					     const ATL_T_INT curidx,
					     const ATL_T_INT idxmax));
  CEXT void atl_cmp_range_int_c ATL_PP ((ATS_CTXP
					 const ATL_T_INT idx,
					 const ATL_T_INT v_rec,
					 const ATL_T_INT v_min_opt,
					 const ATL_T_INT v_min,
					 const ATL_T_INT v_max_opt,
					 const ATL_T_INT v_max,
					 const ATL_T_INT curidx,
					 const ATL_T_INT idxmax));
  CEXT void atl_cmp_range_long_c    ATL_PP ((ATS_CTXP
					     const ATL_T_INT idx,
					     const ATL_T_LONG v_rec,
					     const ATL_T_INT v_min_opt,
					     const ATL_T_LONG v_min,
					     const ATL_T_INT v_max_opt,
					     const ATL_T_LONG v_max,
					     const ATL_T_INT curidx,
					     const ATL_T_INT idxmax));
  CEXT void atl_cmp_range_float_c   ATL_PP ((ATS_CTXP
					     const ATL_T_INT idx,
					     const ATL_T_REAL v_rec,
					     const ATL_T_INT v_min_opt,
					     const ATL_T_REAL v_min,
					     const ATL_T_INT v_max_opt,
					     const ATL_T_REAL v_max,
					     const ATL_T_INT curidx,
					     const ATL_T_INT idxmax));
  CEXT void atl_cmp_range_double_c  ATL_PP ((ATS_CTXP
					     const ATL_T_INT idx,
					     const ATL_T_DOUBLE v_rec,
					     const ATL_T_INT v_min_opt,
					     const ATL_T_DOUBLE v_min,
					     const ATL_T_INT v_max_opt,
					     const ATL_T_DOUBLE v_max,
					     const ATL_T_INT curidx,
					     const ATL_T_INT idxmax));
  CEXT void atl_dmp_int_c           ATL_PP ((ATS_CTXP
					     const ATL_T_INT index,
					     const ATL_T_INT v_rec,
					     const ATL_T_INT base,
					     const ATL_T_INT idxcur,
					     const ATL_T_INT idxmax ));
  CEXT void atl_dmp_long_c          ATL_PP ((ATS_CTXP
					     const ATL_T_INT index,
					     const ATL_T_LONG v_rec,
					     const ATL_T_INT base,
					     const ATL_T_INT idxcur,
					     const ATL_T_INT idxmax));
  CEXT void atl_dmp_float_c         ATL_PP ((ATS_CTXP
					     const ATL_T_INT index,
					     const ATL_T_REAL v_rec,
					     const ATL_T_INT idxcur,
					     const ATL_T_INT idxmax));
  CEXT void atl_dmp_double_c        ATL_PP ((ATS_CTXP
					     const ATL_T_INT index,
					     const ATL_T_DOUBLE v_rec,
					     const ATL_T_INT idxcur,
					     const ATL_T_INT idxmax));
  CEXT void atl_dmp_pointer_c       ATL_PP ((ATS_CTXP
					     const ATL_T_INT index,
					     const ATL_PT_VOID v_rec,
					     const ATL_T_INT idxcur,
					     const ATL_T_INT idxmax));
  CEXT void atl_dmp_nonil_c         ATL_PP ((ATS_CTXP
					     const ATL_T_INT index,
					     ATL_PT_VOID v_rec,
					     const ATL_T_INT idxcur,
					     const ATL_T_INT idxmax));
  CEXT void atl_dmp_byte_c          ATL_PP ((ATS_CTXP
					     const ATL_T_INT index,
					     const ATL_T_CAR v_rec,
					     const ATL_T_INT base,
					     const ATL_T_INT idxcur,
					     const ATL_T_INT idxmax ));
  CEXT void atl_dmp_bytetab_c       ATL_PP ((ATS_CTXP
					     const ATL_T_INT index,
					     const ATL_T_CAR v_rec,
					     const ATL_T_INT idxcur,
					     const ATL_T_INT idxmax));
  CEXT void atl_dmp_char_c          ATL_PP ((ATS_CTXP
					     const ATL_T_INT index,
					     const ATL_T_CAR v_rec,
					     const ATL_T_INT base,
					     const ATL_T_INT idxcur,
					     const ATL_T_INT idxmax));
  CEXT void atl_dmp_string_c        ATL_PP ((ATS_CTXP
					     const ATL_T_INT index,
					     const ATL_PT_CAR v_rec,
					     const ATL_T_INT idxcur,
					     const ATL_T_INT idxmax));
  CEXT void atl_dmp_range_byte_c    ATL_PP ((ATS_CTXP
					     const ATL_T_INT idx,
					     const ATL_T_INT v_min_opt,
					     const ATL_T_UCAR v_min,
					     const ATL_T_INT v_max_opt,
					     const ATL_T_UCAR v_max,
					     const ATL_T_INT base,
					     const ATL_T_INT curidx,
					     const ATL_T_INT idxmax));
  CEXT void atl_dmp_range_int_c     ATL_PP ((ATS_CTXP
					     const ATL_T_INT idx,
					     const ATL_T_INT v_min_opt,
					     const ATL_T_INT v_min,
					     const ATL_T_INT v_max_opt,
					     const ATL_T_INT v_max,
					     const ATL_T_INT base,
					     const ATL_T_INT curidx,
					     const ATL_T_INT idxmax));
  CEXT void atl_dmp_range_long_c    ATL_PP ((ATS_CTXP
					     const ATL_T_INT idx,
					     const ATL_T_INT v_min_opt,
					     const ATL_T_LONG v_min,
					     const ATL_T_INT v_max_opt,
					     const ATL_T_LONG v_max,
					     const ATL_T_INT base,
					     const ATL_T_INT curidx,
					     const ATL_T_INT idxmax));
  CEXT void atl_dmp_range_float_c   ATL_PP ((ATS_CTXP
					     const ATL_T_INT idx,
					     const ATL_T_INT v_min_opt,
					     const ATL_T_REAL v_min,
					     const ATL_T_INT v_max_opt,
					     const ATL_T_REAL v_max,
					     const ATL_T_INT curidx,
					     const ATL_T_INT idxmax));
  CEXT void atl_dmp_range_double_c  ATL_PP ((ATS_CTXP
					     const ATL_T_INT idx,
					     const ATL_T_INT v_min_opt,
					     const ATL_T_DOUBLE v_min,
					     const ATL_T_INT v_max_opt,
					     const ATL_T_DOUBLE v_max,
					     const ATL_T_INT curidx,
					     const ATL_T_INT idxmax));
  CEXT void atl_binit               ATL_PP ((ATS_CTXP
					     const ATL_T_INT state,
					     const ATL_T_INT index,
					     const ATL_T_INT scn_level,
					     const ATL_T_INT scn_loop));
  CEXT void atl_ainit               ATL_PP ((ATS_CTXP
					     const ATL_T_INT state,
					     const ATL_T_INT index,
					     const ATL_T_INT scn_level,
					     const ATL_T_INT scn_loop));
  CEXT void atl_bterm               ATL_PP ((ATS_CTXP
					     const ATL_T_INT state,
					     const ATL_T_INT index,
					     const ATL_T_INT scn_level,
					     const ATL_T_INT scn_loop));
  CEXT void atl_aterm               ATL_PP ((ATS_CTXP
					     const ATL_T_INT state,
					     const ATL_T_INT index,
					     const ATL_T_INT scn_level,
					     const ATL_T_INT scn_loop));
  CEXT void atl_bexcep              ATL_PP ((ATS_CTXP
					     const ATL_T_INT state,
					     const ATL_T_INT index,
					     const ATL_T_INT scn_level,
					     const ATL_T_INT scn_loop));
  CEXT void atl_aexcep              ATL_PP ((ATS_CTXP
					     const ATL_T_INT state,
					     const ATL_T_INT index,
					     const ATL_T_INT scn_level,
					     const ATL_T_INT scn_loop));
  CEXT void atl_end                 ATL_PP ((ATS_CTXP1));
  CEXT void atl_numofid             ATL_PP ((ATS_CTXP
					     const ATL_T_INT index));
  CEXT ATL_T_UTIME atl_time         ATL_PP ((RTRT_CLOCK_TYPE *timer));
  CEXT void atl_pause               ATL_PP ((const ATL_T_UTIME pause));
  CEXT void atl_print               ATL_PP ((ATS_CTXP
					     ATL_CONST ATL_T_INT index,
					     ATL_CONST ATL_T_ULONG value));
  CEXT void atl_bproc               ATL_PP ((ATS_CTXP
					     ATL_T_INT index));
  CEXT void atl_aproc               ATL_PP ((ATS_CTXP
					     ATL_T_INT index));
  CEXT void atl_set_trace_functions ATL_PP ((ATS_CTXP1));
  CEXT void atl_flush_circular_buffer
				    ATL_PP ((ATS_CTXP
					     ATL_T_UCAR priority));
  CEXT void atl_flush_trace         ATL_PP ((ATS_CTXP
					     const ATL_T_INT index));
  CEXT void ats_exit         ATL_PP ((const ATL_T_INT ec));

#if RTRT_CPLUSPLUS_LANGUAGE == 1
  void atl_userexit (ATL_PT_CAR str);
  void atl_v1_3 ();

  void atl_set_occid                (ATL_T_INT argc,
				     ATL_T_CAR** argv,
				     ATL_T_INT defaultOccidValue);
  void atl_set_testernaming ();

  void atl_set_riofilename          (ATL_T_INT argc,
				     ATL_T_CAR** argv,
				     ATL_PT_CAR defaultRioFileName);
  void atl_set_instancevalid        (ATL_T_INT status);
  void atl_set_instancerequired     (ATL_T_INT status);
  void atl_set_instancename         (ATL_T_INT argc, ATL_PT_CAR* argv,
				     ATL_PT_CAR f_instance);
  void atl_set_lstflt               (ATL_T_INT argc,
				     ATL_T_CAR** argv,
				     ATL_PT_CAR defaultLstFlt);
  private:
#endif

#if RTRT_CPLUSPLUS_LANGUAGE == 1 || defined(RTRT_TDP_COMPILATION)

  CSTATIC void atl_bufindex_error   (ATS_CTXP1);
  CSTATIC void atl_bin2ascii        (ATS_CTXP
	                             const ATL_T_CAR car,
		                     ATL_PT_CAR dest);
  CSTATIC void atl_p_out_bin        (ATS_CTXP
	                             const ATL_T_CAR car);
  CSTATIC void atl_p_inc_bufindex   (ATS_CTXP1);
  CSTATIC void atl_p_out_car        (ATS_CTXP
	                             const ATL_T_CAR car);
  CSTATIC void atl_p_out_str        (ATS_CTXP
	                             ATL_PT_CAR str);
  CSTATIC void atl_p_out_quot       (ATS_CTXP
	                             ATL_PT_CAR str);
  CSTATIC void atl_p_flush_line     (ATS_CTXP1);

#if ATS_DYNFLW
  CSTATIC void atl_p_raise_dynfline (ATS_CTXP
	                             const ATL_PT_CAR suppl_info,
	                             const ATL_T_INT tsfindex,
	                             const ATL_T_INT tsffileindex);
#endif

  CSTATIC void atl_p_stackblock     (ATS_CTXP
	                             ATL_PT_CAR dest,
				     const ATL_T_INT raz);
  CSTATIC void atl_p_flushstack     (ATS_CTXP1);
  CSTATIC void atl_p_circular_buffer_flush_bufout_line
				    (ATS_CTXP1);
  CSTATIC ATL_PT_CAR atl_p_circular_buffer_eval_flush_line
				    (ATS_CTXP
				     ATL_PT_CAR begin,
				     ATL_PT_CAR end);
  CSTATIC ATL_PT_CAR atl_p_circular_buffer_flush_line
				    (ATS_CTXP
				     ATL_PT_CAR begin,
				     ATL_PT_CAR end);
  CSTATIC ATL_PT_CAR atl_car_image  (ATS_CTXP
	                             const ATL_PT_CAR str,
				     ATL_PT_CAR dest);
  CSTATIC ATL_PT_CAR atl_string_image
				    (ATS_CTXP
				     const ATL_PT_CAR str,
				     ATL_PT_CAR dest,
				     ATL_T_INT maxlen);
  CSTATIC ATL_PT_CAR atl_long_image (ATS_CTXP
	                             const ATL_T_LONG value,
				     ATL_PT_CAR str,
				     const ATL_T_INT base);
  CSTATIC ATL_PT_CAR atl_byte_image (ATS_CTXP
	                             const ATL_T_UCAR value,
				     ATL_PT_CAR str,
				     const ATL_T_INT base);
  CSTATIC ATL_PT_CAR atl_int_image  (ATS_CTXP
	                             const ATL_T_INT value,
				     ATL_PT_CAR str,
				     const ATL_T_INT base);
  CSTATIC ATL_PT_CAR atl_pointer_image
				    (ATS_CTXP
				     const ATL_PT_VOID ptr,
				     ATL_PT_CAR str,
				     const ATL_T_INT base);
  CSTATIC ATL_PT_CAR atl_float_image(ATS_CTXP
	                             const ATL_T_REAL f,
				     ATL_PT_CAR str);
  CSTATIC ATL_PT_CAR atl_double_image
				    (ATS_CTXP
				     const ATL_T_DOUBLE,
				     ATL_PT_CAR str);
  CSTATIC void atl_p_begin_line     (ATS_CTXP
	                             ATL_PT_CAR pcar,
				     const ATL_T_INT index);
  CSTATIC void atl_p_erasecleanup   (ATS_CTXP1);
  CSTATIC void atl_p_execcleanup    (ATS_CTXP1);
  CSTATIC void atl_p_range          (ATS_CTXP
	                             const ATL_T_INT min,
				     const ATL_T_INT max,
				     const ATL_T_INT state);
  CSTATIC void atl_p_var_elem       (ATS_CTXP
	                             const ATL_T_INT index ,
				     ATL_PT_CAR v_rec,
				     ATL_PT_CAR v_expect,
				     const ATL_T_INT idxcur,
				     const ATL_T_INT idxmax,
				     const ATL_T_INT passed);
  CSTATIC void atl_p_cmp_elem       (ATS_CTXP
	                             const ATL_T_INT index,
				     const ATL_PT_CAR v_rec,
				     const ATL_PT_CAR v_expect,
				     const ATL_T_INT idxcur,
				     const ATL_T_INT idxmax,
				     const ATL_T_INT passed);
  CSTATIC void atl_p_dmp_elem       (ATS_CTXP
	                             const ATL_T_INT index,
				     ATL_PT_CAR v_rec,
				     const ATL_T_INT idxcur,
				     const ATL_T_INT idxmax);

  CSTATIC void atl_p_CYToverpass_warning
				    (ATS_CTXP1);
  CSTATIC void atl_p_CYTupdate_circularbuffer_sizeleft
				    (ATS_CTXP1);
  CSTATIC void atl_p_CYTflush_bufout_line
				    (ATS_CTXP1);
  CSTATIC void atl_p_CYTtoplvl_flushbufoutline
				    (ATS_CTXP1);
#if ATS_RT_BUFFER
  CSTATIC void atl_p_decode         (ATL_PT_CAR p,
				     ATL_PT_CAR p2,
				     ATL_T_CAR **adr_succ_p,
				     ATL_T_CAR **adr_succ_p2);
  CSTATIC void atl_p_toplvl_decode  (ATS_CTXP
	                             ATL_PT_CAR begin);
#endif
  CSTATIC void atl_p_CYTpushstack   (ATS_CTXP1);
  CSTATIC void atl_p_CYTpopstack    (ATS_CTXP1);
  CSTATIC void atl_p_CYTupdatestack (ATS_CTXP1);
  CSTATIC void atl_p_CYTflushclosingstack
				    (ATS_CTXP1);
  CSTATIC void atl_p_CYTbegintoendblock_conversion
				    (ATL_PT_CAR dest,
				     ATL_PT_CAR source,
				     ATL_PT_CAR added_params);
  CSTATIC void atl_p_CYTcopystack   (ATS_CTXP1);
  CSTATIC void atl_hexa_image       (ATL_CONST ATL_T_UCAR value,
				     ATL_PT_CAR hex1,
				     ATL_PT_CAR hex2);
  CSTATIC void ats_writeln (ATS_CTXP ATL_PT_CAR buf);

#endif /* RTRT_CPLUSPLUS_LANGUAGE == 1 || defined(RTRT_TDP_COMPILATION) */

#if RTRT_CPLUSPLUS_LANGUAGE == 1
#define atl_logdate atl_ctxlogdate
#define atl_endsend atl_end_send
};
#endif

#if RTRT_SIGNAL != RTRT_NONE
_ATL_EXTERN_C void atl_handler           ATL_PP ((ATL_T_INT foo));
#endif


#endif
