

/*
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2011 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */


/*******************************************************************************
* $Descr: Trace Header (included across TP.h) $
*******************************************************************************/


/* Multiple inclusions guard: */
#ifndef ATT_H_INCLUDED
#define ATT_H_INCLUDED


/*******************************************************************************
*  C Instrumentor
*******************************************************************************/
#if ! RTRT_CPLUSPLUS_LANGUAGE

/* i is source number */
#define _ATT_IN(i) \
  _att_c_start_function(&_atl_stack_link);

/* no args */
#define _ATT_OUT \
  _att_c_end_function(&_atl_stack_link);

/* no args */
#define _ATT_OUT_EXPR \
  _att_c_end_function(&_atl_stack_link)

/* t is text to be attached in a note to the current instance */
#define _ATT_USER_NOTE(t) \
  _att_note_text(t,_atl_stack_link.InstanceId);

/* added for attolccp */
#define _ATT_DECLARE(i)
#define _ATT_IN_FCT(i,m) _ATT_IN(i)

#endif /* C language */


/*******************************************************************************
*  C++ Instrumentor
*******************************************************************************/
#if RTRT_CPLUSPLUS_LANGUAGE

/* a is class mask */
#if RTRT_64_BITS_PTR
#define _ATT_THIS(a) (((RTRT_U_INT32)((RTRT_U_INT64)this&0x0FFFFFFFFUL)) ^ (a))
#else
#define _ATT_THIS(a) (((RTRT_U_INT32)this) ^ (a))
#endif /* RTRT_64_BITS_PTR */

/* i is source number */
#define _ATT_DEFINE_CLASS_IN(i) \
class _ATT_CLASS_IN(i) : public _att_StackLink{ \
  public : \
    _ATT_CLASS_IN(i) (_AtlStackLinkP _atl_stack_link, RTRT_U_INT32 instance_id) \
    { StackLink = _atl_stack_link; \
      StackLink->InstanceId = instance_id; \
      _att_source_file(StackLink); \
      _att_method_entry(StackLink); } \
    ~_ATT_CLASS_IN(i) () \
    { _att_method_return(StackLink); } \
    _ATL_ANOTHER_METHOD \
};

/* If class and constructor are in the same source file */
/* i is source number */
#define _ATT_DEFINE_CLASS_IN_CTOR(i) \
class _ATT_CLASS_IN_CTOR(i) : public _att_StackLink { \
  public : \
    _ATT_CLASS_IN_CTOR(i) (_AtlStackLinkP _atl_stack_link, RTRT_U_INT32 class_id, RTRT_U_INT32 instance_id, char *text = (char *)0) \
    { StackLink = _atl_stack_link; \
      StackLink->InstanceId = instance_id; \
      _att_source_file(StackLink); \
      _att_new_instance(StackLink,_ATL_STACK_SOURCE(i).SourceNumber,class_id,text); } \
    ~_ATT_CLASS_IN_CTOR(i) () \
    { _att_method_return(StackLink); } \
    _ATL_ANOTHER_METHOD \
};

/* If class and constructor are in different source files */
/* i_j is made of i and j,
   i is class source number, ki is key and ci is checksum of class source file,
   j is constructor source number, kj is key and cj is checksum of constructor
   source file */
#define _ATT_DEFINE_CLASS_IN_CTOR_FAR(i_j,i,ki,ci,j,kj,cj) \
class _ATT_CLASS_IN_CTOR_FAR(i_j) : public _att_StackLink { \
  public : \
    _ATT_CLASS_IN_CTOR_FAR(i_j) (_AtlStackLinkP _atl_stack_link, RTRT_U_INT32 class_id, RTRT_U_INT32 instance_id, char *text = (char *)0) \
    { StackLink = _atl_stack_link; \
      StackLink->InstanceId = instance_id; \
      _att_source_file_alt(&(_ATL_STACK_SOURCE(i).SourceNumber),ki,ci); \
      _att_source_file(StackLink); \
      _att_new_instance(StackLink,_ATL_STACK_SOURCE(i).SourceNumber,class_id,text); } \
    ~_ATT_CLASS_IN_CTOR_FAR(i_j) () \
    { _att_method_return(StackLink); } \
    _ATL_ANOTHER_METHOD \
};

/* i is source number */
#define _ATT_DEFINE_CLASS_IN_DTOR(i) \
class _ATT_CLASS_IN_DTOR(i) : public _att_StackLink { \
  public : \
    _ATT_CLASS_IN_DTOR(i) (_AtlStackLinkP _atl_stack_link, RTRT_U_INT32 instance_id) \
    { StackLink = _atl_stack_link; \
      StackLink->InstanceId = instance_id; \
      _att_source_file(StackLink); \
      _att_method_entry(StackLink); } \
    ~_ATT_CLASS_IN_DTOR(i) () \
    { _att_delete_instance(StackLink); } \
    _ATL_ANOTHER_METHOD \
};

#if ATL_MULTI_THREADS
#define ATL_RESET_THREAD_INDEX(_atl_stack_link) _atl_stack_link.ThreadIndex = 0;
#else
#define ATL_RESET_THREAD_INDEX(_atl_stack_link)
#endif

/* i is source number */
#define _ATT_DEFINE_CLASS_IN_CATCH(i) \
class _ATT_CLASS_IN_CATCH(i) { \
  public : \
    _AtlStackLink _atl_stack_link; /* Not to be linked ! */ \
    RTRT_U_INT32 EndTextTsf; \
    _ATT_CLASS_IN_CATCH(i) (RTRT_U_INT32 text_tsf, RTRT_U_INT32 end_text_tsf, RTRT_U_INT32 instance_id) \
    { _ATL_STACK_INIT(_ATL_STACK_SOURCE_ADR(i),text_tsf) \
      _atl_stack_link.InstanceId = instance_id; \
      _ATL_STACK_SET_MAGIC(_atl_stack_link.Magic1) \
      _ATL_STACK_SET_MAGIC(_atl_stack_link.Magic2) \
      ATL_RESET_THREAD_INDEX(_atl_stack_link) \
      EndTextTsf = end_text_tsf; \
      _att_source_file(&_atl_stack_link); \
      _att_catch_exception(&_atl_stack_link); } \
    ~_ATT_CLASS_IN_CATCH(i) () \
    { _att_note_tsf(&_atl_stack_link,EndTextTsf); } \
    _ATL_ANOTHER_METHOD \
};

#define _ATT_DEFINE_CLASS_DUMP_AT_RETURN \
class _AttStartDumpAtReturn { \
  public : \
    ~_AttStartDumpAtReturn () \
    { _att_set_dump(1); } \
    _ATL_ANOTHER_METHOD \
};

/* i is source number */
#define _ATT_DECLARE(i) \
_ATT_DEFINE_CLASS_IN(i) \
_ATT_DEFINE_CLASS_IN_CTOR(i) \
_ATT_DEFINE_CLASS_IN_DTOR(i) \
_ATT_DEFINE_CLASS_IN_CATCH(i)

/* i is source number and m is message_tsf */
#define _ATT_IN_FCT(i,m) \
  _ATT_CLASS_IN(i) _att_object(&(_atl_stack_object._atl_stack_link),0); _ATL_ANOTHER_METHOD_CALL(_att_object)

/* i is source number, m is message_tsf, a is class_mask */
#define _ATT_IN_MTD(i,m,a) \
  _ATT_CLASS_IN(i) _att_object(&(_atl_stack_object._atl_stack_link),_ATT_THIS(a)); _ATL_ANOTHER_METHOD_CALL(_att_object)

/* i is source number, m is message_tsf, c is class_id, a is class mask */
#define _ATT_IN_CTOR(i,m,c,a) \
  _ATT_CLASS_IN_CTOR(i) _att_object(&(_atl_stack_object._atl_stack_link),c,_ATT_THIS(a)); _ATL_ANOTHER_METHOD_CALL(_att_object)

/* i is source number, m is message_tsf, c is class_id , a is class mask */
#define _ATT_IN_TMPL_CTOR(i,m,c,a) \
  _ATT_CLASS_IN_CTOR(i) _att_object(&(_atl_stack_object._atl_stack_link),c,_ATT_THIS(a),_att_tmpl_text_gen()); _ATL_ANOTHER_METHOD_CALL(_att_object)

/* i_j is made of class and constructor source numbers, m is message_tsf,
   c is class_id, a is class mask */
#define _ATT_IN_CTOR_FAR(i_j,m,c,a) \
  _ATT_CLASS_IN_CTOR_FAR(i_j) _att_object(&(_atl_stack_object._atl_stack_link),c,_ATT_THIS(a)); _ATL_ANOTHER_METHOD_CALL(_att_object)

/* i_j is made of class and constructor source numbers, m is message_tsf,
   c is class_id, a is class mask */
#define _ATT_IN_TMPL_CTOR_FAR(i_j,m,c,a) \
  _ATT_CLASS_IN_CTOR_FAR(i_j) _att_object(&(_atl_stack_object._atl_stack_link),c,_ATT_THIS(a),_att_tmpl_text_gen()); _ATL_ANOTHER_METHOD_CALL(_att_object)

/* i is source number, m is message_tsf, a is class mask */
#define _ATT_IN_DTOR(i,m,a) \
  _ATT_CLASS_IN_DTOR(i) _att_object(&(_atl_stack_object._atl_stack_link),_ATT_THIS(a)); _ATL_ANOTHER_METHOD_CALL(_att_object)

#define _ATT_PARAM_CCTOR _att_from
#define _ATT_NO_TYPE_MARKER

/* s may be _ATT_NO_TYPE_MARKER, class, struct or union and n is class name */
#define _ATT_DEFAULT_IN_CCTOR_START_CONST(s,n) \
  public : n(const s n &_ATT_PARAM_CCTOR)

/* s may be _ATT_NO_TYPE_MARKER, class, struct or union and n is class name */
#define _ATT_DEFAULT_IN_CCTOR_START(s,n) \
  public : n(s n &_ATT_PARAM_CCTOR)

/* s may be _ATT_NO_TYPE_MARKER, class, struct or union and n is class name */
#define _ATT_DEFAULT_IN_CCTOR_START_CONST_NOPARAM(s,n) \
  public : n(const s n &)

/* s may be _ATT_NO_TYPE_MARKER, class, struct or union and n is class name */
#define _ATT_DEFAULT_IN_CCTOR_START_NOPARAM(s,n) \
  public : n(s n &)

/* i is source number, m is message_tsf, c is class_id, a is class mask */
#define _ATT_DEFAULT_IN_CCTOR_END(i,m,c,a) \
  {_ATL_STACK_IN(i,m)_ATT_IN_CTOR(i,m,c,a);}

/* i is source number, m is message_tsf, c is class_id, a is class mask */
#define _ATT_DEFAULT_IN_TMPL_CCTOR_END(i,m,c,a) \
  {_ATL_STACK_IN(i,m)_ATT_IN_TMPL_CTOR(i,m,c,a);}

/* i is source number, m is message_tsf, c is class_id, a is class mask */
#define _ATT_DEFAULT_IN_CCTOR_UNION_END(i,m,c,a) \
  {_ATL_STACK_IN(i,m)_ATT_IN_CTOR(i,m,c,a); *this = _ATT_PARAM_CCTOR; }

/* i is source number, m is message_tsf, c is class_id, a is class mask */
#define _ATT_DEFAULT_IN_TMPL_CCTOR_UNION_END(i,m,c,a) \
  {_ATL_STACK_IN(i,m)_ATT_IN_TMPL_CTOR(i,m,c,a); *this = _ATT_PARAM_CCTOR; }

#define _ATT_CCTOR_INITIALIZER :
#define _ATT_CCTOR_SEPARATOR ,

/* a is attribute name */
#define _ATT_CALL_ATTR_CCTOR(a) a(_ATT_PARAM_CCTOR.a)

/* a is array attribute name, t is item type and s is array size */
#define _ATT_IAA_ITERATOR1(a) RTRT_CONCAT_MACRO(_att_iterator_,a)
#define _ATT_IAA_ITERATOR2(a) RTRT_CONCAT_MACRO(_att_iterator_2_,a)
#define _ATT_IAA_ITERATOR3(a) RTRT_CONCAT_MACRO(_att_iterator_3_,a)
#define _ATT_IAA_ITERATOR4(a) RTRT_CONCAT_MACRO(_att_iterator_4_,a)
#define _ATT_IAA_ITERATOR5(a) RTRT_CONCAT_MACRO(_att_iterator_5_,a)

#define _ATT_INIT_ARRAY_ATTR_1(a,t,s1) \
  for (int _ATT_IAA_ITERATOR1(a) = 0; \
       _ATT_IAA_ITERATOR1(a) < s1; \
       _ATT_IAA_ITERATOR1(a)++) \
    a[_ATT_IAA_ITERATOR1(a)] = \
      _ATT_PARAM_CCTOR.a[_ATT_IAA_ITERATOR1(a)];

#define _ATT_INIT_ARRAY_ATTR_2(a,t,s1,s2) \
  for (int _ATT_IAA_ITERATOR1(a) = 0; \
       _ATT_IAA_ITERATOR1(a) < s1; \
       _ATT_IAA_ITERATOR1(a)++) \
  for (int _ATT_IAA_ITERATOR2(a) = 0; \
       _ATT_IAA_ITERATOR2(a) < s2; \
       _ATT_IAA_ITERATOR2(a)++) \
    a[_ATT_IAA_ITERATOR1(a)][_ATT_IAA_ITERATOR2(a)] = \
      _ATT_PARAM_CCTOR.a[_ATT_IAA_ITERATOR1(a)][_ATT_IAA_ITERATOR2(a)];

#define _ATT_INIT_ARRAY_ATTR_3(a,t,s1,s2,s3) \
  for (int _ATT_IAA_ITERATOR1(a) = 0; \
       _ATT_IAA_ITERATOR1(a) < s1; \
       _ATT_IAA_ITERATOR1(a)++) \
  for (int _ATT_IAA_ITERATOR2(a) = 0; \
       _ATT_IAA_ITERATOR2(a) < s2; \
       _ATT_IAA_ITERATOR2(a)++) \
  for (int _ATT_IAA_ITERATOR3(a) = 0; \
       _ATT_IAA_ITERATOR3(a) < s3; \
       _ATT_IAA_ITERATOR3(a)++) \
    a[_ATT_IAA_ITERATOR1(a)][_ATT_IAA_ITERATOR2(a)][_ATT_IAA_ITERATOR3(a)] = \
      _ATT_PARAM_CCTOR.a[_ATT_IAA_ITERATOR1(a)][_ATT_IAA_ITERATOR2(a)][_ATT_IAA_ITERATOR3(a)];

#define _ATT_INIT_ARRAY_ATTR_4(a,t,s1,s2,s3,s4) \
  for (int _ATT_IAA_ITERATOR1(a) = 0; \
       _ATT_IAA_ITERATOR1(a) < s1; \
       _ATT_IAA_ITERATOR1(a)++) \
  for (int _ATT_IAA_ITERATOR2(a) = 0; \
       _ATT_IAA_ITERATOR2(a) < s2; \
       _ATT_IAA_ITERATOR2(a)++) \
  for (int _ATT_IAA_ITERATOR3(a) = 0; \
       _ATT_IAA_ITERATOR3(a) < s3; \
       _ATT_IAA_ITERATOR3(a)++) \
  for (int _ATT_IAA_ITERATOR4(a) = 0; \
       _ATT_IAA_ITERATOR4(a) < s4; \
       _ATT_IAA_ITERATOR4(a)++) \
    a[_ATT_IAA_ITERATOR1(a)][_ATT_IAA_ITERATOR2(a)][_ATT_IAA_ITERATOR3(a)][_ATT_IAA_ITERATOR4(a)] = \
      _ATT_PARAM_CCTOR.a[_ATT_IAA_ITERATOR1(a)][_ATT_IAA_ITERATOR2(a)][_ATT_IAA_ITERATOR3(a)][_ATT_IAA_ITERATOR4(a)];

#define _ATT_INIT_ARRAY_ATTR_5(a,t,s1,s2,s3,s4,s5) \
  for (int _ATT_IAA_ITERATOR1(a) = 0; \
       _ATT_IAA_ITERATOR1(a) < s1; \
       _ATT_IAA_ITERATOR1(a)++) \
  for (int _ATT_IAA_ITERATOR2(a) = 0; \
       _ATT_IAA_ITERATOR2(a) < s2; \
       _ATT_IAA_ITERATOR2(a)++) \
  for (int _ATT_IAA_ITERATOR3(a) = 0; \
       _ATT_IAA_ITERATOR3(a) < s3; \
       _ATT_IAA_ITERATOR3(a)++) \
  for (int _ATT_IAA_ITERATOR4(a) = 0; \
       _ATT_IAA_ITERATOR4(a) < s4; \
       _ATT_IAA_ITERATOR4(a)++)  \
  for (int _ATT_IAA_ITERATOR5(a) = 0; \
       _ATT_IAA_ITERATOR5(a) < s5; \
       _ATT_IAA_ITERATOR5(a)++) \
    a[_ATT_IAA_ITERATOR1(a)][_ATT_IAA_ITERATOR2(a)][_ATT_IAA_ITERATOR3(a)][_ATT_IAA_ITERATOR4(a)][_ATT_IAA_ITERATOR5(a)] = \
      _ATT_PARAM_CCTOR.a[_ATT_IAA_ITERATOR1(a)][_ATT_IAA_ITERATOR2(a)][_ATT_IAA_ITERATOR3(a)][_ATT_IAA_ITERATOR4(a)][_ATT_IAA_ITERATOR5(a)];


#define _ATT_TMPL_TEXT \
  public : inline char *_att_tmpl_text ( ) const; \
           inline char *_att_tmpl_text_gen ( ) const;

#define _ATT_TMPL_NOTE_NOT_COMPUTED (char *)"Template instance not computed"
#define _ATT_TMPL_TEXT_CODE \
  public : char *_att_tmpl_text ( ) const { return _ATT_TMPL_NOTE_NOT_COMPUTED; } \
           char *_att_tmpl_text_gen ( ) const { return _att_tmpl_text ( ); }

#define _ATT_MAGIC_NUMBER 0x4A596877

#if !ATL_CHECK_STACK
#define ATT_NO_MAGIC_IN_ADDED_FIELDS
#endif

#ifndef ATT_NO_MAGIC_IN_ADDED_FIELDS
#define _ATT_MAGIC_DECL RTRT_U_INT32 Magic;
#define _ATT_MAGIC_INIT Magic = _ATT_MAGIC_NUMBER;
#define _ATT_MAGIC_TEST if ( Magic != _ATT_MAGIC_NUMBER ) return (RTRT_U_INT32)0;
#else /* ATT_NO_MAGIC_IN_ADDED_FIELDS */
#define _ATT_MAGIC_DECL
#define _ATT_MAGIC_INIT
#define _ATT_MAGIC_TEST
#endif /* ATT_NO_MAGIC_IN_ADDED_FIELDS */

#define _ATT_CTOR_NO_ACTION(i,m,c,a)
#define _ATT_DTOR_NO_ACTION(i,m,a)

/* i is source number, m is default ctor message_id (m+1 fo const cctor and
   m+2 for dtor), c is class_id, a is class_mask, ctor is ctor action, cctor
   is cctor action and dtor is dtor action */
#define _ATT_DEFINE_CLASS_FIELD(i,m,c,a,ctor,cctor,dtor) \
protected : class _AttClassField { \
  public : \
    _ATT_MAGIC_DECL \
    _AttClassField ( ) \
    { _ATL_STACK_CLASS_IN(i) _atl_stack_object(m); \
      _ATT_MAGIC_INIT \
      _ATL_ANOTHER_METHOD_CALL(_atl_stack_object) \
      ctor(i,m,c,a); } \
    _AttClassField ( const _AttClassField & ) \
    { _ATL_STACK_CLASS_IN(i) _atl_stack_object(m+1); \
      _ATT_MAGIC_INIT \
      _ATL_ANOTHER_METHOD_CALL(_atl_stack_object) \
      cctor(i,m+1,c,a); } \
    ~_AttClassField ( ) \
    { _ATL_STACK_CLASS_IN(i) _atl_stack_object(m+2); \
      dtor(i,m+2,a); } \
    RTRT_U_INT32 _att_this ( ) const \
    { _ATT_MAGIC_TEST \
      return _ATT_THIS(a); } \
}; \
_AttClassField _att_field;

/* i is source number, m is default ctor message_id (m+1 fo const cctor and
   m+2 for dtor), c is class_id, a is class_mask, ctor is ctor action, cctor
   is cctor action and dtor is dtor action */
#define _ATT_DEFINE_CLASS_FIELD_TMPL(i,m,c,a,ctor,cctor,dtor) \
protected : class _AttClassField { \
  public : \
    _ATT_MAGIC_DECL \
    _AttClassField ( ) \
    { _ATL_STACK_CLASS_IN(i) _atl_stack_object(m); \
      _ATT_MAGIC_INIT \
      _ATL_ANOTHER_METHOD_CALL(_atl_stack_object) \
      ctor(i,m,c,a); } \
    _AttClassField ( const _AttClassField & ) \
    { _ATL_STACK_CLASS_IN(i) _atl_stack_object(m+1); \
      _ATT_MAGIC_INIT \
      _ATL_ANOTHER_METHOD_CALL(_atl_stack_object) \
      cctor(i,m+1,c,a); } \
    ~_AttClassField ( ) \
    { _ATL_STACK_CLASS_IN(i) _atl_stack_object(m+2); \
      dtor(i,m+2,a); } \
    RTRT_U_INT32 _att_this ( ) const \
    { _ATT_MAGIC_TEST \
      return _ATT_THIS(a); } \
    _ATT_TMPL_TEXT \
}; \
_AttClassField _att_field; \

/* i is source number, m is default ctor message_id (m+1 for const cctor and
   m+2 for dtor), c is class_id, a is class_mask, ctor is ctor action, cctor
   is cctor action and dtor is dtor action */
#define _ATT_DEFINE_CLASS_FIELD_TMPL_CODE(i,m,c,a,ctor,cctor,dtor) \
protected : class _AttClassField { \
  public : \
    _ATT_MAGIC_DECL \
    _AttClassField ( ) \
    { _ATL_STACK_CLASS_IN(i) _atl_stack_object(m); \
      _ATT_MAGIC_INIT \
      ctor(i,m,c,a); } \
    _AttClassField ( const _AttClassField & ) \
    { _ATL_STACK_CLASS_IN(i) _atl_stack_object(m+1); \
      _ATT_MAGIC_INIT \
      cctor(i,m+1,c,a); } \
    ~_AttClassField ( ) \
    { _ATL_STACK_CLASS_IN(i) _atl_stack_object(m+2); \
      dtor(i,m+2,a); } \
    RTRT_U_INT32 _att_this ( ) const \
    { _ATT_MAGIC_TEST \
      return _ATT_THIS(a); } \
    _ATT_TMPL_TEXT_CODE \
}; \
_AttClassField _att_field; \

/* i is source number, m is message_tsf, a is class_mask */
#define _ATT_FIELD_MTD(i,m) \
  _ATT_CLASS_IN(i) _att_object(&(_atl_stack_object._atl_stack_link),_att_field._att_this()); _ATL_ANOTHER_METHOD_CALL(_att_object)

/* i is source number, m is message_tsf, c is class_id, a is class mask */
#define _ATT_FIELD_CTOR(i,m,c,a) \
  _ATT_CLASS_IN_CTOR(i) _att_object(&(_atl_stack_object._atl_stack_link),c,_att_field._att_this()); _ATL_ANOTHER_METHOD_CALL(_att_object)

/* i is source number, m is message_tsf, c is class_id , a is class mask */
#define _ATT_FIELD_TMPL_CTOR(i,m,c,a) \
  _ATT_CLASS_IN_CTOR(i) _att_object(&(_atl_stack_object._atl_stack_link),c,_att_field._att_this(),_att_field._att_tmpl_text_gen()); _ATL_ANOTHER_METHOD_CALL(_att_object)

/* i_j is made of class and constructor source numbers, m is message_tsf,
   c is class_id, a is class mask */
#define _ATT_FIELD_CTOR_FAR(i_j,m,c,a) \
  _ATT_CLASS_IN_CTOR_FAR(i_j) _att_object(&(_atl_stack_object._atl_stack_link),c,_att_field._att_this()); _ATL_ANOTHER_METHOD_CALL(_att_object)

/* i_j is made of class and constructor source numbers, m is message_tsf,
   c is class_id, a is class mask */
#define _ATT_FIELD_TMPL_CTOR_FAR(i_j,m,c,a) \
  _ATT_CLASS_IN_CTOR_FAR(i_j) _att_object(&(_atl_stack_object._atl_stack_link),c,_att_field._att_this(),_att_field._att_tmpl_text_gen()); _ATL_ANOTHER_METHOD_CALL(_att_object)

/* i is source number, m is message_tsf, a is class mask */
#define _ATT_FIELD_DTOR(i,m,a) \
  _ATT_CLASS_IN_DTOR(i) _att_object(&(_atl_stack_object._atl_stack_link),_att_field._att_this()); _ATL_ANOTHER_METHOD_CALL(_att_object)

/* i is source number and m is message_id (exception message) */
#define _ATT_THROW(i,m) \
  _att_throw_exception(&(_atl_stack_object._atl_stack_link),m)

/* i is source number, t is text_tsf and e is end_text_tsf */
#define _ATT_CATCH(i,t,e) \
  _ATT_CLASS_IN_CATCH(i) _att_catch_object(t,e,_atl_stack_object._atl_stack_link.InstanceId); _ATL_ANOTHER_METHOD_CALL(_att_catch_object)

/* t is text to be attached in a note to the current instance */
#define _ATT_USER_NOTE(t) \
  _att_note_text(t,_atl_stack_object._atl_stack_link.InstanceId);

/* how to start messages dump at return time */
#define _ATT_START_DUMP_AT_RETURN \
  _AttStartDumpAtReturn _att_start_dump_at_return; _ATL_ANOTHER_METHOD_CALL(_att_start_dump_at_return)

#endif /* C++ language */


/*******************************************************************************
* User Directives:
*******************************************************************************/

/* how to start messages dump */
#define _ATT_START_DUMP_EXPR _att_set_dump(1)
#define _ATT_START_DUMP _ATT_START_DUMP_EXPR;

/* how to stop messages dump */
#define _ATT_STOP_DUMP _att_set_dump(0);

/* how to toggle alternatively between messages dump and no messages dump */
#define _ATT_TOGGLE_DUMP _att_set_dump(2);

/* how to dump current call stack */
#define _ATT_DUMP_STACK _att_dump_stack();

/* how to dump current call stack */
#define _ATT_FLUSH_ITEMS _att_flush_all_items();


/*******************************************************************************
* Functions prototypes:
*******************************************************************************/

#if !RTRT_KR

_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     int  _att_source_file      ( _AtlStackLinkP );
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     int  _att_source_file_alt  ( RTRT_U_INT32 *,
                                              RTRT_U_INT32,
                                              RTRT_U_INT32 );
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     void _att_new_instance     ( _AtlStackLinkP,
                                              RTRT_U_INT32,
                                              RTRT_U_INT32,
                                              char * );
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     void _att_delete_instance  ( _AtlStackLinkP );
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     void _att_method_entry     ( _AtlStackLinkP );
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     void _att_method_return    ( _AtlStackLinkP );
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     void _att_throw_exception  ( _AtlStackLinkP,
                                              RTRT_U_INT32 );
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     void _att_catch_exception  ( _AtlStackLinkP );
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     void _att_note_tsf         ( _AtlStackLinkP,
                                              RTRT_U_INT32 );
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     void _att_note_text        ( RTRT_CONST_STRING_DECL ,
                                              RTRT_U_INT32 );
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     void _att_c_start_function ( _AtlStackLinkP );
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     int  _att_c_end_function   ( _AtlStackLinkP );
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     void _att_dump_stack       ( void );
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     void _att_set_dump         ( int );
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     void _att_flush_all_items  ( void );
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     void att_early_init        ( void );
#ifdef _ATL_EXIT_PENDING
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     void att_exit_pending      ( void );
#else /* _ATL_EXIT_PENDING */
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     void att_late_exit         ( void );
#endif /* _ATL_EXIT_PENDING */

#else /* RTRT_KR */

_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     int  _att_source_file      ( );
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     int  _att_source_file_alt  ( );
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     void _att_new_instance     ( );
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     void _att_delete_instance  ( );
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     void _att_f_method_entry   ( );
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     void _att_method_return    ( );
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     void _att_f_method_return  ( );
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     void _att_throw_exception  ( );
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     void _att_catch_exception  ( );
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     void _att_note_tsf         ( );
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     void _att_note_text        ( );
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     void _att_c_start_function ( );
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     int  _att_c_end_function   ( );
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     void _att_dump_stack       ( );
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     void _att_set_dump         ( );
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     void _att_flush_all_items  ( );
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     void att_early_init        ( );
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     void _att_method_return    ( );
#ifdef _ATL_EXIT_PENDING
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     void att_exit_pending      ( void );
#else /* _ATL_EXIT_PENDING */
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     void att_late_exit         ( void );
#endif /* _ATL_EXIT_PENDING */

#endif /* RTRT_KR */


#ifdef RTRT_TDP_COMPILATION
/*******************************************************************************
* Function prototype declaration for TP compilation
*******************************************************************************/
RTRT_STATICFCT void _att_print_note_text(
#if !RTRT_KR
 RTRT_CONST_STRING_DECL text,
  RTRT_U_INT32 instance_id
#endif
);
#if ATL_MULTI_THREADS
#if !RTRT_KR
RTRT_STATICFCT    void _att_thread_fall_asleep ( );
RTRT_STATICFCT    void _att_thread_schedule    ( RTRT_THIDX_T );
RTRT_STATICFCT    void _att_thread_create      ( _AtlStackLinkP, RTRT_THIDX_T );
RTRT_STATICFCT   RTRT_THIDX_T _att_check_thread( _AtlStackLinkP stack_link );
#else
RTRT_STATICFCT    void _att_thread_fall_asleep ( );
RTRT_STATICFCT    void _att_thread_schedule    ( );
RTRT_STATICFCT    void _att_thread_create      ( );
RTRT_STATICFCT   RTRT_THIDX_T _att_check_thread( );
#endif
#endif /* ATL_MULTI_THREADS */

#endif /* RTRT_TDP_COMPILATION */

#endif /* ATT_H_INCLUDED */

