

/*
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2011 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */


/*******************************************************************************
* $Descr: Observation Tools common functions Source $
*******************************************************************************/


/*******************************************************************************
* Global variables
*******************************************************************************/

#if USE_ATC || USE_ATP || USE_ATQ || USE_ATT
#if !USE_ATU && !USE_ATO && !USE_ATV
char *atl_test_name;
#if ATL_OBSTOOLS_DUMP_FREQ
unsigned int obstools_dump_freq_count;
#endif /*ATL_OBSTOOLS_DUMP_FREQ*/
#endif
#endif

#if defined(ATL_SIGNAL) && ATL_SIGNAL_DUMP
int _atl_catch_reset;
#endif


#if !RTRT_KR
_ATL_EXTERN_C void _atl_obstools_finished ( int );
#else
_ATL_EXTERN_C void _atl_obstools_finished ( );
#endif

/*******************************************************************************
*  Service: _atl_obstools_finished
*******************************************************************************/
_ATL_C_GLOBALFCT void _atl_obstools_finished
#if !RTRT_KR
( int reset )
#else
( reset )
  int reset;
#endif
{
#ifdef ATL_OBSTOOLS_FINISHED
  ATL_OBSTOOLS_FINISHED
#endif
  if(reset & _ATCPQ_CLOSE)
  {
#if RTRT_IO == RTRT_NONE
#ifdef NEEDS_atl_stop
          atl_stop=1;
#endif  /* NEEDS_atl_stop */
	  priv_close(0);
#else
	  priv_close(atl_out);
	  atl_out=0;
#endif
  }
}

#if USE_ATC || USE_ATP || USE_ATQ || USE_ATT

#if USE_ATC
#if !RTRT_KR
_ATL_EXTERN_C void _atc_dump ( int );
#else
_ATL_EXTERN_C void _atc_dump ( );
#endif
#endif /* USE_ATC */

#if USE_ATP
#if !RTRT_KR
_ATL_EXTERN_C int _atp_dump ( int );
#else
_ATL_EXTERN_C int _atp_dump ( );
#endif
#endif /* USE_ATP */

#if USE_ATQ
#if !RTRT_KR
_ATL_EXTERN_C void _atq_dump ( int );
#else
_ATL_EXTERN_C void _atq_dump ( );
#endif
#endif /* USE_ATQ */

#ifdef ATL_T_FAMILY
#if USE_ATC
#if !RTRT_KR
_ATL_EXTERN_C void _atc_remove_family ( ATL_T_FAMILY );
#else
_ATL_EXTERN_C void _atc_remove_family ( );
#endif
#endif /* USE_ATC */

#if USE_ATP
#if !RTRT_KR
_ATL_EXTERN_C void _atp_remove_family ( ATL_T_FAMILY );
#else
_ATL_EXTERN_C void _atp_remove_family ( );
#endif
#endif /* USE_ATP */

#if USE_ATQ
#if !RTRT_KR
_ATL_EXTERN_C void _atq_remove_family ( ATL_T_FAMILY );
#else
_ATL_EXTERN_C void _atq_remove_family ( );
#endif
#endif /* USE_ATQ */

#endif /* ATL_T_FAMILY */

/*******************************************************************************
*  Service: _atl_obstools_dump
*******************************************************************************/
_ATL_C_GLOBALFCT void _atl_obstools_dump
#if !RTRT_KR
( int reset )
#else
( reset )
  int reset;
#endif
{
#if USE_ATP
  char * depth; /* must be pointer for alignment */
#endif /* USE_ATP */
#if !(USE_ATU || USE_ATV || USE_ATO)
#if ATL_OBSTOOLS_DUMP_FREQ
  if(++obstools_dump_freq_count < ATL_OBSTOOLS_DUMP_FREQ) return;
  obstools_dump_freq_count=0;
#endif /*ATL_OBSTOOLS_DUMP_FREQ*/
  /* reset value of 0 or 1 means _ATCPQ_ALL */
  if(reset <= _ATCPQ_CLOSE ) reset += _ATCPQ_ALL;
#endif /* USE_ATC || USE_ATP || USE_ATQ */
  atl_early_init(); /* init if it was not */
#if USE_ATC
  _atc_dump ( reset );
#endif /* USE_ATC */
#if USE_ATQ
  _atq_dump ( reset );
#endif /* USE_ATQ */
#if USE_ATP /* dump ATP in last in case of atp make an access violation */
  atl_stack_end = (char *)&depth; /* memorise the top of the stack avoiding to check stack used by TP */
  (void)_atp_dump ( reset );
#endif /* USE_ATP */
#if !(USE_ATU || USE_ATV || USE_ATO)
/* when RTA standalone */
  if(reset & _ATCPQ_CLOSE )
  {
	  /* mark the end of the test */
	  ATL_LINE_ADD_OPCODE("tdp end");
	  atl_line_dump();
  }
  /* Dump the last lines remaining in the multi-line buffer */
  atl_buffer_dump ( );
  /* And say it's done */
  _atl_obstools_finished ( reset );
#endif
}

#if defined(ATL_SIGNAL) && ATL_SIGNAL_DUMP
/*******************************************************************************
*  Service: _atc_signal_receipt
*  Goal: called each time the process catch an environment defined signal
*******************************************************************************/
RTRT_STATICFCT void _atl_signal_receipt
#if !RTRT_KR
( int sig )
#else
( sig )
  int sig;
#endif
{
  _atl_obstools_dump ( _atl_catch_reset );

  /* Re-arm signal-catching function */
  ATL_SIGNAL ( sig, _atl_signal_receipt );
}

/*******************************************************************************
*  Service: _atc_register_signal
*  Goal: called once to register a signal catch handler to dump traces
*******************************************************************************/
RTRT_STATICFCT void _atl_register_signal
#if !RTRT_KR
( void )
#else
( )
#endif
{
  int signal_to_catch;

  _atl_catch_reset=0;
#ifndef ATL_GETENV
  signal_to_catch = SIGUSR1;
#else
  {
    char *value;

    signal_to_catch = 0;
    if ( ( value = ATL_GETENV ( "ATL_SIGNAL_DUMP" ) ) )
    {
      char *p;

      p = value;
      while ( *p >= '0' && *p <= '9' )
      {
        signal_to_catch *= 10;
        signal_to_catch += *p - '0';
        p++;
      }
      if ( *p == ':' )
      {
        p++;
		  while ( *p >= '0' && *p <= '9' )
		  {
			_atl_catch_reset *= 10;
			_atl_catch_reset += *p - '0';
			p++;
		  }
      }
    }
    if ( signal_to_catch <= 0 )
    {
      signal_to_catch = SIGUSR1;
    }
  }
#endif /* ATL_GETENV */
  if ( ATL_SIGNAL ( signal_to_catch, _atl_signal_receipt ) == SIG_ERR )
  {
    ATL_ERROR ( 0x28,"TestRT: Can't catch signal" );
  }
}

#endif /*  (ATL_SIGNAL) && ATL_SIGNAL_DUMP */

/*******************************************************************************
*  Service: _atl_remove_family remove data for a source Family
*******************************************************************************/
#ifdef ATL_T_FAMILY
_ATL_C_GLOBALFCT void _atl_remove_family
#if !RTRT_KR
(ATL_T_FAMILY family)
#else
(family)
ATL_T_FAMILY family;
#endif
{
#if USE_ATC
  _atc_remove_family ( family );
#endif /* USE_ATC */
#if USE_ATP
  _atp_remove_family ( family );
#endif /* USE_ATP */
#if USE_ATQ
  _atq_remove_family ( family );
#endif /* USE_ATQ */
#ifdef OTHER_REMOVE
OTHER_REMOVE( family );
#endif
}
#endif /* ATL_T_FAMILY */
#endif /* USE_ATC || USE_ATP || USE_ATQ || USE_ATT */

#if ! ( ( USE_ATC && ATC_DUMP_DRIVER ) || ( USE_ATQ && ATQ_DUMP_DRIVER ) )

#if USE_ATC && (USE_ATV || USE_ATU)
/*******************************************************************************
*  Service: _atl_obstools_test_suffix
*******************************************************************************/
RTRT_STATICFCT void _atl_obstools_test_suffix
#if !RTRT_KR
( void )
#else
( )
#endif
{
	int ref=-1;
    if ( _atu_num_current_test != (unsigned int)ref )
    {
      atl_line_add_char ( ' ' );
      atl_line_inc ( unsigned_reverse_image_16 ( (RTRT_U_INT32)_atu_num_current_test,
                                                 atl_line_cur ) );
    }

    if ( _atu_num_current_op != _atc_atu_num_last_op )
    {
      atl_line_add_char ( ' ' );
      atl_line_inc ( unsigned_reverse_image_16 ( (RTRT_U_INT32)_atu_num_current_op,
                                                 atl_line_cur ) );
      _atc_atu_num_last_op = _atu_num_current_op;
    }
}
#endif /* USE_ATC && USE_ATU */
/*******************************************************************************
*  Service: _atl_obstools_test_name
*******************************************************************************/
RTRT_STATICFCT void _atl_obstools_test_name
#if !RTRT_KR
( void )
#else
( )
#endif
{
#if USE_ATU || USE_ATV

    atl_line_add_char ( ' ' );
    atl_line_add_char ( RTRT_QUOT );
#if !RTRT_TINY_TARGET
  if ( _atu_name_current_op != (char *)0 )
  {
    atl_buffer_add ( _atu_name_current_op );
  }

  if ( _atu_name_current_test != (char *)0 )
  {
      atl_line_add_char ( '/' );
    atl_buffer_add ( _atu_name_current_test );
  }
#endif /* ATL_TINY_TARGET */
    atl_line_add_char ( RTRT_QUOT );

#elif USE_ATO

#if USE_ATC || USE_ATP || USE_ATQ
  if ( _ato_obstools_block_name != (char *)0 )
  {
    atl_line_add_char ( ' ' );
    atl_line_add_char ( RTRT_QUOT );
    atl_buffer_add ( _ato_obstools_block_name );
    atl_line_add_char ( RTRT_QUOT );
  }
#endif /* USE_ATC || USE_ATP || USE_ATQ */

#else /* Standalone */

#ifdef RTRT_TEST_NAME
    _atl_obstools_str_add_char ( ' ' );
    _atl_obstools_str_add_char ( RTRT_QUOT );
    _atl_obstools_str_add ( RTRT_TEST_NAME );
    _atl_obstools_str_add_char ( RTRT_QUOT );
#endif /* RTRT_TEST_NAME */

#ifdef ATL_GETENV
  char *current_test_name;

  if ( ( current_test_name = ATL_GETENV ( "ATL_TEST_NAME" ) )
       != (char *)0 )
  {
    atl_line_add_char ( ' ' );
    atl_line_add_char ( RTRT_QUOT );
    atl_buffer_add ( current_test_name );
    atl_line_add_char ( RTRT_QUOT );
  }
  else
#endif /* ATL_GETENV */
  if(atl_test_name != (char *)0 )
  {
    atl_line_add_char ( ' ' );
    atl_line_add_char ( RTRT_QUOT );
    atl_buffer_add ( atl_test_name );
    atl_line_add_char ( RTRT_QUOT );
  }
#endif /* USE_ATU || USE_ATV*/
}

#else /* ( USE_ATC && ATC_DUMP_DRIVER ) || ( USE_ATQ && ATQ_DUMP_DRIVER ) */

/*******************************************************************************
*  Service: _atl_obstools_str_add
*******************************************************************************/
RTRT_STATICFCT void _atl_obstools_str_add
#if !RTRT_KR
( RTRT_CONST_STRING_DECL str )
#else
( str )
  char *str;
#endif
{
  while (*str)
  {
    *_atl_obstools_str_cur++ = *str++;
  }
  *_atl_obstools_str_cur = (char)0;
}

#define _atl_obstools_str_add_char(c) \
  ( ( *_atl_obstools_str_cur++ = (c) ), \
    ( *_atl_obstools_str_cur = (char)0 ) )

#define _atl_obstools_str_inc(len) \
  ( _atl_obstools_str_cur += (len) )

#if USE_ATC && (USE_ATV || USE_ATU)
/*******************************************************************************
*  Service: _atl_obstools_test_suffix
*******************************************************************************/
RTRT_STATICFCT void _atl_obstools_test_suffix
#if !RTRT_KR
( void )
#else
( )
#endif
{
    if ( _atu_num_current_test != -1 )
    {
      _atl_obstools_str_add_char ( ' ' );
      _atl_obstools_str_inc ( unsigned_image_16 (
                               _atu_num_current_test,
                               atl_line_cur ) );
    }

    if ( _atu_num_current_op != _atc_atu_num_last_op )
    {
      _atl_obstools_str_add_char ( ' ' );
      _atl_obstools_str_inc ( unsigned_image_16 (
                               _atu_num_current_op,
                               atl_line_cur ) );
      _atc_atu_num_last_op = _atu_num_current_op;
    }
}
#endif /* USE_ATC && USE_ATU */

/*******************************************************************************
*  Service: _atl_obstools_test_name   used by DUMP_DRIVER
*******************************************************************************/
_ATL_C_GLOBALFCT void _atl_obstools_test_name
#if USE_ATU && 0
#if !RTRT_KR
( int suffix )
#else
( suffix )
  int suffix;
#endif
#else /* USE_ATU */
#if !RTRT_KR
( void )
#else
( )
#endif
#endif /* USE_ATU */
{
#if USE_ATU || USE_ATV

    _atl_obstools_str_add_char ( ' ' );
    _atl_obstools_str_add_char ( RTRT_QUOT );
#if !RTRT_TINY_TARGET
  if ( _atu_name_current_op != (char *)0 )
  {
    _atl_obstools_str_inc ( string_image ( _atu_name_current_op,
                                          _atl_obstools_str_cur,
                                          48 ) );
  }

  if ( _atu_name_current_test != (char *)0 )
  {
      _atl_obstools_str_add_char ( '/' );
    _atl_obstools_str_inc ( string_image ( _atu_name_current_test,
                                          _atl_obstools_str_cur,
                                          48 ) );
  }
#endif
    _atl_obstools_str_add_char ( RTRT_QUOT );

#elif USE_ATO

#if USE_ATC || USE_ATP || USE_ATQ
  if ( _ato_obstools_block_name != (char *)0 )
  {
    _atl_obstools_str_add_char ( ' ' );
    _atl_obstools_str_add_char ( RTRT_QUOT );
    _atl_obstools_str_inc ( string_image ( _ato_obstools_block_name,
                                           _atl_obstools_str_cur,
                                           48 ) );
    _atl_obstools_str_add_char ( RTRT_QUOT );
  }
#endif /* USE_ATC || USE_ATP || USE_ATQ */

#else /* Standalone */

#ifdef ATL_GETENV
  char *current_test_name;

  if ( ( current_test_name = ATL_GETENV ( "ATL_TEST_NAME" ) )
       != (char *)0 )
  {
    atl_test_name=current_test_name ;
  }
#endif /* ATL_GETENV */
  if(atl_test_name != (char *)0 )
  {
    _atl_obstools_str_add_char ( ' ' );
    _atl_obstools_str_add_char ( RTRT_QUOT );
    _atl_obstools_str_add ( atl_test_name );
    _atl_obstools_str_add_char ( RTRT_QUOT );
  }
#endif /* USE_ATU || USE_ATV */
}

#endif /* ( USE_ATC && ATC_DUMP_DRIVER ) || ( USE_ATQ && ATQ_DUMP_DRIVER ) */

