

/*
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */


/*******************************************************************************
* $Descr: Observation Tools common functions Header $
*******************************************************************************/


/* Multiple inclusions guard: */
#ifndef OBSTOOLS_H_INCLUDED
#define OBSTOOLS_H_INCLUDED

#if !RTRT_VARIABLE_INIT_SUPPORTED

/* Guarded because of Test need (should be defined to = 0 in test mode) */
#ifndef ATL_OBSTOOLS_INITIALIZE_TO_ZERO
#define ATL_OBSTOOLS_INITIALIZE_TO_ZERO
#endif /* ATL_OBSTOOLS_INITIALIZE_TO_ZERO */

#else /* RTRT_VARIABLE_INIT_SUPPORTED */

#define ATL_OBSTOOLS_INITIALIZE_TO_ZERO = 0

#endif /* RTRT_VARIABLE_INIT_SUPPORTED */

#if USE_ATC || USE_ATP || USE_ATQ || USE_ATT

/* what to do by default when Coverage/Memory and Performance Profiling dump
 * has been called using -dump... instrumentor options
 * use -D_ATCPQ_RESET=x to change the value
 * or '|' can be used to define what to do
 *
 * 0: as 1
 * 1: dump all then reset tables (default)
 * 2: close the file at the end of the dump
 * 4: coverage Dump without reset
 * 8: coverage reset tables
 * 16:quantify dump
 * 32:quantify reset table
 * 64:free the freeQ durring dump/check
 *128:Purify complete dump
 *256:Purify ABWL & FMWL check only
 */
#define _ATCPQ_CLOSE 		2
#define _ATCPQ_COV			4
#define _ATCPQ_RESET_COV	8
#define _ATCPQ_QTF			16
#define _ATCPQ_RESET_QTF	32
#define _ATCPQ_FREE_FRQ		64
#define _ATCPQ_PRF			128
#define _ATCPQ_CHK_WL		256
#define _ATCPQ_ALL			508 /* 0b0111111100 */
#ifndef _ATCPQ_RESET
#define _ATCPQ_RESET _ATCPQ_ALL

#if !USE_ATU && !USE_ATV && !USE_ATO
_ATL_EXTERN_C char *atl_test_name;
#endif
#endif

/* The reset parameter value of 0 is deprecated: 1 is assumed */
#if !RTRT_KR
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX void _atl_obstools_dump ( int reset );
#ifdef ATL_T_FAMILY
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX void _atl_remove_family (ATL_T_FAMILY family);
#endif /* ATL_T_FAMILY */
#else
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX void _atl_obstools_dump ( );
#ifdef ATL_T_FAMILY
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX void _atl_remove_family ( );
#endif /* ATL_T_FAMILY */
#endif

#ifdef ATL_T_FAMILY
#define _REMOVE_DYNAMIC_LIB() _atl_remove_family (ATL_FAMILY_INIT_VALUE);
#else
#define _REMOVE_DYNAMIC_LIB()
#endif
#endif /* USE_ATC || USE_ATP || USE_ATQ || USE_ATT */

/*******************************************************************************
* User Directives:
*******************************************************************************/


/* how to dump Coverage, PurifyLT and QuantifyLT */
#if USE_ATC || USE_ATP || USE_ATQ || USE_ATT
#define _ATCPQ_DUMP(r) _atl_obstools_dump(r)
#define _ATCPQ_DUMP_NAME(n) atl_test_name=n
/* no ; for -DUMPCALLING */
#else /* USE_ATC || USE_ATP || USE_ATQ || USE_ATT */
#define _ATCPQ_DUMP(r)
#define _ATCPQ_DUMP_NAME(n)
#endif /* USE_ATC || USE_ATP || USE_ATQ || USE_ATT */

/* how to check for ABWL and FMWL */
#if USE_ATP
#if !RTRT_KR
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX void _atp_insert_chk( void * );
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX int  _atp_chk_blocks( void );
#else
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX void _atp_insert_chk( );
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX int  _atp_chk_blocks( );
#endif

#define _ATP_CHK    (void)_atp_chk_blocks();
#define _ATP_TRACK(p) _atp_insert_chk(p);
#define _ATP_CHECK(o) _ATP_LINE_OFFSET_EXPR_START(0,o);_ATP_CHK
#else /* USE_ATP */
#define _ATP_CHK
#define _ATP_CHECK(l)
#define _ATP_TRACK(p)
#endif /* USE_ATP */

#ifdef RTRT_TDP_COMPILATION
#if !RTRT_KR
RTRT_STATICFCT void _atl_obstools_test_name ( void );
#if USE_ATC && (USE_ATV || USE_ATU)
RTRT_STATICFCT void _atl_obstools_test_suffix ( void );
#endif
#else
RTRT_STATICFCT void _atl_obstools_test_name ();
#if USE_ATC && (USE_ATV || USE_ATU)
RTRT_STATICFCT void _atl_obstools_test_suffix ();
#endif
#endif

#if USE_ATC && (USE_ATV || USE_ATU)
#define ATL_OBSTOOLS_TEST_SUFFIX _atl_obstools_test_suffix();
#else
#define ATL_OBSTOOLS_TEST_SUFFIX
#endif

#if ( ( USE_ATC && ATC_DUMP_DRIVER ) || ( USE_ATQ && ATQ_DUMP_DRIVER ) )
RTRT_GLOBALVAR char *_atl_obstools_str_cur;

/* Advised size for the user buffer: 128 */
#define _atl_obstools_str_init(str) \
  { _atl_obstools_str_cur = str; \
    *_atl_obstools_str_cur = (char)0; }

RTRT_STATICFCT void _atl_obstools_str_add
#if !RTRT_KR
( RTRT_CONST_STRING_DECL str )
#else
( str )
  char *str;
#endif
;

#endif /* ( USE_ATC && ATC_DUMP_DRIVER ) || ( USE_ATQ && ATQ_DUMP_DRIVER ) */

#endif /* RTRT_TDP_COMPILATION */


#endif /* OBSTOOLS_H_INCLUDED */

