

/*
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2011 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */


/*******************************************************************************
* $Descr: I/O and Systems routines header (included by TP.h) $
*******************************************************************************/
/* Multiple inclusions guard: */
#ifndef PRIV_H_INCLUDED
#define PRIV_H_INCLUDED

#if USE_ATK
#define NEEDS_MT_REC_MUTEX
#endif

#if defined(RTRT_TDP_COMPILATION) || \
  (USE_ATS && \
   !defined(ATL_CPLUSPLUS_INSTRUMENTOR) && \
   !defined(ATL_C_INSTRUMENTOR))

#if USE_ATS
/*! Needs priv_flush() */
#define NEEDS_priv_flush()
#endif

#if USE_ATS
/*! Needs priv_sleep() */
#define NEEDS_priv_sleep
#endif

#if USE_ATS
/*! Needs priv_realloc */
#define NEEDS_priv_realloc
#endif

#if USE_ATS
/*! Needs priv_*_tagtlk() */
#define NEEDS_tagtlk
#endif

#if RTRT_IO == RTRT_STD
#include <stdio.h>
#include <stdlib.h>
/*! Type handled by I/O routines */
#define RTRT_FILE_TYPE FILE *
#elif RTRT_IO == RTRT_SOCKET
#ifndef RTRT_SOCKET_INCLUDES
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <signal.h>
#include <sys/types.h>
#include <netdb.h>
#endif /* RTRT_SOCKET_INCLUDES */
#elif RTRT_IO == RTRT_NONE
#define RTRT_FILE_TYPE char
#endif /* RTRT_IO */

_ATL_EXTERN_C RTRT_FILE_TYPE priv_open
#if !RTRT_KR
  (char *)
#else
  ()
#endif
  ;

_ATL_EXTERN_C RTRT_FILE_TYPE priv_init
#if !RTRT_KR
  (char * name)
#else
  ()
#endif
  ;

_ATL_EXTERN_C void priv_close
#if !RTRT_KR
  (RTRT_FILE_TYPE file)
#else
  ()
#endif
  ;

#ifdef NEEDS_priv_flush
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX void priv_flush
#if !RTRT_KR
  (RTRT_FILE_TYPE file)
#else
  ()
#endif
  ;
#endif /* NEEDS_priv_flush */

_ATL_EXTERN_C void priv_writeln
#if !RTRT_KR
  (RTRT_FILE_TYPE, char *)
#else
  ()
#endif
  ;

_ATL_EXTERN_C ATL_TP_PROTO_PREFIX void priv_check_failed
#if !RTRT_KR
  (void)
#else
  ()
#endif
  ;

_ATL_EXTERN_C ATL_TP_PROTO_PREFIX int priv_rdv
#if !RTRT_KR
  (char *)
#else
  ()
#endif
  ;

#ifdef NEEDS_tagtlk

#if RTRT_DYNAMIC_TRACKING
/*! Type used for communications between Virtual Testers and Agent */
#define ATL_TSTAGT_COMMTYPE int
#endif /* RTRT_DYNAMIC_TRACKING */

_ATL_EXTERN_C int priv_boot_tagtlk
#if !RTRT_KR
  (ATL_TSTAGT_COMMTYPE *, char *, char *, int *, int *)
#else
  ()
#endif
  ;

_ATL_EXTERN_C void priv_close_tagtlk
#if !RTRT_KR
  (ATL_TSTAGT_COMMTYPE *)
#else
  ()
#endif
  ;

_ATL_EXTERN_C int priv_raiseline_tagtlk
#if !RTRT_KR
  (ATL_TSTAGT_COMMTYPE *, char *)
#else
  ()
#endif
  ;

_ATL_EXTERN_C ATL_TP_PROTO_PREFIX int priv_interrecv
#if !RTRT_KR
  (ATL_TSTAGT_COMMTYPE *, int, char *, int, char **, char **)
#else
  ()
#endif
  ;

_ATL_EXTERN_C ATL_TP_PROTO_PREFIX int priv_send_chk
#if !RTRT_KR
  (ATL_TSTAGT_COMMTYPE *, char *)
#else
  ()
#endif
  ;

#endif /* NEEDS_tagtlk */

#ifdef NEEDS_priv_sleep
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX void priv_sleep
#if !RTRT_KR
  (RTRT_TIME_TYPE)
#else
  ()
#endif
  ;
#endif

_ATL_EXTERN_C void priv_exit
#if !RTRT_KR
  (RTRT_INT32 ec)
#else
  ()
#endif
  ;

#ifdef NEEDS_priv_realloc
_ATL_EXTERN_C void *priv_realloc
#if !RTRT_KR
  (void *p,
   RTRT_SIZE_T new_size)
#else
  ()
#endif
  ;
#endif

#endif /* RTRT_TDP_COMPILATION ... */

#ifdef NEEDS_priv_free
_ATL_EXTERN_C void priv_free
#if !RTRT_KR
  (void *p)
#else
  ()
#endif
  ;
#endif

#ifdef RTRT_HEAP_NEEDS
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX void *priv_malloc
#if !RTRT_KR
  (RTRT_SIZE_T size)
#else
  ()
#endif
  ;
#endif


#ifdef RTRT_MEMCPY_NEEDS
_ATL_EXTERN_C void *priv_memcpy
#if !RTRT_KR
  (void *, void *, unsigned int)
#else
  ()
#endif
  ;
#endif


_ATL_EXTERN_C void _atl_error_dump
#if !RTRT_KR
( const char error )
#else
()
#endif
 ;

#endif
