

/*
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2011 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */


/*******************************************************************************
* $Descr: TDF support library (included by TP.c[pp]) $
*******************************************************************************/

#if (ATL_MULTI_THREADS == ATL_YES) && (USE_ATK || USE_ATP || USE_ATQ || USE_ATT)
#define NEEDS_instances_table
#endif

/*! Highest source dynamic number allocated (threads shared) */
RTRT_STATICVAR RTRT_U_INT32 _atl_highest_source;

/*! Current source dynamic number (threads shared) */
RTRT_STATICVAR RTRT_U_INT32 _atl_current_source;

#if USE_ATO || USE_ATK
/*! Last dynamic instance number allocated (ATO/ATK only) */
RTRT_STATICVAR RTRT_U_INT32 _atl_highest_instance;
#endif

/*! Last MC number generated */
RTRT_STATICVAR RTRT_U_INT32 _atl_last_mc_id;

#ifndef NEEDS_instances_table

/*! Current instance (single-thread mode) */
RTRT_GLOBALVAR RTRT_U_INT32 _atl_current_instance;

#else

/*! Current instance table (one per thread) */
RTRT_GLOBALVAR RTRT_U_INT32 _atl_current_instance_table[ATL_THREADS_MAX+1];

/*! Current instance of the current thread */
#define _atl_current_instance \
   _atl_current_instance_table[thread_index]

#endif /* NEEDS_instances_table */

/*! Mutex handling access to TDF resources */
RTRT_MT_CRITICAL_VAR_DECL(atl_tdf_mutex)

#if USE_ATO || USE_ATK

/*! Offsets added to Object Testing IDs to get TSF IDs */
#define _ATL_TDF_INSTANCE_OFFSET 0
#define _ATL_TDF_MESSAGE_OFFSET 1000000
#define _ATL_TDF_TEXT_OFFSET 2000000

#endif

#if ATT_ON_THE_FLY
/*! Flush dump needed */
RTRT_STATICFCT void _atl_tdf_line_dump
#if !RTRT_KR
( void )
#else
( )
#endif
{
  atl_line_add_char (RTRT_NEWLINE_CHAR);
  /* Each line is immediatly dumped */
  atl_buffer_dump ();
}
#else
/*! Big buffer used (implies less calls to upload mechanism) */
#define _atl_tdf_line_dump atl_line_dump
#endif /* ATT_ON_THE_FLY */

/*******************************************************************************
* TDF Styles
*******************************************************************************/

#define _ATO_STYLE_CHECK_OK             "ST 150"
#define _ATO_STYLE_CHECK_KO             "ST 151"
#define _ATO_STYLE_CHECK_PROPERTY_OK    "ST 152"
#define _ATO_STYLE_CHECK_PROPERTY_KO    "ST 153"
#define _ATO_STYLE_CHECK_EXCEPTION_OK   "ST 154"
#define _ATO_STYLE_CHECK_EXCEPTION_KO   "ST 155"
#define _ATO_STYLE_CHECK_STUB_OK        "ST 156"
#define _ATO_STYLE_CHECK_STUB_KO        "ST 157"
#define _ATO_STYLE_CHECK_METHOD_OK      "ST 158"
#define _ATO_STYLE_CHECK_METHOD_KO      "ST 159"
#define _ATO_STYLE_PREPOS_OK            "ST 160"
#define _ATO_STYLE_PREPOS_KO            "ST 161"
#define _ATO_STYLE_UNEXPECTED_EXCEPTION "ST 162"
#define _ATO_STYLE_EXCEPTION_THROWN     "ST 163"
/*      _ATO_STYLE_ON_ERROR             "ST 164" */
#define _ATO_STYLE_COMMENT              "ST 165"
#define _ATO_STYLE_PRINT                "ST 166"
#define _ATO_STYLE_GLYPH_OK             "GL 150"
#define _ATO_STYLE_GLYPH_KO             "GL 151"

/*! Initialize TDF support */
RTRT_STATICFCT void _atl_tdf_init
#if !RTRT_KR
( void )
#else
( )
#endif
{
#if ! RTRT_RAM_CLEAN

#ifdef NEEDS_instances_table
  int i;
#endif

  _atl_highest_source = 0;
  _atl_current_source = 0;
#if USE_ATO || USE_ATK
  _atl_highest_instance = 0;
#endif
  _atl_last_mc_id = 0;
#ifndef NEEDS_instances_table
  _atl_current_instance = 0;
#else
  for (i = 0; i < ATL_THREADS_MAX; i++)
  {
    _atl_current_instance_table[i] = 0;
  }
#endif
#endif

  RTRT_MT_CRITICAL_VAR_INIT(atl_tdf_mutex)
}

/*! Hack to generate the right hex number format */
RTRT_STATICFCT int _atl_tdf_id_image (RTRT_U_INT32 n, char *w)
{
   *w++=' ';
   if(n<10)
   {
	   *w=(char)(n+'0');
	   return 2;
   }
   *w++='h';
   return (unsigned_reverse_image_16 (n, w)+2);
}

#if USE_ATK || USE_ATO || (USE_ATT && (ATT_TIMESTAMP || USE_ATU || USE_ATV || (USE_ATP && ATT_ON_THE_FLY) || (USE_ATT && USE_ATP && ATP_STACK_SIZE_LEVEL)))
RTRT_STATICFCT void _atl_tdf_line(RTRT_CONST_STRING_DECL s)
{
	   RTRT_MT_CRITICAL_VAR_LOCK(atl_line_mutex)
	   atl_line_add (s);
	   _atl_tdf_line_dump ();
	   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_line_mutex)
}
#endif

#if USE_ATK || USE_ATO || USE_ATT || USE_ATU || USE_ATV
RTRT_STATICFCT void _atl_tdf_line_1(RTRT_CONST_STRING_DECL op, RTRT_U_INT32 n1)
{
	   RTRT_MT_CRITICAL_VAR_LOCK(atl_line_mutex)
	   atl_line_add (op);
	   atl_line_inc (_atl_tdf_id_image (n1, atl_line_cur));
	   _atl_tdf_line_dump ();
	   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_line_mutex)
}
#endif

#if 0
RTRT_STATICFCT void _atl_tdf_line_1s(RTRT_CONST_STRING_DECL op, RTRT_U_INT32 n1,RTRT_CONST_STRING_DECL s)
{
	   RTRT_MT_CRITICAL_VAR_LOCK(atl_line_mutex)
	   atl_line_add (op);
	   atl_line_inc (_atl_tdf_id_image (n1, atl_line_cur));
	   atl_line_add_char ( ' ' );
	   atl_line_add_char ( RTRT_QUOT );
	   atl_line_add (s);
	   atl_line_add_char ( RTRT_QUOT );
	   _atl_tdf_line_dump ();
	   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_line_mutex)
}
#endif

#if USE_ATK || USE_ATO || USE_ATT || USE_ATU || USE_ATV
RTRT_STATICFCT void _atl_tdf_line_2(RTRT_CONST_STRING_DECL op, RTRT_U_INT32 n1,RTRT_U_INT32 n2)
{
	   RTRT_MT_CRITICAL_VAR_LOCK(atl_line_mutex)
	   atl_line_add (op);
	   atl_line_inc (_atl_tdf_id_image (n1, atl_line_cur));
	   atl_line_inc (_atl_tdf_id_image (n2, atl_line_cur));
	   _atl_tdf_line_dump ();
	   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_line_mutex)
}
#endif

#if USE_ATT || USE_ATO
RTRT_STATICFCT void _atl_tdf_line_3(RTRT_CONST_STRING_DECL op, RTRT_U_INT32 n1,RTRT_U_INT32 n2,RTRT_U_INT32 n3)
{
	   RTRT_MT_CRITICAL_VAR_LOCK(atl_line_mutex)
	   atl_line_add (op);
	   atl_line_inc (_atl_tdf_id_image (n1, atl_line_cur));
	   atl_line_inc (_atl_tdf_id_image (n2, atl_line_cur));
	   atl_line_inc (_atl_tdf_id_image (n3, atl_line_cur));
	   _atl_tdf_line_dump ();
	   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_line_mutex)
}
#endif

/*! Prints a "SF" line */
#define _atl_print_source_file(s,k,c) _atl_tdf_line_3(RTRT_CONST_STRING(ATL_TDF_SF),s,k,c);_atl_current_source = s
/*
RTRT_STATICFCT void _atl_print_source_file
#if !RTRT_KR
( RTRT_U_INT32 source_number,
  RTRT_U_INT32 key,
  RTRT_U_INT32 checksum )
#else
( source_number, key, checksum )
  RTRT_U_INT32 source_number;
  RTRT_U_INT32 key;
  RTRT_U_INT32 checksum;
#endif
{
  RTRT_MT_CRITICAL_VAR_LOCK(atl_line_mutex)
  ATL_LINE_ADD_OPCODE_SPACE(RTRT_CONST_STRING(ATL_TDF_SF));
  atl_line_inc (unsigned_image_16 (source_number, atl_line_cur));
  atl_line_add ( "h " );
  atl_line_inc (unsigned_image_16 (key, atl_line_cur));
  atl_line_add ( "h " );
  atl_line_inc (unsigned_image_16 (checksum, atl_line_cur));
  atl_line_add_char ( 'h' );
  _atl_tdf_line_dump ();
  _atl_current_source = source_number;
  RTRT_MT_CRITICAL_VAR_UNLOCK(atl_line_mutex)
}
*/
/*! Prints a "CS" line */
RTRT_STATICFCT void _atl_print_change_source
#if !RTRT_KR
( RTRT_U_INT32 source_number )
#else
( source_number )
  RTRT_U_INT32 source_number;
#endif
{
  if ( source_number != _atl_current_source )
  {
    _atl_tdf_line_1(RTRT_CONST_STRING(ATL_TDF_CF),source_number);
    _atl_current_source = source_number;
  }
}

#if USE_ATO || USE_ATP || USE_ATT
/*! Hack to generate the string note in locale encoding */
RTRT_STATICFCT void _atl_tdf_note_string (RTRT_CONST_STRING_DECL w)
{
  RTRT_MT_CRITICAL_VAR_LOCK(atl_line_mutex)
  ATL_LINE_ADD_OPCODE_SPACE ( ATL_TDF_NO );
  atl_line_add_char ( RTRT_QUOT );
  atl_line_add (ATL_MIME_NAME );
  atl_line_add (RTRT_CONST_STRING( "\" \"" ));
  atl_buffer_note ( w );
  atl_line_add_char ( RTRT_QUOT );
  _atl_tdf_line_dump ( );
  RTRT_MT_CRITICAL_VAR_UNLOCK(atl_line_mutex)
}
#endif

#if 0
/*! Hack to generate the string note in locale encoding */
RTRT_STATICFCT void _atl_tdf_note_string_3 (RTRT_CONST_STRING_DECL w, RTRT_CONST_STRING_DECL x, RTRT_CONST_STRING_DECL y)
{
  RTRT_MT_CRITICAL_VAR_LOCK(atl_line_mutex)
  ATL_LINE_ADD_OPCODE_SPACE ( ATL_TDF_NO );
  atl_line_add_char ( RTRT_QUOT );
  atl_line_add (ATL_MIME_NAME );
  atl_line_add (RTRT_CONST_STRING("\" \"" ));
  atl_buffer_note ( w );
  atl_buffer_note ( x );
  atl_buffer_note ( y );
  atl_line_add_char ( RTRT_QUOT );
  _atl_tdf_line_dump ( );
  RTRT_MT_CRITICAL_VAR_UNLOCK(atl_line_mutex)
}
#endif

#if USE_ATT && ATT_ON_THE_FLY || USE_ATP
RTRT_STATICFCT void _atl_tdf_err_note(RTRT_CONST_STRING_DECL s)
{
	  _atl_tdf_line(_ATO_STYLE_CHECK_KO);
	  _atl_tdf_note_string(s);
	  _atl_tdf_line( ATL_TDF_CI " 0" );
}
#endif /* USE_ATT && ATT_ON_THE_FLY && USE_ATP */
